
#ifndef MGA_COMMONSTL_H
#define MGA_COMMONSTL_H

#include <vector>
#include <string>

#ifndef MGA_COMMONSMART_H
#include "CommonSmart.h"
#endif

// --------------------------- string

// we use "const string &" for [in] and "string &" for [in, out]

// these THROW exceptions

inline void CopyTo(const string &s, BSTR *b) { CopyTo(s.begin(), s.length(), b); }
inline void CopyTo(const string &s, VARIANT *v) { CopyTo(s.begin(), s.length(), v); }
inline void CopyTo(const string &s, CComBstrObj &a) { CopyTo(s.begin(), s.length(), a); }
inline void CopyTo(const string &s, CComVariant &a) { CopyTo(s.begin(), s.length(), a); }

inline void CopyTo(BSTR b, string &s)
{
	s.resize(GetCharLength(b));
	CopyTo(b, s.begin(), s.length());
}

inline void CopyTo(VARIANT &v, string &s)
{
	s.resize(GetCharLength(v));
	CopyTo(v, s.begin(), s.length());
}

void vFormat(string &s, const char *format, va_list args);

inline void Format(string &s, const char *format, ...)
{
	va_list args;
	va_start(args, format);

	vFormat(s, format, args);
}

typedef vector<wchar_t>::iterator bstr_iterator;
typedef vector<wchar_t>::const_iterator bstr_const_iterator;

inline bstr_const_iterator begin(BSTR p) { return p; }
inline bstr_const_iterator end(BSTR p) { return p ? p + SysStringLen(p) : 0; }

inline void CopyTo(bstr_const_iterator i, bstr_const_iterator e, CComBstrObj &b)
{
	ASSERT( i <= e );
	b.Attach(SysAllocStringLen(i, e - i));
}

// --------------------------- bindata

typedef vector<unsigned char> bindata;

inline void CopyTo(const bindata &b, VARIANT *p) { CopyTo(b.begin(), b.end(), p); }
inline void CopyTo(const bindata &b, CComVariant &a) { CopyTo(b.begin(), b.end(), a); }

inline void CopyTo(VARIANT &v, bindata &b)
{
	if( v.vt == (VT_I4 | VT_ARRAY) )
	{
		b.resize(4 * GetArrayLength(v));
		CopyTo(v, (long*)b.begin(), (long*)b.end());
	}
	else
	{
		b.resize(GetArrayLength(v));
		CopyTo(v, b.begin(), b.end());
	}
}

inline void MoveTo(SAFEARRAY *p, vector<CComBstrObj> &bstrobjs)
{
	ASSERT( p != NULL );

	bstrobjs.resize(GetArrayLength(p));
	MoveTo(p, bstrobjs.begin(), bstrobjs.end());
}

inline void MoveTo(vector<CComBstrObj> &bstrobjs, SAFEARRAY **p)
{
	ASSERT( p != NULL && *p == NULL );

	MoveTo(bstrobjs.begin(), bstrobjs.end(), p);
}

inline void CopyTo(const vector<CComBstrObj> &bstrobjs, SAFEARRAY **p)
{
	ASSERT( p != NULL && *p == NULL );

	CopyTo(bstrobjs.begin(), bstrobjs.end(), p);
}

inline void CopyTo(const vector<GUID> &guids, SAFEARRAY **p)
{
	ASSERT( p != NULL && *p == NULL );

	CopyTo(guids.begin(), guids.end(), p);
}

// --------------------------- STL function objects

template<class T>
struct ptr_hashfunc
{
	size_t operator()(const T *ptr) const { return (size_t)ptr; }
};

template<class T>
struct ptr_equalkey
{
	bool operator()(const T *a, const T *b) const { return a == b; }
};

template<class T>
struct ptr_compare
{
	bool operator()(const T *a, const T *b) const { return a < b; }
};

// --------------------------- Iterator

#ifdef _DEBUG

template<class CONTAINER>
inline bool IsValidIterator(const CONTAINER &container, CONTAINER::const_iterator i)
{
	CONTAINER::const_iterator b = container.begin();
	CONTAINER::const_iterator e = container.end();

	while( b != e )
	{
		if( b == i )
			return true;
		++b;
	}

	return false;
}

#endif

template<class CONTAINER>
inline int limited_size(const CONTAINER &container, int limit)
{
	ASSERT( limit >= 1 );
	int counter = limit;

	CONTAINER::const_iterator i = container.begin();
	CONTAINER::const_iterator e = container.end();
	while( i != e && --counter > 0 )
		++i;

	ASSERT( (container.size() < limit ? container.size() : limit) == (limit - counter) );
	return limit - counter;
}

// --------------------------- Collection

template<class INTERFACE>
class CComObjPtrVector : public vector< CComObjPtr<INTERFACE> >
{
};

template<class ELEM>
void CopyTo(TYPENAME_ELEM2COLL(ELEM) *p, vector< CComObjPtr<ELEM> > &q)
{
	ASSERT( p != NULL );
	ASSERT( &q != NULL );

	q.clear();

	long count = 0;
	COMTHROW( p->get_Count(&count) );
	ASSERT( count >= 0 );

	q.insert(q.begin(), count, CComObjPtr<ELEM>());

	COMTHROW( p->GetAll(count, (ELEM**)q.begin()) );
}

template<class ELEM>
class PutOutCollObj
{
public:
	typedef ELEM elem_interface;
	typedef TYPENAME_ELEM2COLL(ELEM) coll_interface;

	typedef vector< CComObjPtr<elem_interface> > vector_type;

	operator coll_interface** () { return PutOut(coll_ptr); }

	~PutOutCollObj()
	{
		// we need this because COM methods might
		// not fill in the [out] parameter

		if( coll_ptr != NULL )
			CopyTo(coll_ptr, coll_vector);
	}

private:
	PutOutCollObj(vector_type &v) : coll_vector(v) { }

	friend PutOutCollObj<ELEM> PutOut(vector< CComObjPtr<ELEM> > &coll_vector);

	// to prevent misuse
	PutOutCollObj();
	PutOutCollObj(const PutOutCollObj<ELEM> &a);

public:
	CComObjPtr<coll_interface> coll_ptr;
	vector_type &coll_vector;
};

template<class ELEM>
PutOutCollObj<ELEM> PutOut(vector< CComObjPtr<ELEM> > &coll_vector)
{
	return PutOutCollObj<ELEM>(coll_vector);
}

#endif//MGA_COMMONSTL_H
