// GMEDoc.cpp : implementation of the CGMEDoc class
//

#include "stdafx.h"

#include "GMEApp.h"
#include "GMEstd.h"

#include "Mainfrm.h"
#include "GMEDoc.h"
#include "GMEView.h"
#include "GuiObject.h"
#include "GMEEventLogger.h"


#include "slist"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CGMEDoc

CGMEDoc* CGMEDoc::theInstance = 0;

IMPLEMENT_DYNCREATE(CGMEDoc, CDocument)

BEGIN_MESSAGE_MAP(CGMEDoc, CDocument)
	//{{AFX_MSG_MAP(CGMEDoc)
	ON_COMMAND(ID_MODE_AUTOCONNECT, OnModeAutoconnect)
	ON_UPDATE_COMMAND_UI(ID_MODE_AUTOCONNECT, OnUpdateModeAutoconnect)
	ON_COMMAND(ID_MODE_DISCONNECT, OnModeDisconnect)
	ON_UPDATE_COMMAND_UI(ID_MODE_DISCONNECT, OnUpdateModeDisconnect)
	ON_COMMAND(ID_MODE_EDIT, OnModeEdit)
	ON_UPDATE_COMMAND_UI(ID_MODE_EDIT, OnUpdateModeEdit)
	ON_COMMAND(ID_MODE_SET, OnModeSet)
	ON_UPDATE_COMMAND_UI(ID_MODE_SET, OnUpdateModeSet)
	ON_COMMAND(ID_MODE_VISUALIZE, OnModeVisualize)
	ON_UPDATE_COMMAND_UI(ID_MODE_VISUALIZE, OnUpdateModeVisualize)
	ON_COMMAND(ID_MODE_ZOOM, OnModeZoom)
	ON_UPDATE_COMMAND_UI(ID_MODE_ZOOM, OnUpdateModeZoom)
	ON_UPDATE_COMMAND_UI(ID_FILE_CLOSE, OnUpdateFileClose)
	ON_COMMAND(ID_VIEW_REFRESH, OnViewRefresh)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGMEDoc construction/destruction

CGMEDoc::CGMEDoc()
{
	theInstance = this;

	editMode = GME_EDIT_MODE;

	modeBar.CreateEx(CMainFrame::theInstance,TBSTYLE_TRANSPARENT | TBSTYLE_FLAT);
	modeBar.LoadToolBar(IDR_TOOLBAR_MODE);
	modeBar.SetBarStyle(modeBar.GetBarStyle() | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
	modeBar.EnableDocking(CBRS_ALIGN_ANY);
	CMainFrame::theInstance->ShowControlBar(&modeBar, true, FALSE);
	CMainFrame::theInstance->DockControlBar(&modeBar,AFX_IDW_DOCKBAR_LEFT);

	try {
	    COMTHROW( resolver.CoCreateInstance(L"Mga.MgaResolver"));
	}
	catch(hresult_exception e) {
		AfxMessageBox("Fatal error: cannot create MGA Resolver component!",MB_OK | MB_ICONSTOP);
	}
}

CGMEDoc::~CGMEDoc()
{
	theInstance = 0;
	if (modeBar.GetSafeHwnd() && (!modeBar.IsWindowVisible())) {
		CMainFrame::theInstance->ShowControlBar(&modeBar, FALSE, FALSE);
	}
}

bool CGMEDoc::CreateFcoList(CGuiObjectList* list,CComPtr<IMgaFCOs> &fcos,CGMEView *view)
{
	try {
		view->BeginTransaction(TRANSACTION_READ_ONLY);
		ASSERT(view);

		CComPtr<IMgaFCOs> coll;
		COMTHROW(coll.CoCreateInstance(OLESTR("Mga.MgaFCOs")));
		POSITION pos = list->GetHeadPosition();
		while(pos) {
			COMTHROW(coll->Append(list->GetNext(pos)->mgaFco));
		}
		COMTHROW(view->currentModel->AddInternalConnections(coll,&fcos));
		view->CommitTransaction();
	}
	catch(hresult_exception e) {
		view->AbortTransaction(e.hr);
		return false;
	}
	return true;
}

bool CGMEDoc::CreateAnnotationList(CGuiAnnotatorList* list,CComPtr<IMgaRegNodes> &anns,CGMEView *view)
{
	try {
		view->BeginTransaction(TRANSACTION_READ_ONLY);
		ASSERT(view);
		COMTHROW(anns.CoCreateInstance(OLESTR("Mga.MgaRegNodes")));
		POSITION pos = list->GetHeadPosition();
		while(pos) {
			COMTHROW(anns->Append(list->GetNext(pos)->rootNode));
		}
		view->CommitTransaction();
	}
	catch(hresult_exception e) {
		view->AbortTransaction(e.hr);
		return false;
	}
	return true;
}

void CGMEDoc::CopyToClipboard(CGuiObjectList* list, CGuiAnnotatorList *annList,CGMEDataDescriptor* desc,CGMEView *view)
{
	ASSERT( list && desc );

	CGMEDataSource* pDataSource = new CGMEDataSource(theApp.mgaProject);
	CComPtr<IMgaFCOs> fcos;
	CreateFcoList(list,fcos,view);
	CComPtr<IMgaRegNodes> anns;
	CreateAnnotationList(annList, anns,view);
	pDataSource->SetNodes(fcos);
	pDataSource->SetAnnotations(anns);
	pDataSource->CacheDescriptor(desc);
	pDataSource->SetClipboard();
	
}

void CGMEDoc::CopyClosureToClipboard( CGuiObjectList* list, CGuiAnnotatorList *annList, CGMEDataDescriptor* desc, CGMEView *view)
{
	ASSERT( list && desc );

	CComPtr<IMgaFCOs> fcos;
	CreateFcoList( list, fcos, view);

	CComPtr<IMgaClosure> cl;
	CComObjPtr<IMgaFCOs> sel_fcos, top_fcos;
	CComObjPtr<IMgaFolders> sel_folders, top_folders;
	long options;
	COMTHROW( cl.CoCreateInstance(L"Mga.MgaClosure") );
	COMTHROW( cl->SelectiveClosure( fcos, 0, PutOut( sel_fcos), PutOut( sel_folders), PutOut( top_fcos), PutOut( top_folders), 0, &options ));

	if ( sel_fcos || sel_folders) // if both null the operation is not needed (either dumped to file already or unsuccessful)
	{
		CGMEClosureDataSource * pDataSource = new CGMEClosureDataSource( theApp.mgaProject);

		if ( sel_fcos)		pDataSource->SetNodes( sel_fcos);
		if ( sel_folders)	pDataSource->SetFolders( sel_folders);
		
		if ( top_fcos)		pDataSource->SetTopNodes( top_fcos);
		if ( top_folders)	pDataSource->SetTopNodes( top_folders);
		
		pDataSource->SetOptions( options);

		CComPtr<IMgaRegNodes> anns; 
		CreateAnnotationList( annList, anns, view);
		pDataSource->SetAnnotations( anns);
		pDataSource->CacheDescriptor( desc);
		pDataSource->SetClipboard();
		pDataSource->FlushClipboard();
	}
}

DROPEFFECT CGMEDoc::DoDragDrop(CGuiObjectList* list, CGuiAnnotatorList *annList, CGMEDataDescriptor* desc,
	DWORD dwEffects, LPCRECT lpRectStartDrag,CGMEView *view)
{
	ASSERT( list && desc && annList );

	CGMEDataSource dataSource(theApp.mgaProject);
	CComPtr<IMgaFCOs> fcos;
	CreateFcoList(list,fcos,view);
	CComPtr<IMgaRegNodes> anns;
	CreateAnnotationList(annList, anns,view);
	dataSource.SetNodes(fcos);
	dataSource.SetAnnotations(anns);
	dataSource.CacheDescriptor(desc);
	DROPEFFECT de = dataSource.DoDragDrop(dwEffects,lpRectStartDrag);
	return de;
}

DROPEFFECT CGMEDoc::DoDragDrop(CGuiObject *guiObj, CGMEDataDescriptor* desc,
	DWORD dwEffects, LPCRECT lpRectStartDrag)
{
	CGMEDataSource dataSource(theApp.mgaProject);
	dataSource.SetMetaRole(guiObj->metaRole);
	dataSource.CacheDescriptor(desc);
	DROPEFFECT de = dataSource.DoDragDrop(dwEffects,lpRectStartDrag);
	return de;
}

CGMEView *CGMEDoc::FindView(CComPtr<IMgaModel> model)
{
	if(model != 0) {
		POSITION pos = GetFirstViewPosition();
		while (pos != NULL) {
			CGMEView* view = (CGMEView *)GetNextView(pos);
			VARIANT_BOOL b; 
			// COMTHROW(model->get_IsEqual(view->GetCurrentModel(), &b));  Territory problems
			// bool b = model.IsEqualObject(view->GetCurrentModel());
			view->BeginTransaction(TRANSACTION_READ_ONLY);
			COMTHROW(view->GetCurrentModel()->get_IsEqual(model, &b));
			view->CommitTransaction();
			if(b) 
				return view;
		}
	}
	return 0;
}

void CGMEDoc::InvalidateAllViews(bool thorough,bool /*fullAutoRoute*/)
{
	POSITION pos = GetFirstViewPosition();
	while (pos != NULL) {
		CGMEView* pView = (CGMEView *)GetNextView(pos);
/*
		if(!pView->IsIconic())
			fullAutoRoute ? pView->AutoRoute() : pView->IncrementalAutoRoute();
*/
		pView->Invalidate(thorough);
	}
	if(thorough) {
		CMDIFrameWnd *pFrame = (CMDIFrameWnd*)AfxGetApp()->m_pMainWnd;
		CMDIChildWnd *pChild = (CMDIChildWnd *) pFrame->GetActiveFrame();
		((CGMEView *)(pChild->GetActiveView()))->Invalidate(thorough);
	}
}

void CGMEDoc::ShowObject(CComPtr<IUnknown> alienObject)
{
	// Be prepared for in-transaction and out-transaction scenarios
	// mgaFCO must be transfered to current territory
		
	if(alienObject == NULL) {
		return;
	}

	if (!theApp.mgaProject) {
		return;
	}

	CComPtr<IMgaFCO> fco;
	CComPtr<IMgaModel> model;
	
	
	if (!SUCCEEDED(alienObject.QueryInterface(&model))) {  // Alien object is not a modell
		if (SUCCEEDED(alienObject.QueryInterface(&fco))) {
			
			long status;
			COMTHROW(theApp.mgaProject->get_ProjectStatus(&status));
			bool inTrans = (status & 0x08L) != 0;

			CComPtr<IMgaTerritory> terr;
			
			try {
				if (!inTrans) {
					COMTHROW(theApp.mgaProject->CreateTerritory(NULL, &terr));
					COMTHROW(theApp.mgaProject->BeginTransaction(terr));
				}
				else {
					COMTHROW(theApp.mgaProject->get_ActiveTerritory(&terr));
				}
				CComPtr<IMgaFCO> tmpfco = fco;
				fco = NULL;
				COMTHROW(terr->OpenFCO(tmpfco, &fco));
			}
			catch (hresult_exception e) {
				return;
			}

			try {
				CComPtr<IMgaObject> parent;
				COMTHROW(fco->GetParent(&parent));
				model = NULL;
				COMTHROW(parent.QueryInterface(&model));
			}
			catch (hresult_exception e) {
				model = NULL;
				if (!inTrans) 
					theApp.mgaProject->AbortTransaction();
			}

			if (!inTrans) {
				theApp.mgaProject->CommitTransaction();
			}
		}
	}
	
	if (model) {
		CGMEView *view  = NULL;

		if (!theApp.multipleView) {
			view = FindView(model);
			if(!view) {
				SetNextToView(model,"", fco);
			}
			else {
				view->SetCenterObject(fco);
			}
		}
		else {
			SetNextToView(model,"", fco);
		}

		CMainFrame::theInstance->CreateNewView(view);
		// PETER: This is needed to get the focus (SetFocus does not work, since it uses SendMessage())
		::PostMessage(CMainFrame::theInstance->GetSafeHwnd(), WM_SETFOCUS, 0, 0);
	}
}

void CGMEDoc::ShowObject(LPCTSTR objectID)
{
	CComBSTR bstrID(objectID);
	CComPtr<IMgaObject> object;
	CComPtr<IUnknown> unk;
	bool inTrans = false;


	try {
		long status;
		COMTHROW(theApp.mgaProject->get_ProjectStatus(&status));
		inTrans = (status & 0x08L) != 0;
		CComPtr<IMgaTerritory> terr;
		if (!inTrans) {
			COMTHROW(theApp.mgaProject->CreateTerritory(NULL, &terr));
			COMTHROW(theApp.mgaProject->BeginTransaction(terr));
		}
		else {
			COMTHROW(theApp.mgaProject->get_ActiveTerritory(&terr));
		}

		COMTHROW(theApp.mgaProject->GetObjectByID(bstrID, &object));
		COMTHROW(object.QueryInterface(&unk));

		if (!inTrans) {
			theApp.mgaProject->CommitTransaction();
		}
	}
	catch (hresult_exception e) {
		if (!inTrans) 
			theApp.mgaProject->AbortTransaction();
		return;
	}

	
	ShowObject(unk);
}

void CGMEDoc::ResetAllViews()
{
	POSITION pos = GetFirstViewPosition();
	if(pos) {
		while (pos != NULL) {
			CGMEView* pView = (CGMEView *)GetNextView(pos);
			pView->Reset(true);
		}
//		InvalidateAllViews(true);
	}
}

void CGMEDoc::ChangeAspects(CString aspName)
{
	POSITION pos = GetFirstViewPosition();
	if(pos) {
		while (pos != NULL) {
			CGMEView* pView = (CGMEView *)GetNextView(pos);
			pView->ChangeAspect(aspName);
		}
	}
}

void CGMEDoc::ViewModeChange()
{
	POSITION pos = GetFirstViewPosition();
	while (pos != NULL) {
		CGMEView* pView = (CGMEView *)GetNextView(pos);
		pView->ModeChange();
	}

	// make sure that the currently open model has the focus
	CMainFrame::theInstance->SetFocus();
}

/////////////////////////////////////////////////////////////////////////////
// CGMEDoc diagnostics

#ifdef _DEBUG
void CGMEDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CGMEDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGMEDoc commands

void CGMEDoc::OnModeAutoconnect() 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnModeAutoconnect\r\n");
	editMode = GME_AUTOCONNECT_MODE;
	CMainFrame::theInstance->WriteStatusMode("AUTOCONNECT");
	ViewModeChange();
}

void CGMEDoc::OnUpdateModeAutoconnect(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(editMode == GME_AUTOCONNECT_MODE);
}

void CGMEDoc::OnModeDisconnect() 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnModeDisconnect\r\n");
	editMode = GME_DISCONNECT_MODE;
	CMainFrame::theInstance->WriteStatusMode("DISCONNECT");
	ViewModeChange();
}

void CGMEDoc::OnUpdateModeDisconnect(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(editMode == GME_DISCONNECT_MODE);
}

void CGMEDoc::OnModeEdit() 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnModeEdit\r\n");
	editMode = GME_EDIT_MODE;
	CMainFrame::theInstance->WriteStatusMode("EDIT");
	ViewModeChange();
}

void CGMEDoc::OnUpdateModeEdit(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(editMode == GME_EDIT_MODE);
}

void CGMEDoc::OnModeSet() 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnModeSet\r\n");
	editMode = GME_SET_MODE;
	CMainFrame::theInstance->WriteStatusMode("SET");
	ViewModeChange();
}

void CGMEDoc::OnUpdateModeSet(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(editMode == GME_SET_MODE);
}

void CGMEDoc::OnModeZoom() 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnModeZoom\r\n");
	editMode = GME_ZOOM_MODE;
	CMainFrame::theInstance->WriteStatusMode("ZOOM");
	ViewModeChange();
}

void CGMEDoc::OnUpdateModeZoom(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(editMode == GME_ZOOM_MODE);
}

void CGMEDoc::OnModeVisualize() 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnModeVisualize\r\n");
	editMode = GME_VISUAL_MODE;
	CMainFrame::theInstance->WriteStatusMode("VISUALIZE");
	ViewModeChange();
}

void CGMEDoc::OnUpdateModeVisualize(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(editMode == GME_VISUAL_MODE);
}

void CGMEDoc::OnUpdateFileClose(CCmdUI* pCmdUI) 
{
	POSITION pos = GetFirstViewPosition();
	pCmdUI->Enable(pos != NULL);
}

void CGMEDoc::OnCloseDocument(bool suppressErrors) 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnCloseDocument\r\n");
	POSITION pos = GetFirstViewPosition();
	while (pos) {
		CGMEView* view = dynamic_cast<CGMEView*>(GetNextView(pos));
		ASSERT(view);
		bool canClose = view->SendCloseModelEvent();
		if ((!suppressErrors) && (!canClose)) {
			// View cannot be closed, eg.: constraint violations
			return;
		}
	}

	CDocument::OnCloseDocument();
	resolver = NULL;
	theApp.CloseProject();
	delete this;
}

BOOL CGMEDoc::OnNewDocument()
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnNewDocument\r\n");
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}

BOOL CGMEDoc::OnOpenDocument(LPCTSTR /*lpszPathName*/) 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnOpenDocument\r\n");
	if (!CDocument::OnNewDocument())
		return FALSE;
	
	// TODO: Add your specialized creation code here
	
	return TRUE;
}

void CGMEDoc::OnViewRefresh() 
{
	CGMEEventLogger::LogGMEEvent("CGMEDoc::OnViewRefresh\r\n");
	ResetAllViews();
	CGMEBrowser::theInstance->RefreshAll();
}
