#include "stdafx.h"
#include "MgaFCO.h"


#include "MgaFolder.h"
#include "MgaModel.h"
#include "MgaConnection.h"
#include "MgaReference.h"
#include "MgaSet.h"


// ----------------------------------------
// Constructor, destructor, FinalConstruct, getinterace
// ----------------------------------------

FCO::FCO()  { 
		INITSIG('F')
#ifdef DEBUG
		MGA_TRACE("Constructed: %s - %08X\n", sig, this);
#endif
		notifymask = changemask = temporalmask = 0;
}

STDMETHODIMP FCO::FinalConstruct() {
	COMTRY {
			{
				CoreObj myself;
				QueryInterface(IID_ICoreObject,(void **)&myself);
				self = myself;  // self does not hold reference!!!
			}
			CComPtr<ICoreProject> cp;
			COMTHROW(self->get_Project(&cp));
			COMTHROW(cp->QueryInterface(IID_IMgaProject,(void **)&mgaproject));
	} COMCATCH(;);
}


FCO::~FCO() { 
#ifdef DEBUG
		MGA_TRACE("Destructed: %s - %08X\n", sig, this);
#endif
		MARKSIG('9');
		ASSERT(pubfcos.begin() == pubfcos.end());
}


// return an IMgaObject for this FCO. By default, use the active territory to create it
void FCO::getinterface(IMgaObject **p, CMgaTerritory *terr) {
	ASSERT(mgaproject->activeterr);
	if(terr == NULL) terr = mgaproject->activeterr;
	pubfcohash::iterator ii = pubfcos.find(terr);
	if(ii != pubfcos.end())  ((*p) = (*ii).second)->AddRef();
	else {
		switch(GetMetaID(self)) {
		case DTID_MODEL:		{ CComPtr< CMgaModel > np; CreateComObject(np); np->Initialize(terr, this); *p = np.Detach(); } break;
		case DTID_ATOM:			{ CComPtr< CMgaAtom > np; CreateComObject(np); np->Initialize(terr, this); *p = np.Detach(); } break;
		case DTID_REFERENCE:	{ CComPtr< CMgaReference > np; CreateComObject(np); np->Initialize(terr, this); *p = np.Detach(); } break;
		case DTID_CONNECTION:	{ CComPtr< CMgaConnection > np; CreateComObject(np); np->Initialize(terr, this); *p = np.Detach(); } break;
		case DTID_SET:			{ CComPtr< CMgaSet > np; CreateComObject(np); np->Initialize(terr, this); *p = np.Detach(); } break;
		case DTID_FOLDER:		{ CComPtr< CMgaFolder > np; CreateComObject(np); np->Initialize(terr, this); *p = np.Detach(); } break;
		}
	}
	ASSERT(("Could not create MGA object",p));
}

// ----------------------------------------
// FCO status information methods: readibility, writability, IsEqual
// ----------------------------------------
//throws!!
long FCO::getstatus() {
		if(!mgaproject) COMTHROW(E_MGA_ZOMBIE_NOPROJECT);
		if(!mgaproject->opened) COMTHROW(E_MGA_ZOMBIE_CLOSED_PROJECT);
		if(!mgaproject->activeterr) COMTHROW(E_MGA_NOT_IN_TRANSACTION);
		VARIANT_BOOL pp;
		if(self->get_IsDeleted(&pp) != S_OK) return OBJECT_ZOMBIE;
		return pp ? OBJECT_DELETED  :  OBJECT_EXISTS;
}

//throws!!
void FCO::CheckDeletedRead() {
		if(getstatus() == OBJECT_ZOMBIE) COMTHROW(E_MGA_OBJECT_ZOMBIE);
}

//throws!!
void FCO::CheckRead() {
		if(getstatus() != OBJECT_EXISTS) COMTHROW(E_MGA_OBJECT_DELETED);
}

//throws!!
void FCO::CheckWrite() {
		CheckRead();
		ASSERT(mgaproject->alreadynested());
		if(mgaproject->read_only) COMTHROW(E_MGA_READ_ONLY_ACCESS);
		if(self[ATTRID_PERMISSIONS] & LIBRARY_FLAG) COMTHROW(E_MGA_LIBOBJECT);
}

HRESULT FCO::get_IsEqual(IMgaObject *o, VARIANT_BOOL *pVal) {
	COMTRY {
		CHECK_OUTPAR(pVal);
		if(o == NULL) *pVal = VARIANT_FALSE;
		else {
			CHECK_MYINPTRPAR(o);
			*pVal = (ObjFor(o) == this) ? VARIANT_TRUE : VARIANT_FALSE;
		}
	} COMCATCH(;)
}


HRESULT FCO::get_Status(long *p) {
	COMTRY {
		CHECK_OUTPAR(p);
		*p = getstatus(); 
	} COMCATCH(;);
}

HRESULT FCO::get_IsWritable(VARIANT_BOOL *p) {
	COMTRY {
		CHECK_OUTPAR(p);
		locking_type ppp;
		COMTHROW(self->get_PeerLockValue(ATTRID_LOCK, &ppp));
		*p =  (ppp & LOCKING_READ) ? VARIANT_FALSE : VARIANT_TRUE;
	} COMCATCH(;)
}


// is its metaobject a simpleconnection??
bool FCO::simpleconn() {
	CComPtr<IMgaMetaFCO> meta;
	if(get_Meta(&meta) != S_OK) return false;
	CComQIPtr<IMgaMetaConnection> metac = meta;
	if(!metac) return false;
	VARIANT_BOOL s;
	if(metac->get_IsSimple(&s) != S_OK) return false;
	return s ? true : false;
}


// ----------------------------------------
// Basic FCO info: project, ID, relID, meta, metarole, 
// ----------------------------------------

HRESULT FCO::get_Project(IMgaProject **pVal) { 
	COMTRY {
		CHECK_OUTPTRPAR(pVal); 
		*pVal = mgaproject; (*pVal)->AddRef(); 
	} COMCATCH(;)
} 


HRESULT FCO::CheckProject(IMgaProject *project) {
	return(project == mgaproject? S_OK : E_MGA_FOREIGN_PROJECT);
}


HRESULT FCO::get_ObjType(objtype_enum *pVal) { 
		COMTRY {
			CheckDeletedRead();
			CHECK_OUTPAR(pVal);
			*pVal = static_cast<objtype_enum>(GetMetaID(self) - DTID_BASE);
		} COMCATCH(;)
	};


HRESULT FCO::get_ID(ID_type *pVal) {
	COMTRY {
		CheckDeletedRead();
		CHECK_OUTSTRPAR(pVal);
		OLECHAR t[20];
		objid_type ss;
		COMTHROW(self->get_ObjID(&ss));
		swprintf(t,OLESTR("id-%04lx-%08lx"),GetMetaID(self), ss);
		*pVal = CComBSTR(t).Detach();
	} COMCATCH(;);
}

HRESULT FCO::get_RelID(long *pVal) {
	COMTRY {
		CheckRead();
		CHECK_OUTPAR(pVal);
		*pVal = self[ATTRID_RELID];
	} COMCATCH(;);
}

HRESULT FCO::put_RelID(long newVal) {
	COMTRY_IN_TRANSACTION {
		CheckWrite();
		if(newVal <= 0 || newVal >= RELIDSPACE) COMTHROW(E_MGA_ARG_RANGE);
		self[ATTRID_RELID] = newVal;
	} COMCATCH_IN_TRANSACTION(;);
}


HRESULT FCO::get_MetaRole(/**/ IMgaMetaRole **pVal) {
		COMTRY {
			CheckDeletedRead();
			CHECK_OUTPTRPAR(pVal);
			metaref_type role = self[ATTRID_ROLEMETA];
			if(role) {
				*pVal = CComQIPtr<IMgaMetaRole>(mgaproject->FindMetaRef(role)).Detach();
				if(!(*pVal)) COMTHROW(E_MGA_META_INCOMPATIBILITY);
			}

		} COMCATCH(;);
}

HRESULT FCO::get_Meta(IMgaMetaFCO **pVal) { 
	COMTRY {
		CheckDeletedRead();
		CHECK_OUTPTRPAR(pVal);
		COMTRY {
			*pVal = CComQIPtr<IMgaMetaFCO>(mgaproject->FindMetaRef(self[ATTRID_META])).Detach();
		} COMCATCH(e.hr = E_MGA_META_INCOMPATIBILITY;)

		if(!(*pVal)) COMTHROW(E_MGA_META_INCOMPATIBILITY);
	} COMCATCH(;)
}

HRESULT FCO::get_MetaBase(IMgaMetaBase **pVal) { 
	COMTRY {
		CheckDeletedRead();
		CHECK_OUTPTRPAR(pVal);
		*pVal = mgaproject->FindMetaRef(self[ATTRID_META]).Detach();
	} COMCATCH(;)
}

// ----------------------------------------
// Parent and grandparent access
// ----------------------------------------

HRESULT FCO::GetParent(IMgaContainer **pVal, objtype_enum *l) { 
		COMTRY {
			CheckRead();
			CHECK_OUTPTRPARVALIDNULL(pVal);
			CComPtr<IMgaContainer> pv;
			CoreObj par;
			par = self[ATTRID_PARENT];
			if(par.GetMetaID() != DTID_ROOT) {
				ObjForCore(par)->getinterface(&pv);
				if(l != NULL) pv->get_ObjType(l);
				if(pVal != NULL) {
					*pVal = pv.Detach();
					COMTHROW((*pVal)->Open());
				}
			}
			else {
				if(l != NULL) *l = OBJTYPE_NULL;
				if(pVal != NULL) *pVal = NULL;
			}
		} COMCATCH(;)
	};


HRESULT FCO::get_ParentModel(IMgaModel **pVal) {
		COMTRY {
			CheckRead();
			CHECK_OUTPTRPAR(pVal);
			CoreObj par;
			par = self[ATTRID_FCOPARENT];
			if(par.GetMetaID() == DTID_MODEL) {
				CComPtr<IMgaModel> pv;
				ObjForCore(par)->getinterface(&pv);
				*pVal = pv.Detach();
				COMTHROW((*pVal)->Open());
			}
		} COMCATCH(;);
}

HRESULT FCO::get_ParentFolder(IMgaFolder **pVal) {
		COMTRY {
			CheckRead();
			CHECK_OUTPTRPAR(pVal);
			CoreObj par;
			par = self[ATTRID_FCOPARENT];
			if(par.GetMetaID() == DTID_FOLDER) {
				CComPtr<IMgaFolder> pv;
				ObjForCore(par)->getinterface(&pv);
				*pVal = pv.Detach();
				COMTHROW((*pVal)->Open());
			}
		} COMCATCH(;);
}



HRESULT FCO::get_RootParent(IMgaFCO **parent, long *distance) {       // retuns itself if root model
	COMTRY {
		long d = 0;
		CComPtr<IMgaFCO> cur;
		getinterface(&cur);
		while(true) {
			CComPtr<IMgaContainer> f;
			objtype_enum t;
			COMTHROW(cur->GetParent(&f, &t));
			if(t != OBJTYPE_MODEL) break;
			cur = NULL;
			f.QueryInterface(&cur);
			d++;
		}
		if(parent) {
			CHECK_OUTPTRPAR(parent);
			*parent = cur.Detach();
		}
		if(distance) *distance = d;
	} COMCATCH(;)
}

HRESULT FCO::get_ParentChain(IMgaFCOs **parents) {    // returns the real parents only (not self)
	COMTRY {
		CComPtr<IMgaFCO> cur;
		getinterface(&cur);
		CREATEEXCOLLECTION_FOR(MgaFCO, q);
		while(true) {
			CComPtr<IMgaContainer> f;
			objtype_enum t;
			COMTHROW(cur->GetParent(&f, &t));
			if(t != OBJTYPE_MODEL) break;
			f.QueryInterface(&cur);
			q->Add(cur);
		}
		*parents = q.Detach();
	} COMCATCH(;)
}

// check if object is a child or any grandchild of parentobject (also true if itself)
bool FCO::IsDescendantOf(IMgaFCO *object, long *distance) {
		int d;
		if(IsContained(self, CoreObj(object), &d)) {
			if(distance) *distance = d;
			return true;
		}
		else return false;

}

// ----------------------------------------
// Name access
// ----------------------------------------
HRESULT FCO::get_Name(BSTR *pVal)		{ 
	COMTRY {
		CheckDeletedRead();
		CHECK_OUTPAR(pVal);  
		*pVal = CComBSTR(self[ATTRID_NAME]).Detach();
	} COMCATCH(;);
};


class put_NameTask : public DeriveTreeTask {
	CComBSTR oldname, newname;

	bool Do(CoreObj self, vector<CoreObj> *peers = NULL) {
		if(!(oldname == CComBSTR(self[ATTRID_NAME]))) return false;
		if(ObjForCore(self)->mgaproject->preferences & MGAPREF_NODUPLICATENAMES) {
			CoreObj parent = self[ATTRID_PARENT];
			if(parent.GetMetaID() != DTID_ROOT) {  // Make sure it is not rootfolder (=parent is not rootobj)
				CoreObjs children = parent[ATTRID_PARENT+ATTRID_COLLECTION];
				ITERATE_THROUGH(children) {
					CComBSTR n = ITER[ATTRID_NAME];
					if(n == newname && !COM_EQUAL(self, ITER)) COMTHROW(E_MGA_NAME_DUPLICATE);
				}
			}		
		}
		self[ATTRID_NAME] = newname;
		ObjForCore(self)->SelfMark(OBJEVENT_PROPERTIES);
		if(self.IsFCO() && self.IsRootOfDeriv()) return false;   // Type renaming does not affect instances
		return true;
	}

public:
	put_NameTask(BSTR oname, BSTR nname) : oldname(oname), newname(nname) { ; }
};


void  FCO::initialname() { 
	if(mgaproject->preferences & MGAPREF_NAMENEWOBJECTS) {
		metaref_type mr;
		if(self.IsFCO() && !self.IsRootFCO()) mr = self[ATTRID_ROLEMETA];
		else mr = self[ATTRID_META];
		CComBSTR nname;
		COMTHROW(mgaproject->FindMetaRef(mr)->get_Name(&nname));
		if(mgaproject->preferences & MGAPREF_NODUPLICATENAMES) {
			size_t len = nname.Length();
			unsigned int freenum = 0;
			CoreObj parent = self[ATTRID_PARENT];
			if(parent.GetMetaID() != DTID_ROOT) {  // Make sure it is not rootfolder (=parent is not rootobj)
				CoreObjs children = parent[ATTRID_PARENT+ATTRID_COLLECTION];
				ITERATE_THROUGH(children) {
					CComBSTR n = ITER[ATTRID_NAME];
					if(!wcsncmp(nname, n, len)) {
						unsigned int f = 0;
						if(n.Length() == len) f = 1;
						else {
							swscanf(n+len,L"-%d", &f);
							f++;
							if(f > NEWNAME_MAXNUM) COMTHROW(E_MGA_GEN_OUT_OF_SPACE);
						}
						if(f > freenum) freenum = f;
					}
				}
			}
			if(freenum) {
				OLECHAR p[10];
				swprintf(p,L"-%d",freenum);
				nname.Append(p);
			}
		}
		self[ATTRID_NAME] = nname;
	}
}

HRESULT FCO::put_Name(BSTR newVal)    { 
	COMTRY_IN_TRANSACTION {
		CheckWrite();
		CHECK_INPAR(newVal);  
		if(self[ATTRID_PERMISSIONS] & LIBROOT_FLAG) COMTHROW(E_MGA_LIBOBJECT);
		if(CComBSTR(self[ATTRID_NAME]) != newVal) {
			put_NameTask(CComBSTR(self[ATTRID_NAME]), newVal).DoWithDeriveds(self);;
		}
	}
	COMCATCH_IN_TRANSACTION(;);	
};


// gives back the path to an object starting from the rootfolder (does not include project name)
HRESULT FCO::get_AbsPath(BSTR *pVal)		{ 
	COMTRY {
		CComBSTR path("/@");
		path.Append( self[ATTRID_NAME]);

		CoreObj par = self[ATTRID_PARENT];
		while( par.GetMetaID() != DTID_ROOT)
		{
			CComBSTR tp("/@");
			tp.Append( par[ATTRID_NAME]);

			par = par[ATTRID_PARENT];

			if( par.GetMetaID() != DTID_ROOT) // do not include project name
			{
				tp.AppendBSTR( path);
				path = tp;
			}
			
		}
		CheckDeletedRead();
		CHECK_OUTPAR(pVal);  
		*pVal = path.Detach();
	} COMCATCH(;);
};

// ----------------------------------------
// Add FCO to a collection (create new coll if null)
// ----------------------------------------
HRESULT FCO::CreateCollection(IMgaFCOs **pVal) {
	COMTRY {
// check type of collection
		CComPtr<IMgaFCO> tthis;
		getinterface(&tthis);
		if(*pVal) {
			CComPtr< COLLECTIONTYPE_FOR(IMgaFCO)> q;
			q.Attach(static_cast< COLLECTIONTYPE_FOR(IMgaFCO) *>(*pVal));
			q->Add(tthis);
			/* *pVal = */q.Detach(); // modified by ZolMol
		}
		else {
			CREATEEXCOLLECTION_FOR(MgaFCO,q2)
			COMTHROW(q2->Append(tthis));
			*pVal = q2.Detach(); // added by ZolMol
		}
	} COMCATCH(;);
}


// ----------------------------------------
// Parts access
// ----------------------------------------
HRESULT FCO::get_Parts(struct IMgaParts ** pVal) {
		COMTRY {
			CheckRead();
			CHECK_OUTPTRPAR(pVal);

			CComPtr<IMgaMetaRole> metaro;					
			COMTHROW(get_MetaRole(&metaro));
			if(!metaro) COMTHROW( E_MGA_ROOTFCO);

			CComPtr<IMgaMetaParts> metaps;					
			COMTHROW(metaro->get_Parts(&metaps));

			CREATECOLLECTION_FOR(IMgaPart,q);
			MGACOLL_ITERATE(IMgaMetaPart, metaps) {
					metaref_type r;	            				
					COMTHROW(MGACOLL_ITER->get_MetaRef(&r));
					q->Add(ppool.getpoolobj(r, this, mgaproject));
			}
			MGACOLL_ITERATE_END;
			*pVal = q.Detach();

		} COMCATCH(;);
}

HRESULT FCO::get_Part(IMgaMetaAspect * asp, IMgaPart **pVal) {
		COMTRY {
			CheckRead();
			CHECK_INPTRPAR(asp);
			CHECK_OUTPTRPAR(pVal);

			CComPtr<IMgaMetaRole> metaro;					
			COMTHROW(get_MetaRole(&metaro));
			if(!metaro) COMTHROW( E_MGA_ROOTFCO);

			CComPtr<IMgaMetaParts> metaps;					
			COMTHROW(metaro->get_Parts(&metaps));
			MGACOLL_ITERATE(IMgaMetaPart, metaps) {
				CComPtr<IMgaMetaAspect> metaa;					
				COMTHROW(MGACOLL_ITER->get_ParentAspect(&metaa));
				if(COM_EQUAL(metaa, asp)) {
					metaref_type r;	            				
					COMTHROW(MGACOLL_ITER->get_MetaRef(&r));
					*pVal = ppool.getpoolobj(r, this, mgaproject).Detach();
				}
			}

			MGACOLL_ITERATE_END;
		} COMCATCH(;);
}

HRESULT FCO::get_PartByMP(IMgaMetaPart *part, IMgaPart **pVal) {
		COMTRY {
			CheckRead();
			CHECK_INPTRPAR(part);
			CHECK_OUTPTRPAR(pVal);

			CComPtr<IMgaMetaRole> metaro, metaro2;					
			COMTHROW(get_MetaRole(&metaro));
			if(!metaro) COMTHROW( E_MGA_ROOTFCO);

			COMTHROW(part->get_Role(&metaro2));
			if(!COM_EQUAL(metaro, metaro2)) COMTHROW( E_MGA_INVALID_ROLE);

			metaref_type r;	            				
			COMTHROW(part->get_MetaRef(&r));
			*pVal = ppool.getpoolobj(r, this, mgaproject).Detach();
		} COMCATCH(;);
}



// ----------------------------------------
// Attributes access
// ----------------------------------------
HRESULT FCO::get_Attributes(IMgaAttributes **pVal) { 
		COMTRY {
			CheckRead();
			CHECK_OUTPTRPAR(pVal);

			CComPtr<IMgaMetaFCO> metao;					
			COMTHROW(get_Meta(&metao));

			CComPtr<IMgaMetaAttributes> metaas;					
			COMTHROW(metao->get_Attributes(&metaas));

			CREATECOLLECTION_FOR(IMgaAttribute,q);
			MGACOLL_ITERATE(IMgaMetaAttribute, metaas) {
					metaref_type r;	            				
					COMTHROW(MGACOLL_ITER->get_MetaRef(&r));
					q->Add(apool.getpoolobj(r, this, mgaproject));
			}
			MGACOLL_ITERATE_END;
			*pVal = q.Detach();

		} COMCATCH(;);
}

HRESULT FCO::get_Attribute(IMgaMetaAttribute *metaa,  IMgaAttribute **pVal) { 
		COMTRY {
			CheckRead();
			CHECK_OUTPTRPAR(pVal);
			CHECK_INPTRPAR(metaa);
			CComPtr<IMgaMetaFCO> metao;					
			COMTHROW(get_Meta(&metao));
			metaref_type r;	            				
			COMTHROW(metaa->get_MetaRef(&r));
#ifdef OWN_META
			CComPtr<IMgaMetaFCO> meta;				    
			COMTHROW(metaa->get_Owner(&meta));
			if(!meta.IsEqualObject(metao)) return E_META_INVALIDATTR;
#else
			{
			CComPtr<IMgaMetaFCOs> metas;				    
			COMTHROW(metaa->get_UsedIn(&metas));
			MGACOLL_ITERATE(IMgaMetaFCO,metas) {
				if(MGACOLL_ITER.IsEqualObject(metao)) break;
			}
			if(MGACOLL_AT_END) return E_META_INVALIDATTR;
			MGACOLL_ITERATE_END;
			}
#endif

			*pVal = apool.getpoolobj(r, this, mgaproject).Detach();
		} COMCATCH(;);
	};

// THROWS!!, always returns a valid attr.
CComPtr<IMgaAttribute> FCO::AbyN(BSTR name) {
		CHECK_INSTRPAR(name);
		CComPtr<IMgaMetaAttribute> metaattr;
		CComPtr<IMgaMetaFCO> meta;
		metaref_type r;
		COMTHROW(get_Meta(&meta));
		COMTHROW(meta->get_AttributeByName(name, &metaattr));
		COMTHROW(metaattr->get_MetaRef(&r));
		return apool.getpoolobj(r, this, mgaproject);
}

HRESULT FCO::get_AttributeByName(BSTR name, VARIANT *pVal) { 
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->get_Value(pVal));
		} COMCATCH(;);
};
HRESULT FCO::put_AttributeByName(BSTR name, VARIANT newVal) { 
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->put_Value(newVal));
		} COMCATCH(;);
}
HRESULT FCO::get_StrAttrByName( BSTR name,  BSTR *pVal) {  
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->get_StringValue(pVal));
		} COMCATCH(;);
}
HRESULT FCO::put_StrAttrByName( BSTR name,  BSTR newVal) {
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->put_StringValue(newVal));
		} COMCATCH(;);
}
HRESULT FCO::get_IntAttrByName( BSTR name,  long *pVal) { 
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->get_IntValue(pVal));
		} COMCATCH(;);
}
HRESULT FCO::put_IntAttrByName( BSTR name,  long newVal) {
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->put_IntValue(newVal));
		} COMCATCH(;);
}
HRESULT FCO::get_FloatAttrByName( BSTR name,  double *pVal) { 
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->get_FloatValue(pVal));
		} COMCATCH(;);
}
HRESULT FCO::put_FloatAttrByName( BSTR name,  double newVal) { 
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->put_FloatValue(newVal));
		} COMCATCH(;);
}
HRESULT FCO::get_BoolAttrByName( BSTR name,  VARIANT_BOOL *pVal) { 
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->get_BoolValue(pVal));
		} COMCATCH(;);
}
HRESULT FCO::put_BoolAttrByName( BSTR name,  VARIANT_BOOL newVal) {
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->put_BoolValue(newVal));
		} COMCATCH(;);
}
HRESULT FCO::get_RefAttrByName( BSTR name,  IMgaFCO **pVal) {
		COMTRY {
			CheckRead();
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->get_FCOValue(pVal));
		} COMCATCH(;);
}
HRESULT FCO::put_RefAttrByName( BSTR name,  IMgaFCO * newVal) {
		COMTRY {
			CheckRead();
			if(newVal) CHECK_MYINPTRPAR(newVal);
			// parameter will be checked by attr. fun
			COMTHROW(AbyN(name)->put_FCOValue(newVal));
		} COMCATCH(;);
}

HRESULT FCO::ClearAttrByName( BSTR name) { 
		COMTRY_IN_TRANSACTION {
			CheckWrite();
			COMTHROW(AbyN(name)->Clear());
		} COMCATCH_IN_TRANSACTION(;);
}

HRESULT FCO::get_RegistryNode( BSTR path,  IMgaRegNode **pVal) {  
	COMTRY {
			CheckRead();
			CHECK_INSTRPAR(path);
			CHECK_OUTPTRPAR(pVal);
			*pVal = rpool.getpoolobj(path, this, mgaproject).Detach();
	} COMCATCH(;)
}


// ----------------------------------------
// Registry access
// ----------------------------------------
HRESULT FCO::get_Registry(VARIANT_BOOL virtuals, IMgaRegNodes **pVal) {  
	COMTRY {  
		CheckRead();
		CHECK_OUTPTRPAR(pVal);
		CREATEEXCOLLECTION_FOR(MgaRegNode,q);
		CoreObj s = self;
		if(!s.IsFCO()) virtuals = VARIANT_FALSE;
		hash_set<CComBSTRNoAt, CComBSTR_hashfunc> match;
		do {
			CoreObjs children = s[ATTRID_REGNOWNER+ATTRID_COLLECTION];
			ITERATE_THROUGH(children) {
				CComBSTR path;
				path	= ITER[ATTRID_NAME];
				if(virtuals) {
					if(match.find(path) != match.end()) continue;
					match.insert(path);
				}
				q->Add(rpool.getpoolobj(path, this, mgaproject));
			}
		} while(virtuals && (s = s[ATTRID_DERIVED]));
		if(virtuals) {
			CComQIPtr<IMgaMetaBase> m;
			COMTHROW(get_MetaBase(&m));
			CComPtr<IMgaMetaRegNodes> rns;
			COMTHROW(m->get_RegistryNodes(&rns));
			MGACOLL_ITERATE(IMgaMetaRegNode, rns) {
				CComBSTR path;
				COMTHROW(MGACOLL_ITER->get_Name(&path));
				if(match.find(path) != match.end()) continue;
				q->Add(rpool.getpoolobj(path, this, mgaproject));
			} MGACOLL_ITERATE_END;
		}

		*pVal = q.Detach();
	} COMCATCH(;);
}
HRESULT FCO::get_RegistryValue( BSTR path,  BSTR *pVal) {  
	COMTRY {  // no arg check, called methods will do that
		CheckRead();
		CComPtr<IMgaRegNode> node;
		COMTHROW(get_RegistryNode(path, &node));
		COMTHROW(node->get_Value(pVal));
	} COMCATCH(;)
}
HRESULT FCO::put_RegistryValue( BSTR path,  BSTR newval) {  
	COMTRY {  // no arg check, called methods will do that
		CheckRead();  // put_Value will check write
		CComPtr<IMgaRegNode> node;
		COMTHROW(get_RegistryNode(path, &node));
		COMTHROW(node->put_Value(newval));
	} COMCATCH(;)
}



#define REQUIRE_NOTIFICATION_SUCCESS 1   // return with error if addons/rwhandlers fail

HRESULT FCO::objrwnotify() {
	COMTRY {
			unsigned long chmask = changemask;
			changemask = 0;
			// No other notification on deleted objects
			if(chmask & OBJEVENT_DESTROYED) chmask = OBJEVENT_DESTROYED;
			CMgaProject::addoncoll::iterator ai, abeg = mgaproject->alladdons.begin(), aend = mgaproject->alladdons.end();
			if(abeg != aend) {
				CComVariant nil;
				COMTHROW(mgaproject->pushterr(*mgaproject->reserveterr));
				for(ai = abeg; ai != aend; ) {
					CComPtr<CMgaAddOn> t = *ai++;	// it is important to incr ii here, while obj 
												    // still holds a reference to the (*ai) object
					unsigned long mmask;
					if((mmask = (t->eventmask & chmask)) != 0) {
						CComPtr<IMgaObject> tt;
						getinterface(&tt);

#if(REQUIRE_NOTIFICATION_SUCCESS)
						COMTHROW(t->handler->ObjectEvent(tt, mmask, nil));
#else
						if((s = t->handler->ObjectEvent(tt, mmask, nil)) != S_OK) {
							ASSERT(("Notification failed", false));
						}
#endif
					    t->notified = true;
					}
				}
				COMTHROW(mgaproject->popterr());
			}
			if(chmask & OBJEVENT_CREATED) {
				// send message to all territories that contain parent, 
				CoreObj parent = self[ATTRID_PARENT];
				if(parent.IsContainer()) {
					FCO &p = *ObjForCore(parent);
					
					// if parent is also new, notify it first
					if(p.notifymask & OBJEVENT_CREATED) COMTHROW(p.objnotify());

					pubfcohash::iterator ii, pbeg = p.pubfcos.begin(), pend = p.pubfcos.end();
					for(ii = pbeg; ii != pend;) {  
						CMgaTerritory *t = (*ii).second->territory;
						CComPtr<IMgaObject> obj = (*ii).second;
						ii++;  // it is important to incr ii here, while obj 
							   // still holds a reference to the (*ii) object
						CComVariant dummy;
						if(t->rwhandler && (t->eventmask & OBJEVENT_CREATED) != 0) {
							COMTHROW(mgaproject->pushterr(*t));
							{
								CComPtr<IMgaObject> newoo;
								getinterface(&newoo, t);
#if(REQUIRE_NOTIFICATION_SUCCESS)
								COMTHROW(t->rwhandler->ObjectEvent(newoo, (unsigned long)OBJEVENT_CREATED, dummy));
#else
								if(t->rwhandler->ObjectEvent(newoo, (unsigned long)OBJEVENT_CREATED, dummy) != S_OK) {
									ASSERT(("Notification failed", false));
								}
#endif
							}
							COMTHROW(mgaproject->popterr());
							t->notified = true;
						}
					}
				}
				chmask &= ~OBJEVENT_CREATED;
			}
			pubfcohash::iterator ii, beg = pubfcos.begin(), end = pubfcos.end();
			for(ii = beg; ii != end;) {  
				CMgaTerritory *t = (*ii).second->territory;
				CComVariant &ud = (*ii).second->userdata;
				CComPtr<IMgaObject> obj = (*ii).second;
				ii++;  // it is important to incr ii here, while obj 
					   // still holds a reference to the (*ii) object
				long mmask;
				if(t->rwhandler && (mmask = (t->rweventmask & chmask)) != 0) {
					COMTHROW(mgaproject->pushterr(*t));
#if(REQUIRE_NOTIFICATION_SUCCESS)
					COMTHROW(t->rwhandler->ObjectEvent(obj, mmask,ud));
#else
					if(t->rwhandler->ObjectEvent(obj, mmask,ud) != S_OK) {
						ASSERT(("Notification failed", false));
					}
#endif
					t->notified = true;
					COMTHROW(mgaproject->popterr());  // this may release the territory!!!
				}
			}
	} COMCATCH(;)
}





HRESULT FCO::objnotify() {
	if(notifymask == 0) return S_OK;
	COMTRY {
			unsigned long chmask = notifymask;
			notifymask = 0;
			// No other notification on deleted objects
			if(chmask & OBJEVENT_DESTROYED) chmask = OBJEVENT_DESTROYED;
			if(chmask & OBJEVENT_CREATED) {
				// send message to all territories that contain parent, 
				CoreObj parent = self[ATTRID_PARENT];
				if(parent.IsContainer()) {
					FCO &p = *ObjForCore(parent);
					
					// if parent is also new, notify it first
					if(p.notifymask & OBJEVENT_CREATED) COMTHROW(p.objnotify());

					pubfcohash::iterator ii, pbeg = p.pubfcos.begin(), pend = p.pubfcos.end();
					for(ii = pbeg; ii != pend;) {  
						CMgaTerritory *t = (*ii).second->territory;
						CComPtr<IMgaObject> obj = (*ii).second;
						ii++;  // it is important to incr ii here, while obj 
							   // still holds a reference to the (*ii) object
						CComVariant dummy;
						if(t->handler && (t->eventmask & OBJEVENT_CREATED) != 0) {
							COMTHROW(mgaproject->pushterr(*t));
							{
								CComPtr<IMgaObject> newoo;
								getinterface(&newoo, t);
								if(t->handler->ObjectEvent(newoo, (unsigned long)OBJEVENT_CREATED, dummy) != S_OK) {
									ASSERT(("Notification failed", false));
								}
							}
							COMTHROW(mgaproject->popterr());
							t->notified = true;
						}
					}
				}
				chmask &= ~OBJEVENT_CREATED;
			}
			pubfcohash::iterator ii, beg = pubfcos.begin(), end = pubfcos.end();
			for(ii = beg; ii != end;) {  
				CMgaTerritory *t = (*ii).second->territory;
				CComVariant &ud = (*ii).second->userdata;
				CComPtr<IMgaObject> obj = (*ii).second;
				ii++;  // it is important to incr ii here, while obj 
					   // still holds a reference to the (*ii) object
				unsigned long mmask;
				if(t->handler && (mmask = (t->eventmask & chmask)) != 0) {
					COMTHROW(mgaproject->pushterr(*t));
					if(t->handler->ObjectEvent(obj, mmask, ud) != S_OK) {
						ASSERT(("Notification failed", false));
					}
					COMTHROW(mgaproject->popterr());
					t->notified = true;
				}
			}
			ASSERT(notifymask == 0);
	} COMCATCH(;)
}

void FCO::objforgetchange() {
	temporalmask = 0;
}

void FCO::objrecordchange() {
	ASSERT(temporalmask != 0);
	if(!changemask) {
		mgaproject->changedobjs.push(this);
	}	
	changemask |= temporalmask;
// 
	if(!notifymask) {
		mgaproject->notifyobjs.push(this);
	}	
	notifymask |= temporalmask;
	temporalmask = 0;
}


HRESULT FCO::SendEvent(long mask) {
	COMTRY_IN_TRANSACTION {
		CheckRead();
		if((mask | OBJEVENT_USERBITS) != OBJEVENT_USERBITS) COMTHROW(E_MGA_BAD_MASKVALUE);
		SelfMark(mask);
	} COMCATCH_IN_TRANSACTION(;)
}

void FCO::SelfMark(long newmask) {
	ASSERT(("Error: event generated in read-only transaction",!mgaproject->read_only));
	ASSERT(newmask);
	if(!temporalmask) {
		mgaproject->temporalobjs.push(this);
	}	
	temporalmask |= (unsigned long)newmask;
}

void FCO::objsetuserdata(CMgaTerritory *t, VARIANT udata) {
		pubfcohash::iterator i = pubfcos.find(t); 
		if(i == pubfcos.end()) COMTHROW(E_MGA_NOT_IN_TERRITORY);
		(*i).second->userdata = udata;
}

/*
HRESULT FCO::Associate(VARIANT userdata) {
  COMTRY {
	CheckDeletedRead();
	objsetuserdata(mgaproject->activeterr, userdata);
  } COMCATCH(;);
}

HRESULT FCO::get_CurrentAssociation(VARIANT *userdata) {
  COMTRY {
	CheckDeletedRead();
		pubfcohash::iterator i = pubfcos.find(mgaproject->activeterr);
		if(i == pubfcos.end()) COMTHROW(E_MGA_NOT_IN_TERRITORY);
		VariantCopy(userdata, &((*i).second->userdata)); 
   } COMCATCH(;);
}
*/

HRESULT FCO::Open( openmode mode)  { 
	COMTRY {	
		ASSERT(mode == LOCKING_READ || mode == LOCKING_EXCLUSIVE);

		CComVariant lock = self[ATTRID_LOCK];
		ASSERT (lock.vt == VT_UI1);
		int  m = lock.bVal;

		switch(m) {
		case LOCKING_READ:
				if(mode == LOCKING_READ) break;
		case LOCKING_WRITE:
				mode = static_cast<openmode>(LOCKING_EXCLUSIVE);
		case LOCKING_NONE:
				self[ATTRID_LOCK] = mode;
		case LOCKING_EXCLUSIVE:
				break;
		default: 
				ASSERT(("Invalid locking value detected",false));
		}
	} COMCATCH(;);
};


// results in a single object with all local subobjects removed
void SingleObjTreeUnLock(CoreObj &self) {
	CComPtr<ICoreAttributes> atts;
	COMTHROW(self->get_Attributes(&atts));
	MGACOLL_ITERATE(ICoreAttribute, atts) {
			attrid_type ai;
			CComPtr<ICoreMetaAttribute> mattr;
			COMTHROW(MGACOLL_ITER->get_MetaAttribute(&mattr));
			COMTHROW(mattr->get_AttrID(&ai));
			if(ai >= ATTRID_COLLECTION) {
				ai -= ATTRID_COLLECTION;
				if(LINKREF_ATTR(ai) && ai != ATTRID_PARENT) {
					CoreObjs collmembers = self[ai + ATTRID_COLLECTION].CachedColl();
					ITERATE_THROUGH(collmembers) {
						SingleObjTreeUnLock(ITER);
					}
				}
			}
	} MGACOLL_ITERATE_END;
	self[ATTRID_LOCK] = LOCKING_NONE;
}


HRESULT FCO::Close () {
	COMTRY {
		SingleObjTreeUnLock(self);
	} COMCATCH(;) 
}




FCOPtr::FCOPtr(FCOPtr const &o) {
	p = o.p;
	p->AddRef();
}

FCOPtr::FCOPtr(FCO *f) {
	p = f;
	p->AddRef();
}

FCOPtr::FCOPtr() {
	p = NULL;
}

FCOPtr::~FCOPtr() {
	if(p) p->Release();
}

void FCOPtr::operator= (FCO *f) {	
	if(p == f) return;
	if(p) p->Release();
	p = f;
	p->AddRef();
}

bool FCOPtr::operator< (const FCOPtr &o) const {
	return p < o.p;
}

void CoreObjMark(CoreObj const &ob, long mask) {
	ObjForCore(ob)->SelfMark(mask);
}
	
