#include "stdafx.h"

#include "MgaClosure.h"
#include "ClosureDlg.h"
#include "ClosureRes.h"
#include "MakeClosure.h"


STDMETHODIMP CMgaClosure::SelectiveClosure(
	IMgaFCOs *in_fcos, IMgaFolders *in_folders, 
	IMgaFCOs **sel_fcos, IMgaFolders **sel_folders, 
	IMgaFCOs **top_fcos, IMgaFolders **top_folders, 
	int fc,	long * pOptions) 
{
	AFX_MANAGE_STATE( AfxGetStaticModuleState());
	ASSERT( pOptions);

	CWaitCursor w;
	CClosureDlg dlg;
	dlg.m_activeFolderContainment = fc != 0;
	if ( dlg.DoModal() == IDOK)
	{
		CMakeClosure mc( 
			in_fcos, in_folders,
			dlg.m_bRefersTo, dlg.m_bReferredBy,
			dlg.m_bConnection, 
			dlg.m_bSetMember, dlg.m_bMemberOfSets, 
			// compare the containment options with the direction option
			dlg.m_direction?dlg.m_bContainment:0, !dlg.m_direction?dlg.m_bPartOfModels:0,
			dlg.m_direction?dlg.m_bFolderContainment:0, !dlg.m_direction?dlg.m_bPartOfFolders:0, 
			dlg.m_bBaseTypes, dlg.m_bDerivedTypes, 
			//dlg.m_bDigIntoLibraries, 
			dlg.m_libraryHandling /*== 2*/, // in case of "continue" ("go on") only
			dlg.m_wrappingOption,
			dlg.m_direction);

		mc.process(); // do the hard job

		// load the results
		mc.getFCOs( sel_fcos);
		mc.getFolders( sel_folders);

		bool normal_tops = !mc.noTopObjs();
		
		if ( normal_tops) // the top fcos/folds is calculated and The RootFolder is not among them
		{
			mc.getTopFCOs( top_fcos);
			mc.getTopFolders( top_folders);
		}

		
		if ( normal_tops &&								// avoid in case of non-normal tops
			dlg.m_wrappingOption > 0 &&					// avoid in case of miniproject
			mc.m_acceptingKindsAndFolders.size() > 0 )	// avoid if is no accepting kind/folder
		{
			CClosureRes resdlg;
			resdlg.m_metaKindsAndFolders = mc.m_acceptingKindsAndFolders;
			resdlg.DoModal();
		}

		*pOptions = 0;
		*pOptions = (dlg.m_libraryHandling == 0)?0x1:0; // in case of "stubs" set the options to 1

		if ( dlg.m_outputOption == 1) // dump to a file
		{
			CComPtr<IMgaDumper> dumper;
			COMTHROW( dumper.CoCreateInstance(L"Mga.MgaDumper") );

			switch( dlg.m_wrappingOption) {
			case 0:
				{
					COMTHROW( dumper->DumpClos( *sel_fcos, *sel_folders, PutInBstr( dlg.m_strOutputFile), *pOptions) );

					break;
				}
			case 1:
			case 2:
				{
					if ( normal_tops)
						COMTHROW( dumper->DumpClosR( *sel_fcos, *sel_folders, PutInBstr( dlg.m_strOutputFile), *top_fcos, *top_folders, *pOptions) );
					else 
						// dump like in miniproject case from The RootFolder
						// if the data is copied to clipboard then the DumpClosR will call the 
						// DumpClos when it will realize the topfco and topfolders are empty
						COMTHROW( dumper->DumpClos( *sel_fcos, *sel_folders, PutInBstr( dlg.m_strOutputFile), *pOptions) );

					*top_fcos = 0;
					*top_folders = 0;
					break;
				}
			} // switch

			*sel_fcos = 0;
			*sel_folders = 0;
		}
		// else: clipboard
	}
	return S_OK;
}
