
#ifndef MGA_GENPARSER_H
#define MGA_GENPARSER_H

#include <xercesc/sax/HandlerBase.hpp>

XERCES_CPP_NAMESPACE_USE

// --------------------------- XmlStr

class XmlStr : public string
{
public:
	XmlStr() { };
	XmlStr(const XMLCh* const input);
	XmlStr(const XMLCh* const input, unsigned int len);
};

// --------------------------- CGenParser

struct CGenParserFunc_Base;

class CGenParser : public HandlerBase
{
public:
	CGenParser() : locator(NULL), elementfuncs(NULL), err_line(0), err_column(0) { }

// ------- Erros

public:
	void SetErrorInfo2(HRESULT hr);
	void ThrowXmlError(const char *format, ...);

// ------- Handler Base

public:
	virtual InputSource *resolveEntity (const XMLCh* const publicId, const XMLCh* const systemId);

    virtual void startDocument();
    virtual void endDocument();

    virtual void startElement(const XMLCh* const name, AttributeList& attributes);
    virtual void endElement(const XMLCh* const name);

	virtual void characters(const XMLCh* const chars, const unsigned int length);

    virtual void error(const SAXParseException& exception);
    virtual void fatalError(const SAXParseException& exception);

	virtual void setDocumentLocator(const Locator *const locator);

public:
	const Locator *locator;

// ------- Attributes

	typedef vector< pair<string, string> > attributes_type;
	typedef attributes_type::const_iterator attributes_iterator;

	static const string *GetByNameX(const attributes_type &attributes, const char *name);

	static const string &GetByName(const attributes_type &attributes, const char *name)
	{
		const string *p = GetByNameX(attributes, name);
		const static string nullstr;

		if( p == NULL ) p = &nullstr;
//			HR_THROW(E_INVALID_DTD);

		return *p;
	}

	template<class INTERFACE, class FUNC_INTERFACE>
	void Attr(attributes_iterator i, const char *name, INTERFACE p, 
		HRESULT (__stdcall FUNC_INTERFACE::*func)(BSTR))
	{
		if( i->first == name )
		{
			FUNC_INTERFACE *q = p;
			ASSERT( q != NULL );

			COMTHROW( (q->*func)(PutInBstr(i->second)) );
		}
	}

	static long toLong(string s);
	static unsigned long toULong(string s);

	template<class INTERFACE, class FUNC_INTERFACE>
	void Attr(attributes_iterator i, const char *name, INTERFACE p,
	HRESULT (__stdcall FUNC_INTERFACE::*func)(long))
	{
		if( i->first == name )
		{
			FUNC_INTERFACE *q = p;
			ASSERT( q != NULL );

			COMTHROW( (q->*func)(toLong(i->second)) );
		}
	}

	template<class INTERFACE, class FUNC_INTERFACE>
	void Attr(attributes_iterator i, const char *name, INTERFACE p,
	HRESULT (__stdcall FUNC_INTERFACE::*func)(unsigned long))
	{
		if( i->first == name )
		{
			FUNC_INTERFACE *q = p;
			ASSERT( q != NULL );

			COMTHROW( (q->*func)(toULong(i->second)) );
		}
	}

// ------- ElementFuncs

public:
	CGenParserFunc_Base *elementfuncs;

	void StartNone(const attributes_type &attributes) { }
	void EndNone() { }

// ------- Properties

public:
	string xmlfile;
	CComBstrObj errorinfo;
	long err_line;
	long err_column;

	typedef unsigned long counter_type;
	counter_type counter;		// we do not advance the counter in GenParser

	struct element_type
	{
		string name;
		string chardata;
		CComObjPtr<IUnknown> object;
		string exstrinfo;
		long   exnuminfo;

		counter_type begin;
		counter_type end;
	};

	vector<element_type> elements;

public:
	element_type &GetCurrent() { ASSERT( !elements.empty() ); return elements.back(); }
	const element_type &GetPrevious() const
	{
		ASSERT( elements.size() >= 2 );

		vector<element_type>::const_iterator i = elements.end();
		--i; --i;
		return *i;
	}

	string &GetCurrData() { return GetCurrent().chardata; }
	const string &GetPrevName() const { return GetPrevious().name; }
	
	template<class T>
	void GetPrevObj(CComObjPtr<T> &obj) const
	{
		ASSERT( GetPrevious().object != NULL );
		COMTHROW( GetPrevious().object.QueryInterface(obj) );
		ASSERT( obj != NULL );
	}

	PutInBstr PutInBstrAttr(const attributes_type &attributes, const char *name)
	{
		const string *s = GetByNameX(attributes, name);
		if( s == NULL )
			return "";

		return *s;
	}
};

// --------------------------- CGenParserFunc

struct CGenParserFunc_Base
{
	CGenParserFunc_Base(const char *n) : name(n) { }

	string name;

	virtual void Start(CGenParser *parser, const CGenParser::attributes_type &attributes) = 0;
	virtual void End(CGenParser *parser) = 0;
};

template<class DERIVED>
struct CGenParserFunc : public CGenParserFunc_Base
{
	typedef void (DERIVED::*StartFunc)(const CGenParser::attributes_type &);
	typedef void (DERIVED::*EndFunc)();

	virtual void Start(CGenParser *parser, const CGenParser::attributes_type &attributes)
	{ 
		(static_cast<DERIVED*>(parser)->*start)(attributes); 
	}

	virtual void End(CGenParser *parser)
	{
		(static_cast<DERIVED*>(parser)->*end)();
	}

	CGenParserFunc(const char *n, StartFunc s, EndFunc e) : 
		CGenParserFunc_Base(n), start(s), end(e) { }

	StartFunc start;
	EndFunc end;
};

#endif//MGA_GENPARSER_H
