
#include "stdafx.h"
#include "Parser.h"
#include "MgaDumper.h"
#include "Transcoder.h"


#define FLUSH_LIMIT			1000

// -----------------------------------------------------------
// --------------------------- GmeEqual ----------------------
// -----------------------------------------------------------
GmeEqual::GmeEqual( CComObjPtr<IMgaObject> obj)
: m_objToFind( obj), m_which( 1)
{ }


GmeEqual::GmeEqual( CComObjPtr<IMgaFCO> fco)
: m_fcoToFind( fco), m_which( 2)
{ }


GmeEqual::GmeEqual( CComObjPtr<IMgaFolder> folder)
: m_folderToFind( folder), m_which( 3)
{ }


GmeEqual::GmeEqual( CComObjPtr<IMgaModel> model)
: m_modelToFind( model), m_which( 4)
{ }


bool GmeEqual::operator() ( CComObjPtr<IMgaObject>& op )
{
	ASSERT( m_which == 1); // which variable is set

	VARIANT_BOOL is_equal;
	COMTHROW( m_objToFind->get_IsEqual( op, &is_equal));

	if (is_equal != VARIANT_FALSE)
		return true;

	return false;
}

bool GmeEqual::operator() ( CComObjPtr<IMgaFCO>& op )
{
	ASSERT( m_which == 2); // which variable is set

	VARIANT_BOOL is_equal;
	COMTHROW( m_fcoToFind->get_IsEqual( op, &is_equal));

	if (is_equal != VARIANT_FALSE)
		return true;

	return false;
}

bool GmeEqual::operator() ( CComObjPtr<IMgaFolder>& op )
{
	ASSERT( m_which == 3); // which variable is set

	VARIANT_BOOL is_equal;
	COMTHROW( m_folderToFind->get_IsEqual( op, &is_equal));

	if (is_equal != VARIANT_FALSE)
		return true;

	return false;
}

bool GmeEqual::operator() ( CComObjPtr<IMgaModel>& op )
{
	ASSERT( m_which == 4); // which variable is set

	VARIANT_BOOL is_equal;
	COMTHROW( m_modelToFind->get_IsEqual( op, &is_equal));

	if (is_equal != VARIANT_FALSE)
		return true;

	return false;
}


// --------------------------- CMgaDumper

void CMgaDumper::InitDump(IMgaProject *p, BSTR xmlfile)
{
	if( p == NULL )
		HR_THROW(E_INVALIDARG);

	string filename;
	CopyTo(xmlfile, filename);

	if( filename.empty() )
		HR_THROW(E_INVALIDARG);

	ofs.init( filename.c_str(), "UTF-8");

	elems.clear();

	ASSERT( project == NULL );
	ASSERT( territory == NULL );

	territory = NULL;
	COMTHROW( p->CreateTerritory(NULL, PutOut(territory), NULL) );

	COMTHROW( p->BeginTransaction(territory, TRANSACTION_READ_ONLY) );
	project = p;

	fco_count = 0;
}

void CMgaDumper::DoneDump(bool abort)
{
	ofs.finalize();
	elems.clear();

	if( territory != NULL )
		territory->Destroy();
	territory = NULL;

	if( project != NULL )
	{
		if( abort )
			project->AbortTransaction();
		else
			project->CommitTransaction();
	}
	
	m_selFcos.clear();
	m_selFolders.clear();
	project = NULL;
}

STDMETHODIMP CMgaDumper::DumpProject(IMgaProject *p, BSTR xmlfile)
{
	CHECK_IN(p);
	m_dumpLibraryStubs = false; // make sure closure logic does not involve in it


	COMTRY
	{
		InitDump(p, xmlfile);

		ofs << "<!DOCTYPE project SYSTEM \"mga.dtd\">\n\n";

		Dump(p);

		DoneDump(false);
	}
	COMCATCH( DoneDump(true); )
}

STDMETHODIMP CMgaDumper::DumpFCOs(IMgaFCOs *p, IMgaFolders *f, BSTR xmlfile)
{
	m_strictDump = false; // this method uses the permissive dump
	// meaning that if a model/folder is in the closure then all children
	// and grandchildren are dumped as well

	dumpversion = 2; // no relids dumped
	m_dumpLibraryStubs = false; // make sure closure logic does not involve in it

	//CHECK_IN( (int)p | (int)f);

	COMTRY
	{
		if ( p) CopyTo(p, m_selFcos);
		if ( f) CopyTo(f, m_selFolders);

		if( m_selFcos.empty() && m_selFolders.empty())
			return S_OK;

		CComObjPtr<IMgaProject> project;

		if ( !m_selFcos.empty())
			COMTHROW( m_selFcos.front()->get_Project( PutOut( project)) );
		else if ( !m_selFolders.empty())
			COMTHROW( m_selFolders.front()->get_Project( PutOut( project)) );
		else
			return S_OK;

		InitDump(project, xmlfile);

		ofs << "<!DOCTYPE clipboard SYSTEM \"mga.dtd\" [\n";
		ofs << "\t<!ELEMENT clipboard (folder|model|atom|reference|set|connection)*>\n";
		ofs << "]>\n\n";

		StartElem("clipboard");

		CComObjPtrVector<IMgaFolder>::iterator fi = m_selFolders.begin();
		while( fi != m_selFolders.end() )
		{
			CComObjPtr<IMgaObject> obj;
			COMTHROW(territory->OpenObj(*fi, PutOut(obj)));
			CComObjPtr<IMgaFolder> folder;
			COMTHROW(obj.QueryInterface( folder));
			*fi = folder;
			++fi;
		}

		CComObjPtrVector<IMgaFCO>::iterator i = m_selFcos.begin();
		while( i != m_selFcos.end() )
		{
			// PETER: put the object into the parser's territory
			CComObjPtr<IMgaObject> obj;
			COMTHROW(territory->OpenObj(*i, PutOut(obj)));
			CComObjPtr<IMgaFCO> fco;
			COMTHROW(obj.QueryInterface( fco));
			*i = fco;
			++i;
		}

		removeInnerObjs();
		
		for( fi = m_selFolders.begin(); fi != m_selFolders.end(); ++fi )
			Dump( *fi);

		for( i = m_selFcos.begin(); i != m_selFcos.end(); ++i )
			Dump ( *i);
		

		EndElem();
		DoneDump(false);
	}
	COMCATCH( DoneDump(true); )
}

// ------- Low level stuff

inline void CMgaDumper::Indent(int i)
{
	for(; i > 0; --i)
		ofs << '\t';
}

inline void CMgaDumper::StartElem(const char *name)
{
	ASSERT( name != NULL );

	if( !elems.empty() && !elems.back().inbody )
	{
		ASSERT( !elems.back().indata );

		ofs << ">\n";
		elems.back().inbody = true;
	}

	elems.push_back();
	elems.back().name = name;
	elems.back().inbody = false;
	elems.back().indata = false;

	Indent(elems.size()-1);
	ofs << '<' << name;
}

inline void CMgaDumper::Attr(const char *name, const char *value)
{
	ASSERT( name != NULL );
	ASSERT( value != NULL );

	ASSERT( !elems.empty() && !elems.back().inbody );
	
	ofs << ' ' << name << "=\"" << Transcoder::StdEscape << value << Transcoder::NoEscape << '"';
}

inline void CMgaDumper::Attr(const char *name, const char *value, int len)
{
	ASSERT( name != NULL );
	ASSERT( value != NULL );
	ASSERT( len >= 0 );

	ASSERT( !elems.empty() && !elems.back().inbody && !elems.back().indata );

	ofs << ' ' << name << "=\"" << Transcoder::StdEscape << string( value, len) << Transcoder::NoEscape << '"';
}

bool CMgaDumper::HasMarkup(const char *value, int len)
{
	while( --len >= 0 )
	{
		char c = *(value++);
		if( c == '<' || c == '>' || c == '&' || c == '\'' || c == '\"' )
			return true;
	}

	return false;
}

inline void CMgaDumper::Data(const char *value, int len)
{
	ASSERT( value != NULL );
	ASSERT( len >= 0 );
	ASSERT( !elems.empty() );

	if( !elems.back().inbody )
	{
		ofs << ">";
		elems.back().inbody = true;
	}

	ofs << Transcoder::StdEscape << string( value, len) << Transcoder::NoEscape;

	elems.back().indata = true;

}

inline void CMgaDumper::EndElem()
{
	ASSERT( !elems.empty() );

	if( elems.back().inbody )
	{
		if( !elems.back().indata )
			Indent(elems.size()-1);

		ofs << "</" << elems.back().name << ">\n";
	}
	else
	{
		ofs << "/>\n";
	}

	elems.pop_back();
}

// ------- CheckInClosure
// there are two ways:
//  strict: those objects are dumped which are selected ( can be found in m_selFCO/Folder)
//          this method is used by the copy closure methods
//  permissive: hierarchical selection, meaning that if a folder/model is selected then
//              its children/grandchildren need not to be selected (need not be in m_selFCO/Folder)
//              this is used by regular clipboard methods
//
inline bool CMgaDumper::CheckInClosure(CComObjPtr<IMgaAttribute> ) { return true;}
inline bool CMgaDumper::CheckInClosure(CComObjPtr<IMgaConnPoint> ) { return true;}

bool CMgaDumper::CheckInClosure(CComObjPtr<IMgaFolder> folder)
{
	if ( m_selFolders.empty() && m_selFcos.empty()) {
		// if the selected FCOs and selected Folders values are empty
		// then everything is in the closure 
		return true;
	}

	bool in_closure = false;

	if (m_strictDump)
	{
		in_closure = find_if( m_selFolders.begin(), m_selFolders.end(), GmeEqual( folder)) != m_selFolders.end();
	}
	else
	{
		CComObjPtr<IMgaFolder> curr_folder = folder;
		while ( curr_folder != NULL && !in_closure)
		{
			in_closure = find_if( m_selFolders.begin(), m_selFolders.end(), GmeEqual( curr_folder)) != m_selFolders.end();

			CComObjPtr<IMgaFolder> parent;
			COMTHROW(curr_folder->get_ParentFolder(PutOut( parent)));

			curr_folder = NULL;
			
			if (parent != NULL) {
				curr_folder = parent;
			}
		}
	}
	return in_closure;
}

bool CMgaDumper::CheckInClosure(CComObjPtr<IMgaFCO> fco) 
{
	if ( m_selFolders.empty() && m_selFcos.empty()) {
		// if the selected FCOs and selected Folders values are empty
		// then everything is in the closure 
		return true;
	}

	bool in_closure = false;

	if (m_strictDump)
	{
		in_closure = find_if( m_selFcos.begin(), m_selFcos.end(), GmeEqual( fco)) != m_selFcos.end();
	}
	else
	{

		CComObjPtr<IMgaFCO> curr_fco = fco;
		CComObjPtr<IMgaFCO> last_fco = fco;
		while ( curr_fco != NULL && !in_closure) {
			
			in_closure = find_if( m_selFcos.begin(), m_selFcos.end(), GmeEqual( curr_fco)) != m_selFcos.end();

			CComObjPtr<IMgaModel> parent;
			COMTHROW(curr_fco->get_ParentModel(PutOut(parent)));
			last_fco = curr_fco;
			curr_fco = NULL;
			if (parent != NULL) {
				COMTHROW( ::QueryInterface(parent, curr_fco) );
			}
		}

		if ( !in_closure) // no parent yet (model) in closure, need to check folders
		{
			CComObjPtr<IMgaFolder> par_folder;
			COMTHROW( last_fco->get_ParentFolder(PutOut( par_folder)));

			while ( par_folder != NULL && !in_closure)
			{
				in_closure = find_if( m_selFolders.begin(), m_selFolders.end(), GmeEqual( par_folder)) != m_selFolders.end();

				CComObjPtr<IMgaFolder> parent;
				COMTHROW(par_folder->get_ParentFolder(PutOut( parent)));

				par_folder = NULL;
				
				if (parent != NULL)
					par_folder = parent;
			}
		}
	}

	return in_closure;
}

// ------- Dumpers

void CMgaDumper::Dump(IMgaProject *project)
{
	ASSERT( project != NULL );

	CComVariant variant;
	CComBstrObj bstr;
	GUID guid;

	StartElem("project");

	COMTHROW( project->get_GUID(PutOut(variant)) );
	CopyTo(variant, guid);
	CopyTo(guid, PutOut(bstr));
	Attr("guid", bstr);

	Attr("cdate", project, IMgaProject::get_CreateTime);
	Attr("mdate", project, IMgaProject::get_ChangeTime);
	Attr("version", project, IMgaProject::get_Version);

	CComObjPtr<IMgaMetaProject> metaproject;
	COMTHROW( project->get_RootMeta(PutOut(metaproject)) );
	ASSERT( metaproject != NULL );

	variant.Clear();
	bstr.Empty();
	COMTHROW( metaproject->get_GUID(PutOut(variant)) );
	CopyTo(variant, guid);
	CopyTo(guid, PutOut(bstr));
	Attr("metaguid", bstr);
	Attr("metaversion", metaproject, IMgaMetaProject::get_Version);
	Attr("metaname", metaproject, IMgaMetaProject::get_Name);
	


	StartElem("name");
	Data(project, IMgaProject::get_Name);
	EndElem();

	StartElem("comment");
	Data(project, IMgaProject::get_Comment);
	EndElem();

	StartElem("author");
	Data(project, IMgaProject::get_Author);
	EndElem();

	CComObjPtr<IMgaFolder> folder;
	COMTHROW( project->get_RootFolder(PutOut(folder)) );
	Dump(folder);

	EndElem();
}

void CMgaDumper::Dump(IMgaFolder *folder)
{
	ASSERT( folder != NULL );

	StartElem("folder");

	Attr("id", folder, IMgaFolder::get_ID);
	if(dumpversion >= 1) {
		if ( dumpversion != 2)
		{
			LAttr("relid", folder, IMgaFolder::get_RelID);
			LAttr("childrelidcntr", folder, IMgaFolder::get_ChildRelIDCounter);
		}

		CComBstrObj libname;
		COMTHROW( folder->get_LibraryName(PutOut(libname)) );
		if(libname) Attr("libref",libname);
	}


	CComObjPtr<IMgaMetaFolder> metafolder;
	COMTHROW( folder->get_MetaFolder(PutOut(metafolder)) );
	Attr("kind", metafolder, IMgaMetaFolder::get_Name);

	StartElem("name");
	Data(folder, IMgaFolder::get_Name);
	EndElem();

	DumpConstraints(folder);
	Dump(folder, IMgaFolder::get_ChildFolders);

	// dumping the regnodes
	CComObjPtrVector<IMgaRegNode> v;
	COMTHROW( folder->get_Registry(VARIANT_FALSE, PutOut(v)) );
	CComObjPtrVector<IMgaRegNode>::iterator i = v.begin();
	CComObjPtrVector<IMgaRegNode>::iterator e = v.end();
	
	Sort(&v);
	
	while( i != e )
	{
		Dump(*i);

		++i;
	}

	Dump(folder, IMgaFolder::get_ChildFCOs);

	EndElem();
}

void CMgaDumper::Dump(IMgaRegNode *regnode)
{
	ASSERT( regnode != NULL );

	StartElem("regnode");

	Attr("name", regnode, IMgaRegNode::get_Name);

	VARIANT_BOOL b;
	COMTHROW( regnode->get_Opacity(&b) );

	if( b != VARIANT_FALSE )
		Attr("isopaque", "yes");

	StartElem("value");
	Data(regnode, IMgaRegNode::get_Value);
	EndElem();

	// dumping the subnodes
	CComObjPtrVector<IMgaRegNode> v;
	COMTHROW( regnode->get_SubNodes(VARIANT_FALSE, PutOut(v)) );
	Sort(&v);

	CComObjPtrVector<IMgaRegNode>::iterator i = v.begin();
	CComObjPtrVector<IMgaRegNode>::iterator e = v.end();
	while( i != e )
	{
		Dump(*i);

		++i;
	}

	EndElem();
}

void CMgaDumper::Dump(IMgaObject *obj)
{
	ASSERT( obj != NULL );

	objtype_enum objtype;
	COMTHROW( obj->get_ObjType(&objtype) );

	switch(objtype)
	{
	case OBJTYPE_MODEL:
		{
			CComObjPtr<IMgaModel> p;
			COMTHROW( ::QueryInterface(obj, p) );
			Dump(p);
			break;
		}

	case OBJTYPE_ATOM:
		{
			CComObjPtr<IMgaAtom> p;
			COMTHROW( ::QueryInterface(obj, p) );
			Dump(p);
			break;
		}

	case OBJTYPE_REFERENCE:
		{
			CComObjPtr<IMgaReference> p;
			COMTHROW( ::QueryInterface(obj, p) );
			Dump(p);
			break;
		}

	case OBJTYPE_CONNECTION:
		{
			CComObjPtr<IMgaConnection> p;
			COMTHROW( ::QueryInterface(obj, p) );
			Dump(p);
			break;
		}

	case OBJTYPE_SET:
		{
			CComObjPtr<IMgaSet> p;
			COMTHROW( ::QueryInterface(obj, p) );
			Dump(p);
			break;
		}

	case OBJTYPE_FOLDER:
		{
			CComObjPtr<IMgaFolder> p;
			COMTHROW( ::QueryInterface(obj, p) );
			Dump(p);
			break;
		}

	default:
		HR_THROW(E_INVALID_MGA);
	};
}

void CMgaDumper::DumpFCO(IMgaFCO *fco, bool dump_attrs, bool dump_name, bool dump_elems)
{
	ASSERT( fco != NULL );

	if( ++fco_count > FLUSH_LIMIT )
	{
		ASSERT( territory != NULL );

		COMTHROW( territory->Flush() );
		COMTHROW( project->CommitTransaction() );
		COMTHROW( project->BeginTransaction(territory, TRANSACTION_READ_ONLY) );

		fco_count = 0;
	}

	if( dump_attrs )
	{
		Attr("id", fco, IMgaFCO::get_ID);

		CComObjPtr<IMgaMetaFCO> metafco;
		COMTHROW( fco->get_Meta(PutOut(metafco)) );
		Attr("kind", metafco, IMgaMetaFCO::get_Name);

		CComObjPtr<IMgaMetaRole> role;
		COMTHROW( fco->get_MetaRole(PutOut(role)) );
		if( role != NULL )
			Attr("role", role, IMgaMetaRole::get_Name);

		VARIANT_BOOL prim = VARIANT_TRUE;
		bool lost_basetype = false;
		CComObjPtr<IMgaFCO> derivedfrom;
		COMTHROW( fco->get_DerivedFrom(PutOut(derivedfrom)) );
		if(derivedfrom != NULL )
		{
			if (CheckInClosure(derivedfrom)) {
				Attr("derivedfrom", derivedfrom, IMgaFCO::get_ID);

				VARIANT_BOOL b;

				COMTHROW( fco->get_IsInstance(&b) );
				Attr("isinstance", b != VARIANT_FALSE ? "yes" : "no");

				COMTHROW( fco->get_IsPrimaryDerived(&prim) );
				Attr("isprimary", prim != VARIANT_FALSE ? "yes" : "no");
			}
			// logic added by ZolMol
			else if ( m_dumpLibraryStubs && !isInLibrary( fco) && isInLibrary( derivedfrom)) { //derived from a type defined in a library
				CComBSTR name;
				COMTHROW( derivedfrom->get_AbsPath( &name));
				string nm;
				CopyTo( name, nm);
				
				Attr("closurelibderivedfrom", nm);

				VARIANT_BOOL b;

				COMTHROW( fco->get_IsInstance(&b) );
				Attr("isinstance", b != VARIANT_FALSE ? "yes" : "no");

				COMTHROW( fco->get_IsPrimaryDerived(&prim) );
				Attr("isprimary", prim != VARIANT_FALSE ? "yes" : "no");
			}
			else {
				lost_basetype = true;
			}
		}
		if(prim && (dumpversion >= 1) && (!lost_basetype)) 
			if ( dumpversion != 2)
				LAttr("relid", fco, IMgaFCO::get_RelID);
	}

	if( dump_name )
	{
		StartElem("name");
		Data(fco, IMgaFCO::get_Name);
		EndElem();
	}

	if( dump_elems )
	{
		DumpConstraints(fco);

		// dumping the regnodes
		CComObjPtrVector<IMgaRegNode> v;
		COMTHROW( fco->get_Registry(VARIANT_FALSE, PutOut(v)) );
		CComObjPtrVector<IMgaRegNode>::iterator i = v.begin();
		CComObjPtrVector<IMgaRegNode>::iterator e = v.end();
	
		Sort(&v);

		while( i != e )
		{
			Dump(*i);

			++i;
		}

		Dump(fco, IMgaFCO::get_Attributes);
	}
}

void CMgaDumper::Dump(IMgaModel *model)
{
	ASSERT( model != NULL );

	StartElem("model");

	DumpFCO(model, true, false, false);	
	if(dumpversion >= 1 && dumpversion != 2) LAttr("childrelidcntr", model, IMgaModel::get_ChildRelIDCounter);
	DumpFCO(model, false, true, true);	

	Dump(model, IMgaModel::get_ChildFCOs);


	EndElem();
}

void CMgaDumper::DumpConstraints(IMgaObject *object)
{
	ASSERT( object != NULL );

	CComObjPtrVector<IMgaConstraint> v;

	CComObjPtr<IMgaFCO> fco;
	if( SUCCEEDED(::QueryInterface(object, fco)) )
	{
		COMTHROW( fco->get_Constraints(VARIANT_FALSE, PutOut(v)) );
	}
	else
	{
		CComObjPtr<IMgaFolder> folder;
		COMTHROW( ::QueryInterface(object, folder) );
		COMTHROW( folder->get_Constraints(VARIANT_FALSE, PutOut(v)) );
	}

	CComObjPtrVector<IMgaConstraint>::iterator i = v.begin();
	CComObjPtrVector<IMgaConstraint>::iterator e = v.end();
	while( i != e )
	{
		Dump(*i);

		++i;
	}
}

void CMgaDumper::Dump(IMgaConstraint *constraint)
{
	ASSERT( constraint != NULL );

	StartElem("constraint");

	StartElem("name");
	Data(constraint, IMgaConstraint::get_Name);
	EndElem();

	StartElem("value");
	Data(constraint, IMgaConstraint::get_Expression);
	EndElem();

	EndElem();
}

void CMgaDumper::Dump(IMgaAttribute *attribute)
{
	ASSERT( attribute != NULL );

	StartElem("attribute");
	
	CComObjPtr<IMgaMetaAttribute> metaattr;
	COMTHROW( attribute->get_Meta(PutOut(metaattr)) );
	Attr("kind", metaattr, IMgaMetaAttribute::get_Name);

	long status;
	COMTHROW( attribute->get_Status(&status) );
	if( status < 0 )
		Attr("status", "meta");
	else if( status > 0 )
		Attr("status", "inherited");

	StartElem("value");

	attval_enum attval;
	COMTHROW( metaattr->get_ValueType(&attval) );

	string data;

	switch(attval)
	{
	case ATTVAL_STRING:
	case ATTVAL_ENUM:
	case ATTVAL_DYNAMIC:
		{
			CComBstrObj bstr;
			COMTHROW( attribute->get_StringValue(PutOut(bstr)) );
			CopyTo(bstr, data);
			break;
		}

	case ATTVAL_INTEGER:
		{
			long l;
			COMTHROW( attribute->get_IntValue(&l) );
			Format(data, "%ld", l);
			break;
		}

	case ATTVAL_DOUBLE:
		{
			double d;
			COMTHROW( attribute->get_FloatValue(&d) );
			Format(data, "%g", d);
			break;
		}

	case ATTVAL_BOOLEAN:
		{
			VARIANT_BOOL b;
			COMTHROW( attribute->get_BoolValue(&b) );

			if( b != VARIANT_FALSE )
				data = "true";
			else
				data = "false";

			break;
		}

	case ATTVAL_REFERENCE:
		{
			CComObjPtr<IMgaFCO> fco;
			COMTHROW( attribute->get_FCOValue(PutOut(fco)) );

			if( (fco != NULL) && (CheckInClosure(fco)) )
			{
				CComBstrObj bstr;
				COMTHROW( fco->get_ID(PutOut(bstr)) );
				CopyTo(bstr, data);
			}
			break;
		}

	default:
		HR_THROW(E_INVALID_META);
	};
		
	Data(data);
	EndElem();

//	Dump(attribute, IMgaAttribute::get_Registry);

	EndElem();
}

void CMgaDumper::Dump(IMgaAtom *atom)
{
	ASSERT( atom != NULL );

	StartElem("atom");

	DumpFCO(atom);

	EndElem();
}

void CMgaDumper::Dump(IMgaReference *reference)
{
	ASSERT( reference != NULL );

	StartElem("reference");

	DumpFCO(reference, true, false, false);

	CComObjPtr<IMgaFCO> fco;
	COMTHROW( reference->get_Referred(PutOut(fco)) );
	
	if( fco != NULL)
	{
		if( CheckInClosure( fco)) // regular dump or the fco is really in the closure
			Attr("referred", fco, IMgaFCO::get_ID);
		// logic added by ZolMol
		else if ( m_dumpLibraryStubs && !isInLibrary( reference) && isInLibrary( fco)) //reference pointing out to a library
		{
			CComBSTR name;
			COMTHROW( fco->get_AbsPath( &name));
			string nm;
			CopyTo( name, nm);
			
			Attr("closurelibreferred", nm);
		}
	}


	CComObjPtr<IMgaFCO> base;
	COMTHROW( reference->get_DerivedFrom(PutOut(base)));
	if(base && CheckInClosure(base)) {
		short stat;
		COMTHROW( reference->CompareToBase(&stat));
		if(!stat) {
			Attr("isbound", "yes" );
		}
	}

	DumpFCO(reference, false, true, true);

	EndElem();
}

void CMgaDumper::Dump(IMgaConnection *connection)
{
	ASSERT( connection != NULL );

	StartElem("connection");

	bool skipdump = false;

	CComObjPtr<IMgaFCO> base;
	COMTHROW( connection->get_DerivedFrom(PutOut(base)));
	if(base && CheckInClosure(base)) {
		short stat;
		COMTHROW( connection->CompareToBase(NULL, &stat));
		if(!stat) {
			Attr("isbound", "yes" );
			skipdump = true;
		}
	}

	DumpFCO(connection);

	if(!skipdump) Dump(connection, IMgaConnection::get_ConnPoints);

	EndElem();
}

void CMgaDumper::Dump(IMgaSet *set)
{
	ASSERT( set != NULL );

	StartElem("set");

	DumpFCO(set, true, false, false);
	bool skipdump = false;
	CComObjPtr<IMgaFCO> base;
	COMTHROW( set->get_DerivedFrom(PutOut(base)));
	if(base && CheckInClosure(base)) {
		short stat;
		COMTHROW( set->CompareToBase(&stat));
		if(!stat) {
			Attr("isbound", "yes" );
			skipdump = true;
		}
	}


	if(!skipdump) {
		CComObjPtrVector<IMgaFCO> members;
		COMTHROW( set->get_Members(PutOut(members)) );
		DumpIDRefs("members", members);
	}

	DumpFCO(set, false, true, true);

	EndElem();
}

void CMgaDumper::Dump(IMgaConnPoint *connpoint)
{
	ASSERT( connpoint != NULL );

	StartElem("connpoint");

	Attr("role", connpoint, IMgaConnPoint::get_ConnRole);
	
	CComObjPtr<IMgaFCO> target;
	COMTHROW( connpoint->get_Target(PutOut(target)) );

	if( target == NULL )
		HR_THROW(E_INVALID_MGA);


	CComPtr<IMgaConnection> conn;
	COMTHROW( connpoint->get_Owner(&conn));

	CComObjPtr<IMgaFCO> base;
	COMTHROW( conn->get_DerivedFrom(PutOut(base)));
	if(base && CheckInClosure(base)) {
		short stat;
		COMTHROW( conn->CompareToBase(connpoint, &stat));
		if(!stat) {
			Attr("isbound", "yes" );
		}
	}

	// m_dumpLibraryStubs is true, so the connection to a library element will be dumped with "closurelibtarget" attribute and a dummy! "target" attribute
	if ( m_dumpLibraryStubs && !isInLibrary( conn) && isInLibrary( target))
	{
		CComBSTR name;
		COMTHROW( target->get_AbsPath( &name));
		string nm;
		CopyTo( name, nm);
		
		Attr("closurelibtarget", nm);
		Attr("target", conn, IMgaFCO::get_ID); // this is required, dump a dummy!! value, since the ID is checked and if not found inside the <clipboard/> section the parser will complain about it
	}
	else
		Attr("target", target, IMgaFCO::get_ID);

	CComObjPtrVector<IMgaFCO> refs;
	COMTHROW( connpoint->get_References(PutOut(refs)) );
	DumpIDRefs("refs", refs);

	EndElem();
}

void CMgaDumper::DumpIDRefs(const char *name, CComObjPtrVector<IMgaFCO> &fcos)
{
	if( !fcos.empty() )
	{
		string idrefs;

		CComObjPtrVector<IMgaFCO>::iterator i = fcos.begin();
		CComObjPtrVector<IMgaFCO>::iterator e = fcos.end();
		while( i != e )
		{
			if (!CheckInClosure(*i)) {
				++i; // inserted by ZolMol
				continue;
			}
			CComBstrObj bstr;
			COMTHROW( (*i)->get_ID(PutOut(bstr)) );

			string id;
			CopyTo(bstr, id);

			if( !idrefs.empty() )
				idrefs += ' ';

			idrefs += id;

			++i;
		}

		if ( !idrefs.empty())
			Attr(name, idrefs);
	}
}

//sort the RegNode Vector in place by Name
void CMgaDumper::Sort(CComObjPtrVector<IMgaRegNode> *v)
{
	int n = v->size();

	//small speed up, build a vector of strings first, then sort the strings
	//while doing the sort operations on the original vector, so we don't have
	//to call the get_Name function each time we look at a RegNode
	vector<string> vPrime;
	for (int i=0; i<n; i++)
	{
		CComBSTR bstr;
		COMTHROW((*v)[i]->get_Name(&bstr));
		string s;
		CopyTo(bstr,s);
		vPrime.push_back(s);
	}
	
	//in place insertion sort
	for (int x=1; x<n; x++) 
	{
		string index = vPrime[x];
		CComObjPtr<IMgaRegNode> indexRegNode = (*v)[x];
		int y = x;

		while ((y>0)&&(vPrime[y-1].compare(index)>0))
		{
				vPrime[y]=vPrime[y-1];

				(*v)[y]=(*v)[y-1];

				--y;
		}

		vPrime[y] = index;
		(*v)[y] = indexRegNode;
	}		

}

//Sort the Attribute Vector by Kind
void CMgaDumper::Sort(CComObjPtrVector<IMgaAttribute> *v)
{
	int n = v->size();

	//small speed up, build a vector of strings first, then sort the strings
	//while doing the sort operations on the original vector, so we don't have
	//to get the Kind Name every time
	vector<string> vPrime;
	for (int i=0; i<n; i++)
	{
		CComBSTR bstr;
		CComPtr<IMgaMetaAttribute> meta;
		COMTHROW((*v)[i]->get_Meta(&meta));
		COMTHROW(meta->get_Name(&bstr));
		string s;
		CopyTo(bstr,s);
		vPrime.push_back(s);
	}
	
	//in place insertion sort
	for (int x=1; x<n; x++) 
	{
		string index = vPrime[x];
		CComObjPtr<IMgaAttribute> indexAttr = (*v)[x];
		int y = x;

		while ((y>0)&&(vPrime[y-1].compare(index)>0))
		{
				vPrime[y]=vPrime[y-1];

				(*v)[y]=(*v)[y-1];

				--y;
		}

		vPrime[y] = index;
		(*v)[y] = indexAttr;
	}	
}

//Sort the Connection Point Vector by Target ID
void CMgaDumper::Sort(CComObjPtrVector<IMgaConnPoint> *v)
{
	int n = v->size();

	//small speed up, build a vector of strings first, then sort the strings
	//while doing the sort operations on the original vector, so we don't have
	//to get the Target ID every time
	vector<string> vPrime;
	for (int i=0; i<n; i++)
	{
		CComBSTR bstr;
		CComPtr<IMgaFCO> fco;
		COMTHROW((*v)[i]->get_Target(&fco));
		COMTHROW(fco->get_ID(&bstr));
		string s;
		CopyTo(bstr,s);
		vPrime.push_back(s);
	}
	
	//in place insertion sort
	for (int x=1; x<n; x++) 
	{
		string index = vPrime[x];
		CComObjPtr<IMgaConnPoint> indexConnPoint = (*v)[x];
		int y = x;

		while ((y>0)&&(vPrime[y-1].compare(index)>0))
		{
				vPrime[y]=vPrime[y-1];

				(*v)[y]=(*v)[y-1];

				--y;
		}

		vPrime[y] = index;
		(*v)[y] = indexConnPoint;
	}	
}

//sort the Folder Vector by ID
void CMgaDumper::Sort(CComObjPtrVector<IMgaFolder> *v)
{
	int n = v->size();
	
	//small speed up, build a vector of strings first, then sort the strings
	//while doing the sort operations on the original vector, so we don't have
	//to call the get_ID function each time we look at a Folder
	vector<string> vPrime;
	for (int i=0; i<n; i++)
	{
		CComBSTR bstr;
		COMTHROW((*v)[i]->get_ID(&bstr));
		string s;
		CopyTo(bstr,s);
		vPrime.push_back(s);
	}	
	
	//in place insertion sort
	for (int x=1; x<n; x++) 
	{
		string index = vPrime[x];
		CComObjPtr<IMgaFolder> indexFolder = (*v)[x];
		int y = x;

		while ((y>0)&&(vPrime[y-1].compare(index)>0))
		{
				vPrime[y]=vPrime[y-1];

				(*v)[y]=(*v)[y-1];

				--y;
		}

		vPrime[y] = index;
		(*v)[y] = indexFolder;
	}	
	
}

//Sort the FCO vector in place by ID
void CMgaDumper::Sort(CComObjPtrVector<IMgaFCO> *v)
{
	int n = v->size();
	
	//small speed up, build a vector of strings first, then sort the strings
	//while doing the sort operations on the original vector, so we don't have
	//to call the get_ID function each time we look at an FCO
	vector<string> vPrime;
	for (int i=0; i<n; i++)
	{
		CComBSTR bstr;
		COMTHROW((*v)[i]->get_ID(&bstr));
		string s;
		CopyTo(bstr,s);
		vPrime.push_back(s);
	}

	//in place insertion sort
	for (int x=1; x<n; x++) 
	{
		string index = vPrime[x];
		CComObjPtr<IMgaFCO> indexFCO = (*v)[x];
		int y = x;

		while ((y>0)&&(vPrime[y-1].compare(index)>0))
		{
				vPrime[y]=vPrime[y-1];

				(*v)[y]=(*v)[y-1];

				--y;
		}

		vPrime[y] = index;
		(*v)[y] = indexFCO;
	}	

}

void CMgaDumper::putInTerritory( CComObjPtrVector<IMgaFCO>& fco_vec)
{		
	CComObjPtrVector<IMgaFCO>::iterator i = fco_vec.begin();
	for( ; i != fco_vec.end(); ++i )
	{
		CComObjPtr<IMgaObject> obj;
		COMTHROW(territory->OpenObj(*i, PutOut(obj)));
		CComObjPtr<IMgaFCO> fco;
		COMTHROW(obj.QueryInterface(fco));
		*i = fco;
	}
}

void CMgaDumper::putInTerritory( CComObjPtrVector<IMgaFolder>& fold_vec)
{
	CComObjPtrVector<IMgaFolder>::iterator j = fold_vec.begin();
	for( ; j != fold_vec.end(); ++j )
	{
		CComObjPtr<IMgaObject> obj;
		COMTHROW(territory->OpenObj(*j, PutOut(obj)));
		CComObjPtr<IMgaFolder> fold;
		COMTHROW(obj.QueryInterface(fold));
		*j = fold;
	}
}

void CMgaDumper::removeInnerObjs()
{
	// try to remove those objects which are part of (child, grandchild) of another selected object
	CComObjPtrVector<IMgaFCO>::iterator i = m_selFcos.begin();
	while( i != m_selFcos.end())
	{
		CComObjPtr<IMgaFCO> last_fco = *i, curr_fco = *i;
		bool found_parent = false;
		do
		{
			CComObjPtr<IMgaModel> parent;
			COMTHROW( curr_fco->get_ParentModel(PutOut(parent)));
			last_fco = curr_fco;
			curr_fco = NULL;
			if (parent != NULL) {
				COMTHROW( ::QueryInterface(parent, curr_fco) );
				found_parent = find_if( m_selFcos.begin(), m_selFcos.end(), GmeEqual( curr_fco)) != m_selFcos.end();
			}
		} while ( curr_fco != NULL && !found_parent);

		if ( !found_parent) // no parent found in the set of fcos
		{
			CComObjPtr<IMgaFolder> par_folder;
			COMTHROW( last_fco->get_ParentFolder(PutOut( par_folder)));

			while ( par_folder != NULL && !found_parent)
			{
				found_parent = find_if( m_selFolders.begin(), m_selFolders.end(), GmeEqual( par_folder)) != m_selFolders.end();

				CComObjPtr<IMgaFolder> parent;
				COMTHROW( par_folder->get_ParentFolder(PutOut( parent)));

				par_folder = parent;
			}
		}

		if ( found_parent)
			i = m_selFcos.erase( i);
		else
			++i;
	}	


	// try to remove those folders which are part of (child, grandchild) of another selected folder
	CComObjPtrVector<IMgaFolder>::iterator k = m_selFolders.begin();
	while( k != m_selFolders.end())
	{
		CComObjPtr<IMgaFolder> par_folder;
		COMTHROW( (*k)->get_ParentFolder(PutOut( par_folder)));
		bool found_parent = false;
		
		while ( par_folder != NULL && !found_parent)
		{
			found_parent = find_if( m_selFolders.begin(), m_selFolders.end(), GmeEqual( par_folder)) != m_selFolders.end();

			CComObjPtr<IMgaFolder> parent;
			COMTHROW(par_folder->get_ParentFolder(PutOut( parent)));
			par_folder = parent;
		}
		
		if ( found_parent)
			k = m_selFolders.erase( k);
		else
			++k;
	}

	ASSERT( m_selFolders.size() + m_selFcos.size());
}

STDMETHODIMP CMgaDumper::DumpClos( IMgaFCOs *p_sel_fcos, IMgaFolders *p_sel_folds,  BSTR xmlfile, int dump_options)
{
	m_strictDump = true; // strict dump meaning that all dumped objects must be strictly in the closure
	dumpversion = 2; // no relids dumped
	m_dumpLibraryStubs = (dump_options & 0x1) == 0x1; // dump library stubs ( absolute path used as libreferred or libderivedfrom attribute)

	COMTRY
	{
		if ( p_sel_fcos )
			CopyTo( p_sel_fcos, m_selFcos);

		if ( p_sel_folds) 
			CopyTo( p_sel_folds, m_selFolders);

		CComObjPtr<IMgaProject> project;

		if ( !m_selFcos.empty())
			COMTHROW( m_selFcos.front()->get_Project( PutOut( project)) );
		else if ( !m_selFolders.empty())
			COMTHROW( m_selFolders.front()->get_Project( PutOut( project)) );
		else
			return S_OK;


		InitDump( project, xmlfile);

		putInTerritory( m_selFcos);
		putInTerritory( m_selFolders);
		
		if ( false) // clipboard format
		{
			ofs << "<!DOCTYPE clipboard SYSTEM \"mga.dtd\" [\n";
			ofs << "\t<!ELEMENT clipboard (folder|model|atom|reference|set|connection)*>\n";
			ofs << "]>\n\n";

			StartElem("clipboard");
			Dump( project); 
			EndElem();
		}
		else
		{
			ofs << "<!DOCTYPE project SYSTEM \"mga.dtd\">\n\n";

			Dump( project); 
		}
		DoneDump(false);
	}
	COMCATCH( DoneDump(true); )

	m_selFcos.clear();
	m_selFolders.clear();
}

STDMETHODIMP CMgaDumper::DumpClosR(
			IMgaFCOs *p_sel_fcos, IMgaFolders *p_sel_folds, BSTR xmlfile, 
			IMgaFCOs *p_top_fcos, IMgaFolders *p_top_folds, int dump_options)
{
	m_strictDump = true; // strict dump meaning that all dumped objects must be strictly in the closure
	dumpversion = 2; // no relids dumped
	m_dumpLibraryStubs = (dump_options & 0x1) == 0x1; // dump library stubs ( absolute path used as closurelibreferred, closurelibderivedfrom, closurelibtarget attribute)

	COMTRY
	{
		if ( p_sel_fcos )
			CopyTo( p_sel_fcos, m_selFcos);

		if ( p_sel_folds) 
			CopyTo( p_sel_folds, m_selFolders);

		CComObjPtr<IMgaProject> project;

		if ( !m_selFcos.empty())
			COMTHROW( m_selFcos.front()->get_Project( PutOut( project)) );
		else if ( !m_selFolders.empty())
			COMTHROW( m_selFolders.front()->get_Project( PutOut( project)) );
		else
			return S_OK;


		CComObjPtrVector<IMgaFCO>	parentless_fcos; // parentless fcos from the selected set
		CComObjPtrVector<IMgaFolder> parentless_folders; // parentless folders from the selected set

		if ( p_top_fcos )
			CopyTo( p_top_fcos, parentless_fcos);

		if ( p_top_folds) 
			CopyTo( p_top_folds, parentless_folders);

		if ( parentless_fcos.empty() && parentless_folders.empty())
		{
			// this case should be handled by the DumpClos()
			ASSERT( 0);
			return DumpClos( p_sel_fcos, p_sel_folds, xmlfile, dump_options);
		}

		InitDump( project, xmlfile);

		putInTerritory( m_selFcos);
		putInTerritory( parentless_fcos);
		putInTerritory( m_selFolders);
		putInTerritory( parentless_folders);

		ofs << "<!DOCTYPE clipboard SYSTEM \"mga.dtd\" [\n";
		ofs << "\t<!ELEMENT clipboard (folder|model|atom|reference|set|connection)*>\n";
		ofs << "]>\n\n";

		StartElem("clipboard");

		for( CComObjPtrVector<IMgaFolder>::iterator j = parentless_folders.begin(); j != parentless_folders.end(); ++j )
			Dump( *j); 

		for( CComObjPtrVector<IMgaFCO>::iterator i = parentless_fcos.begin() ; i != parentless_fcos.end(); ++i )
			Dump( *i); 
		
		EndElem();

		DoneDump(false);
	}
	COMCATCH( DoneDump(true); )

	m_selFcos.clear();
	m_selFolders.clear();
}
