
#ifndef MGA_MGAPARSER_H_
#define MGA_MGAPARSER_H_

#ifndef MGA_GENPARSER_H
#include "GenParser.h"
#endif

#include "resource.h"
#include "CommonVersionInfo.h"
#include <hash_map>
#include <list>

// --------------------------- CMgaParser

class ATL_NO_VTABLE CMgaParser : 
	public CComObjectRootEx<CComSingleThreadModel>,
	public CComCoClass<CMgaParser, &CLSID_MgaParser>,
	public ISupportErrorInfoImpl<&IID_IMgaParser>,
	public IDispatchImpl<IMgaParser, &IID_IMgaParser, &LIBID_PARSERLib>,
	public IMgaVersionInfoImpl,
	public CGenParser
{
public:
	CMgaParser() { }

DECLARE_REGISTRY_RESOURCEID(IDR_MGAPARSER)
DECLARE_PROTECT_FINAL_CONSTRUCT()

BEGIN_COM_MAP(CMgaParser)
	COM_INTERFACE_ENTRY(IMgaParser)
	COM_INTERFACE_ENTRY(IDispatch)
	COM_INTERFACE_ENTRY(ISupportErrorInfo)
	COM_INTERFACE_ENTRY_IID(IID_IMgaVersionInfo, IMgaVersionInfoImpl)
END_COM_MAP()

// ------- Methods

public:
	STDMETHOD(GetXMLInfo)(BSTR filename, BSTR *paradigm, BSTR* parversion, VARIANT *parguid, BSTR* basename, BSTR* version);
	STDMETHOD(ParseFCOs)(IMgaObject *p, BSTR filename);
	STDMETHOD(ParseProject)(IMgaProject *p, BSTR filename);

	void CloseAll();

// ------- Passes

public:
	virtual void startElement(const XMLCh* const name, AttributeList& attributes);
    virtual void endElement(const XMLCh* const name);

	struct range_type
	{
		counter_type begin;
		counter_type end;

		element_type previous;
	};

	list<range_type> ranges;

	int skip_element_level;
	class pass_exception : public exception
	{
	};

	typedef struct librecord {
		string libname;
		CComPtr<IMgaFolder> f;
	} librecord;

	typedef vector<librecord> librecords;
	librecords libstodo;
	CComVariant projectguid;

// ------- Logging

public:

// ------- Attributes

public:
	typedef CGenParserFunc<CMgaParser> elementfunc;

	static elementfunc elementfuncs_mga[];
	static elementfunc elementfuncs_mgainfo[];

	CComObjPtr<IMgaProject> project;
	CComObjPtr<IMgaTerritory> territory;
	CComObjPtr<IMgaResolver> resolver;
	CComObjPtr<IMgaProgressDlg> progress;

	BSTR *infoparname;
	BSTR *infoparversion;
	VARIANT *infoparguid;
	BSTR *infoprojname;
	BSTR *infoversion;

	long project_prefs, project_prefs_orig;

	string constraint_name;
	string constraint_value;

	counter_type max_counter;
	int pass_count;

// ------- Lookup

public:
	void LookupByID(const string &id, CComObjPtr<IMgaObject> &object);
	void LookupByID(const string &id, CComObjPtr<IMgaFCO> &fco);
	void RegisterLookup(const string &id, IMgaObject *object);
	void RegisterLookup(const attributes_type &attributes, IMgaObject *object);

	typedef hash_map<string, CComBstrObj> id_lookup_type;
	typedef id_lookup_type::iterator id_lookup_iterator;

	id_lookup_type id_lookup;

// ------- Derivation

public:
	struct deriv_type
	{
		CComObjPtr<IMgaFCO> from;
		VARIANT_BOOL isinstance;
		bool isprimary;
	};

	void ResolveDerivation(const attributes_type &attributes, deriv_type &deriv);


	bool manual_relid_mode;
	long relid;

	void preparerelid(const attributes_type &attributes);
	void assignrelid(IMgaObject *obj);	
// ------- Element Handlers
	
	public:
	void StartProject(const attributes_type &attributes);
	void EndName();
	void EndComment();
	void EndAuthor();
	void EndValue();
	void StartFolder(const attributes_type &attributes);
	void StartModel(const attributes_type &attributes);
	void StartAtom(const attributes_type &attributes);
	void StartRegNode(const attributes_type &attributes);
	void StartAttribute(const attributes_type &attributes);
	void StartConnection(const attributes_type &attributes);
	void StartConnPoint(const attributes_type &attributes);
	void EndConstraint();
	void StartReference(const attributes_type &attributes);
	void StartSet(const attributes_type &attributes);
	void StartClipboard(const attributes_type &attributes);

	void StartProjectInfo(const attributes_type &attributes);
	void EndNameInfo();
};

#endif //__MGAPARSER_H_
