#ifndef ANY_H
#define ANY_H

#include "BON.h"
#include "BONImpl.h"

#include "Any.h"
#include "string"
#include "vector"
#include "logger.h"
class Method;
/** This class is the abstract base of all kinds of parts/FCOs that can occur during a modeling process. */
class Any;

class AnyLexicographicSort
{
public:
  bool operator()( Any * op1, Any * op2) const;
};

class Any 
{
public: // constant strings
	static const std::string InRootFolder_str;//"InRootFolder"
	static const std::string NameSelectorNode_str; //"myNameIs"
public: // types
	typedef enum
	{
		ATOM,
		MODEL,
		CONN,
		SET,
		REF,
		FCO_REP, // abstract term only
		ASPECT,
		FOLDER,
		CONSTRAINT,
		CONSTRAINTFUNC,
		ATTRIBUTE
	} KIND_TYPE;

	const static std::string KIND_TYPE_STR[];/* =
		{
		"ATOM",	// defined in Any.cpp If the order changes update there please
		"MODEL",
		"CONN",
		"SET",
		"REF",
		"FCO",
		"ASPECT",
		"FOLDER",
		"CONSTRAINT",
		"CONSTRAINFUNC",
		"ATTRIBUTE"
		};*/

public:
	Any( BON::FCO& ptr);
	virtual ~Any();

public:	
	virtual void initAttributes();
	virtual KIND_TYPE getMyKind() const = 0;
	std::string getMyKindStr() const;
	virtual std::string doDump() = 0;
	virtual std::string getName() const;
	virtual std::string getDispName() const;


	BON::FCO getPtr() const { return m_ptr; }
	bool isInRootFolder();

	BON::Folder getParentFolder() const { return m_parentFolder; }
	void setParentFolder( const BON::Folder& folder) { m_parentFolder = folder; }
	void setEquivPeers( const std::set< BON::FCO >& ps) { m_equivs = ps; } 

	std::string getMyPrefix( int which = 0) const;
	BON::RegistryNode getMyRegistry() const;
	BON::RegistryNode getMIRegistry() const;
	bool isFCO() const;

	static void convertToValidName( std::string & p);
	static bool checkIfValidName( const std::string & p);

	virtual void prepare();
	virtual void prepareMacros();
	virtual void prepareIniFin();

	void dumpGlobals();
	std::string dumpOrnament( bool is_abstract = false);
	std::string getDispNameImpl();

	virtual void dumpPre( std::string & h_file, std::string & c_file);
	virtual void dumpPost( std::string & h_file, std::string & c_file);
	virtual std::string dumpClassHeader();

	std::string getUserPart();

	bool isToBeEx() const;
	void toBeEx( bool t);

public:
	static const std::string m_startUPToken;
	static const std::string m_endUPToken;

protected:
	// pointer of the BON object
	BON::FCO m_ptr;

	// pointer of the parent folder
	BON::Folder m_parentFolder;

	// equivalent peers
	std::set< BON::FCO > m_equivs;
	bool	m_isInRootFolder;

	// indicates if it is to be extended or not
	bool m_toEx;

	std::vector< Method > m_inifinMethods;
	std::string m_globalHeader;
	std::string m_globalSource;

private: // forbiding copy
	Any( const Any&);
	const Any& operator=( const Any&);
};

#endif //ANY_H
