#include "stdafx.h"

#include "Dumper.h"
#include "MakeVisitor.h"

#include "algorithm"
#include <afxdlgs.h>

#include "globals.h"
extern Globals global_vars;

int ind; // indent

#include "SelConf.h"
#include "OptionsDlg.h"

bool Dumper::m_bParsePrev = false;
bool Dumper::m_bGenInit = false;
bool Dumper::m_bGenFinalize = false;
bool Dumper::m_bGenTemplates = false;
bool Dumper::m_bGenTemplVersion6 = true;
bool Dumper::m_bGenRegular = true;
bool Dumper::m_bGenVisitor = false;
bool Dumper::m_bSafetyBackup = true;



Sheet * Dumper::getInstance()
{ 
	if (!Sheet::m_theOnlyInstance) 
	{
		Sheet::m_theOnlyInstance = new Dumper();
		return Sheet::m_theOnlyInstance;
	}
	return Sheet::m_theOnlyInstance;
}


Dumper::Dumper(): Sheet() 
{ 
}


Dumper::~Dumper()
{ 
}


bool Dumper::findInFolders( FCO * fco)
{
	std::vector<FolderRep*>::iterator it( m_folderList.begin());
	while( it != m_folderList.end() && !(*it)->isFCOContained( fco))
		++it;
	return it != m_folderList.end();
}


void Dumper::inheritMoReSeCoFolAsp()
{
	// task 1
	ModelRep_Iterator mod_iter( m_modelList.begin());
	for( ; mod_iter != m_modelList.end(); ++mod_iter)
		(*mod_iter)->inherit();

	// task 2
	// the refs first because the connections need the refs !
	ReferenceRep_Iterator ref_iter( m_refList.begin());
	for( ; ref_iter != m_refList.end(); ++ref_iter)
		(*ref_iter)->inherit();

	ref_iter = m_refList.begin();
	for( ; ref_iter != m_refList.end(); ++ref_iter)
		if (!(*ref_iter)->finalize())
		{
			//global_vars.err << (*ref_iter)->getName() << " is referring to too many kinds\n";
		}

	// task 3
	ConnectionRep_Iterator conn_iter( m_connList.begin());
	for( ; conn_iter != m_connList.end(); ++conn_iter)
		(*conn_iter)->inherit();

	// task 4 - for all sets on the sheet
	SetRep_Iterator set_iter( m_setList.begin());
	for( ; set_iter != m_setList.end(); ++set_iter)
		(*set_iter)->extendMembership();

	// task 5 folder element hierarchies
	std::vector<FolderRep*>::iterator folder_iter( m_folderList.begin());
	for( ; folder_iter != m_folderList.end(); ++folder_iter)
		(*folder_iter)->extendMembership();
}


std::string Dumper::getNamespaceName()
{
	std::string t = m_projName + "_BON";
	Any::convertToValidName( t);
	return t;
}


void Dumper::createMethods()
{
	std::string t = m_projName;
	Any::convertToValidName( t);

	MakeVisitor m_visitor( t);

	// connections
	{std::vector<ConnectionRep*>::const_iterator it_cn = m_connList.begin();
	if ( it_cn != m_connList.end())
	{
		for( ; it_cn != m_connList.end(); ++it_cn)
			(*it_cn)->calcLCD();
		for( it_cn = m_connList.begin() ; it_cn != m_connList.end(); ++it_cn)
		{
			if ( (*it_cn)->isToBeEx())
			{
				(*it_cn)->createEndGetters();
				(*it_cn)->prepare();

				//if ( !(*it_cn)->isAbstract())
					m_visitor.addEntity( Any::CONN, (*it_cn)->getDispName());
			}
		}
	}}

	// folders
	{
	std::vector<FolderRep*>::const_iterator it_b = m_folderList.begin();
	if ( it_b != m_folderList.end())
		for( ; it_b != m_folderList.end(); ++it_b)
		{
			if ( (*it_b)->isToBeEx())
			{
				(*it_b)->prepare();
				(*it_b)->createMethods();
				
				m_visitor.addEntity( Any::FOLDER, (*it_b)->getDispName());
			}
		}
	}
	// fcos
	{
	std::vector<FcoRep*>::const_iterator it_b = m_fcoRepList.begin();
	if ( it_b != m_fcoRepList.end())
		for( ; it_b != m_fcoRepList.end(); ++it_b)
		{
			if ( (*it_b)->isToBeEx())
			{
				(*it_b)->prepare();

				//if ( !(*it_b)->isAbstract())
					m_visitor.addEntity( Any::FCO_REP, (*it_b)->getDispName());
			}
		}
	}

	// atoms
	{
	std::vector<AtomRep*>::const_iterator it_b = m_atomList.begin();
	if ( it_b != m_atomList.end())
		for( ; it_b != m_atomList.end(); ++it_b)
		{
			if ( (*it_b)->isToBeEx())
			{

				(*it_b)->prepare();

				//if ( !(*it_b)->isAbstract())
					m_visitor.addEntity( Any::ATOM, (*it_b)->getDispName());
			}
		}
	}

	// sets
	{
	std::vector<SetRep*>::const_iterator it_c = m_setList.begin();
	if ( it_c != m_setList.end())
		for( ; it_c != m_setList.end(); ++it_c)
		{
			if ( (*it_c)->isToBeEx())
			{
				(*it_c)->prepare();
				(*it_c)->createMethods();

				//if ( !(*it_c)->isAbstract())
					m_visitor.addEntity( Any::SET, (*it_c)->getDispName());
			}
		}
	}

	// references
	{
	std::vector<ReferenceRep*>::const_iterator it_ref = m_refList.begin();
	if ( it_ref != m_refList.end())
		for( ; it_ref != m_refList.end(); ++it_ref)
		{
			if ( (*it_ref)->isToBeEx())
			{
				(*it_ref)->prepare();
				(*it_ref)->createMethods();

				//if ( !(*it_ref)->isAbstract())
					m_visitor.addEntity( Any::REF, (*it_ref)->getDispName());
			}
		}
	}

	// models
	{
	std::vector<ModelRep*>::const_iterator it_m = m_modelList.begin();
	if ( it_m != m_modelList.end())
		for( ; it_m != m_modelList.end(); ++it_m)
		{
			if ( (*it_m)->isToBeEx())
			{
				(*it_m)->prepare();
				(*it_m)->createMethods();

				//if ( !(*it_m)->isAbstract())
					m_visitor.addEntity( Any::MODEL, (*it_m)->getDispName());
			}
		}
	}

	if ( m_bGenVisitor)
	{
		std::ofstream visitorh( global_vars.m_visitorHeaderFileName.c_str());
		std::ofstream visitors( global_vars.m_visitorSourceFileName.c_str());

		std::string mmm = m_visitor.dumpVisitorHeader();
		visitorh.write( mmm.c_str(), mmm.length());

		std::string nnn = m_visitor.dumpVisitorSource();
		visitors.write( nnn.c_str(), nnn.length());

		visitorh.close();
		visitors.close();
	}
}


void Dumper::dumpGlobals( std::vector<FCO*>& s)
{
	{
	std::vector<FolderRep*>::const_iterator it_b = m_folderList.begin();
	for( ; it_b != m_folderList.end(); ++it_b)
		if ( (*it_b)->isToBeEx())
			(*it_b)->dumpGlobals();
	}

	std::vector<FCO*>::iterator it = s.begin();
	for( ; it != s.end(); ++it)
	{
		if ( (*it)->isToBeEx())
			(*it)->dumpGlobals();
	}
}


void Dumper::dumpMain( std::vector<FCO*>& s)
{
	// folders
	{
	std::vector<FolderRep*>::const_iterator it_b = m_folderList.begin();
	if ( it_b != m_folderList.end())
		for( ; it_b != m_folderList.end(); ++it_b)
			if ( (*it_b)->isToBeEx())
				(*it_b)->doDump();
	}
	// all other
	std::vector<FCO*>::iterator it = s.begin();
	for( ; it != s.end(); ++it)
	{
		if ( (*it)->isToBeEx())
			(*it)->doDump();
	}
}


void Dumper::doDump()
{
	AnyLexicographicSort lex;
	std::sort( m_folderList.begin(),  m_folderList.end(), lex );
	std::sort( m_fcoRepList.begin(),  m_fcoRepList.end(), lex );
	std::sort( m_atomList.begin(), m_atomList.end(), lex );
	std::sort( m_refList.begin(), m_refList.end(), lex );
	std::sort( m_connList.begin(), m_connList.end(), lex );
	std::sort( m_setList.begin(), m_setList.end(), lex );
	std::sort( m_modelList.begin(), m_modelList.end(), lex );
	// attributes sort by FCO::prepare

	std::vector<FCO*> sorted = sortBasedOnLevels();

	std::string short_name_header_file = shortFileName( global_vars.header_file_name);
	std::string capitalized = capitalizeString( cutExtension( short_name_header_file));

	std::string global_user_part1, global_user_part2;
	Dumper::getGlobalUserParts( global_user_part1, global_user_part2);
	
	DMP_S("#include \"stdafx.h\"\n");
	DMP_S("#include \"" + short_name_header_file + "\"\n\n");
	//DMP_S("using namespace BON;\n\n");
	
	DMP_H("#ifndef " + capitalized + "_H\n");
	DMP_H("#define " + capitalized + "_H\n\n");
	DMP_H("#include \"BON.h\"\n#include \"BONImpl.h\"\n#include \"Extensions.h\"\n\n");

	if ( m_bGenTemplates)
	{
		DMP_H("//\n");
		DMP_H("// this dummy template parameter is needed by VC6 to deduct the return type\n");
		DMP_H("// VC7 users can modify this macro to a value like \"int _dummy = 0\"\n");
		DMP_H("// TDP standing for TEMPLATE DUMMY PAR\n");

		if ( m_bGenTemplVersion6)
		{
			DMP_H("//#define TDP int _dummy = 0			// recommended for VC7 users\n");
			DMP_H("#define TDP const T & _dummyComp		// recommended for VC6 users\n");
		}
		else
		{
			DMP_H("#define TDP int _dummy = 0			// recommended for VC7 users\n");
			DMP_H("//#define TDP const T & _dummyComp		// recommended for VC6 users\n");
		}

		DMP_H("\n");
	}

	DMP_H( Any::m_startUPToken + "\n" + global_user_part1 + Any::m_endUPToken + "\n\n");

	global_vars.m_namespace_name = getNamespaceName();

	DMP_H("namespace " + getNamespaceName() + "\n{\n\n");
	
	DMP_S("\nnamespace BON\n{\n\n");

	selConf( sorted); // select configuration ->set the toBeDumped member to T/F

	CWaitCursor w;

	createMethods();
	dumpGlobals( sorted);

	DMP_H("\n\n");
	DMP_S("\n\n}; // end namespace BON\n\n");
	DMP_S("namespace " + getNamespaceName() + "\n{\n");
	
	dumpMain( sorted);

	DMP_H("\n}; // namespace " + getNamespaceName() + "\n\n");

	DMP_H( Any::m_startUPToken + "\n" + global_user_part2 + Any::m_endUPToken + "\n\n");

	DMP_H("#endif // " + capitalized + "_H\n");

	DMP_S("\n}; // namespace " + getNamespaceName() + "\n\n");
}

#if(0)
bool Dumper::checkAll()
{
	bool res = true;

	bool res1 = checkAllFCOs();
	bool res2 = checkOrphanAttributes();
	res = res && res1 && res2;
	return res;
}


bool Dumper::checkOrphanAttributes()
{
	bool res = true;
	AttributeRep_Iterator it = m_attributeList.begin();
	for( ; it != m_attributeList.end(); ++it)
	{
		if ((*it)->hasAnyOwner() < 1 && !(*it)->isGlobal())
		{
			res = false;
			global_vars.err << "CHECK: \"" << (*it)->getName() << "\" local scope attribute not owned by any FCO.\n";
		}
	}
	return res;
}


//checks those elements which don't have parents
bool Dumper::checkAllFCOs()
{
	bool res = true;
	{FcoRep_Iterator it0( m_fcoRepList.begin());
	for( ; it0 != m_fcoRepList.end(); ++it0 )
	{
		/*if (!(*it0)->checkInheritance()) these kinds always have different descendants
			res = false;*/
		if (!(*it0)->isAbstract()) 
		{ 
			global_vars.err << "CHECK: FCO \"" << (*it0)->getName() << "\" should be abstract.\n";
			res = false; 
		}
		if (!(*it0)->isAbstract() && !(*it0)->checkIsPartOfFinal()) 
		{ 
			global_vars.err << "CHECK: FCO \"" << (*it0)->getName() << "\" is not part of any model, so it doesn't have any role.\n"; 
			res = false; 
		}
	}}

	{AtomRep_Iterator it1 = m_atomList.begin();
	for( ; it1 != m_atomList.end(); ++it1 )
	{
		if (!(*it1)->checkInheritance())
			res = false;
		if (!(*it1)->isAbstract() && !(*it1)->checkIsPartOfFinal()) 
		{ 
			global_vars.err << "CHECK: Atom \"" << (*it1)->getName() << "\" is not part of any model, so it doesn't have any role.\n"; 
			res = false; 
		}
	}}

	{ModelRep_Iterator it2 = m_modelList.begin();
	for( ; it2 != m_modelList.end(); ++it2 )
	{
		if (!(*it2)->checkInheritance())
			res = false;
		res = res && (*it2)->check();
		if (!(*it2)->isAbstract() && !(*it2)->checkIsPartOfFinal() && !m_rootFolder.isInRoot( *it2) && !findInFolders(*it2)) 
		{ 
			global_vars.err << "CHECK: Model \"" << (*it2)->getName() << "\" is not part of any model neither in the root folder nor any other folder, so it doesn't have any role.\n";
			res = false;
			//res = res; // this is an exception
		}
	}}

	{ConnectionRep_Iterator it3 = m_connList.begin();
	for( ; it3 != m_connList.end(); ++it3 )
	{
		if (!(*it3)->checkInheritance())
			res = false;
		res = res && (*it3)->checkConnectionTargets();
		if (!(*it3)->isAbstract() && !(*it3)->checkIsPartOfFinal()) 
		{ 
			global_vars.err << "CHECK: Connection \"" << (*it3)->getName() << "\" is not part of any model, so it doesn't have any role.\n";
			res = false;
		}
	}}

	{SetRep_Iterator it4 = m_setList.begin();
	for( ; it4 != m_setList.end(); ++it4 )
	{
		if (!(*it4)->checkInheritance())
			res = false;
		res = res && (*it4)->checkSetElements();
		if (!(*it4)->isAbstract() && !(*it4)->checkIsPartOfFinal()) 
		{ 
			global_vars.err << "CHECK: Set \"" << (*it4)->getName() << "\" is not part of any model, so it doesn't have any role.\n"; 
			res = false; 
		}
	}}

	{ReferenceRep_Iterator it5 = m_refList.begin();
	for( ; it5 != m_refList.end(); ++it5 )
	{
		if (!(*it5)->checkInheritance())
			res = false;
		if (!(*it5)->checkNotEmpty())
		{
			global_vars.err << "CHECK: Reference \"" << (*it5)->getName() << "\" is not referring to any fco.\n";
			res = false;
		}
		else
		{
			if (!(*it5)->checkAllTheSameKind())
			{
				// removed on 2/25/2004
				/*global_vars.err << "CHECK: Reference \"" << (*it5)->getName() << "\" is referring to more than one kind of fcos.\n"; 
				res = false;*/
			}
			if (!(*it5)->isAbstract() && !(*it5)->checkIsPartOfFinal()) 
			{ 
				global_vars.err << "CHECK: Reference \"" << (*it5)->getName() << "\" is not part of any model, so it doesn't have any role.\n"; 
				res = false; 
			}
		}
	}}

  return res;
}
#endif


void Dumper::showMultipleInheritance()
{
	FcoRep_Iterator it0( m_fcoRepList.begin());
	for( ; it0 != m_fcoRepList.end(); ++it0 )
	{
		(*it0)->showMultipleInheritance();
	}

	AtomRep_Iterator it1 = m_atomList.begin();
	for( ; it1 != m_atomList.end(); ++it1 )
	{
		(*it1)->showMultipleInheritance();
	}

	ModelRep_Iterator it2 = m_modelList.begin();
	for( ; it2 != m_modelList.end(); ++it2 )
	{
		(*it2)->showMultipleInheritance();
	}

	ConnectionRep_Iterator it3 = m_connList.begin();
	for( ; it3 != m_connList.end(); ++it3 )
	{
		(*it3)->showMultipleInheritance();
	}

	SetRep_Iterator it4 = m_setList.begin();
	for( ; it4 != m_setList.end(); ++it4 )
	{
		(*it4)->showMultipleInheritance();
	}

	ReferenceRep_Iterator it5 = m_refList.begin();
	for( ; it5 != m_refList.end(); ++it5 )
	{
		(*it5)->showMultipleInheritance();
	}
}


void Dumper::fetchConfigurationNames( std::vector< std::string>& res)
{
	BON::RegistryNode rn = Sheet::m_BON_Project_Root_Folder->getRegistry()->getChild("BonExtender/SavedConfigurations");
	if ( rn)
	{
		std::set<BON::RegistryNode> ch = rn->getChildren();
		std::set<BON::RegistryNode>::iterator i = ch.begin();
		for( ; i != ch.end(); ++i)
		{
			std::string one_config( (*i)->getName()); // previously getPath used

			if ( (*i)->getIntegerValue() == 1
				&& std::find( res.begin(), res.end(), one_config) == res.end()) // not found
				res.push_back( one_config);
		}
	}
}


void Dumper::selConf( std::vector<FCO*>& s)
{
	CWaitCursor wait;	
	std::vector< std::string > tmp,vec;
	fetchConfigurationNames( tmp);
	if ( std::find( tmp.begin(), tmp.end(), "Default") == tmp.end())
		vec.push_back( "Default");
	vec.insert( vec.end(), tmp.begin(), tmp.end()); // copy tmp to the end of vec

	SelConf sc( vec.size());
	sc.addConfigs( vec);

	int last_id;
	for( std::vector<FolderRep*>::const_iterator it_b = m_folderList.begin(); it_b != m_folderList.end(); ++it_b)
	{
		CString name = (*it_b)->getName().c_str();
		std::string regVal = (*it_b)->getMyRegistry()->getValueByPath( "/Configs/Default");
		if ( regVal == "false")
			last_id = sc.addEntry( name, 0, false, *it_b);
		else
			last_id = sc.addEntry( name, 0, true, *it_b);
	}
	
	for( std::vector< FCO *>::iterator it = s.begin(); it != s.end(); ++it)
	{
		CString name = (*it)->getName().c_str();
		int clique = (*it)->getCliqueId( FCO::INTERFACE);
		std::string regVal = (*it)->getMyRegistry()->getValueByPath( "/Configs/Default");
		if ( regVal == "false")
			last_id = sc.addEntry( name, clique, false, *it);
		else
			last_id = sc.addEntry( name, clique, true, *it);
	}

	if ( sc.DoModal() == IDOK)
	{
		wait.Restore();
		if ( sc.m_currSelConfigStr != "Default")
		{
			std::string path = "/BonExtender/SavedConfigurations/" + sc.m_currSelConfigStr;
			Sheet::m_BON_Project_Root_Folder->getRegistry()->setValueByPath( path, "1");
		}
		// try to select extedAnc for each non to be extended fco
		for( std::vector< FCO *>::iterator it = s.begin(); it != s.end(); ++it)
		{
			if ( !(*it)->isToBeEx() && (*it)->isFCO())
			{
				int my_level = (*it)->getLevel( FCO::INTERFACE);
				int closest_to_my_level = -1;
				FCO * selected_extended_anc = 0;
				std::vector< FCO*> ancestors;
				(*it)->getIntAncestors( ancestors);
				std::vector< FCO*>::iterator anc_it = ancestors.begin();
				for( ; anc_it != ancestors.end(); ++anc_it)
				{
					int level_of_ancestor = (*anc_it)->getLevel( FCO::INTERFACE);
					if ( (*anc_it)->isToBeEx() && // it is extended
						level_of_ancestor < my_level && // it is above the *it
						level_of_ancestor > closest_to_my_level ) // better than others
					{
						closest_to_my_level = level_of_ancestor;
						selected_extended_anc = *anc_it;
					}
				}
				if ( closest_to_my_level != -1 && selected_extended_anc != 0)
				{
					(*it)->setExtedAnc( selected_extended_anc);
					selected_extended_anc->addNonExtedDesc( *it);
				}
			}
		}
	}	
}


bool Dumper::build()
{
	inheritMoReSeCoFolAsp();		// models, references, set, connections, folder and aspect membership
	//showMultipleInheritance();

	doDump();

	return true;
}


/*static*/ int Dumper::backup( const std::string& file_name)
{
	if ( existsFile( file_name.c_str()))
	{
		std::string backup = file_name + ".copy";
		int res = makeFileCopy( file_name.c_str(), backup.c_str());
		if ( res == 2)
		{
			TO("Could not read file during backup process: " + file_name);
			return 2;
		}
		else if ( res == 3)
		{
			TO("Cannot create backup file: " + backup);
			return 3;
		}
	}
	return 1;
}


/*static*/ int Dumper::selectOutputFiles( const std::string& proj_name_to_be_checked)
{
	std::string proj_name = proj_name_to_be_checked;
	Any::convertToValidName( proj_name);

	OptionsDlg opt;
	opt.m_defName = proj_name.c_str();
	if ( opt.DoModal() != IDOK) return 2;
	
	global_vars.output_directory_name = "";
	int i = opt.m_headerName.ReverseFind('\\');
	if ( i != -1)
	{
		global_vars.output_directory_name = opt.m_headerName.Left( i+1);
	}

	global_vars.header_file_name = opt.m_headerName;
	global_vars.source_file_name = opt.m_sourceName;
	global_vars.err_file_name = global_vars.output_directory_name + proj_name + "BONExt.log";

	m_bGenInit = opt.m_bInit == TRUE;
	m_bGenFinalize = opt.m_bFinalize == TRUE;
	m_bGenRegular = opt.m_whichStyle != 1; // 0 or 2
	m_bGenTemplates = (opt.m_whichStyle > 0); // 1 or 2 ( = both)
	m_bGenTemplVersion6 = (opt.m_version71 == 0);

	m_bGenVisitor = opt.m_bVisitor == TRUE;
	if (m_bGenVisitor)
	{
		global_vars.m_visitorHeaderFileName = opt.m_visitorHeaderName;
		global_vars.m_visitorSourceFileName = opt.m_visitorSourceName;
	}

	m_bSafetyBackup = true;
	if ( m_bSafetyBackup)
	{
		int res1( 1), res2( 1), res3( 1), res4( 1);
		res1 = backup( global_vars.header_file_name);
		res2 = backup( global_vars.source_file_name);

		if ( m_bGenVisitor)
		{
			res3 = backup( global_vars.m_visitorHeaderFileName);
			res4 = backup( global_vars.m_visitorSourceFileName);
		}
		if ( res1 != 1 || res2 != 1 || res3 != 1 || res4 != 1)
			return 2; // abort execution
	}

	m_bParsePrev = opt.m_bParse == TRUE;

	global_vars.header_backup_name = ""; // initial value, notifies the dumper NOT to search for UP (user part) declarations
	if ( m_bParsePrev)
	{
		if ( existsFile( (LPCTSTR) opt.m_prevHeaderName))
			global_vars.header_backup_name = opt.m_prevHeaderName;
		else
			TO( opt.m_prevHeaderName + " previously generated header file not found.");
	}

	return 1;
}

/*static*/ void Dumper::getGlobalUserParts( std::string& up1, std::string& up2)
{
	std::ifstream old_xmp_file;
	std::string old_f_name;
	old_f_name = global_vars.header_backup_name;
	if ( old_f_name.empty()) // means that no previous version existed or the user selected not to parse the old file
	{
		up1 = "// add your include files/class definitions here\n";
		up2 = "// add your additional class definitions here\n";
		return;
	}

	old_xmp_file.open( old_f_name.c_str() ,std::ios_base::in);
	if ( !old_xmp_file.is_open())
	{
		up1 = "// ERROR accessing file\n";
		up2 = "// ERROR accessing file\n";
		return;
	}

	std::string line;
	while( !old_xmp_file.eof() && line.substr(0, Any::m_startUPToken.length()) != Any::m_startUPToken) 
	{
		std::getline( old_xmp_file, line);
	}
	if ( !old_xmp_file.eof()) // found the start token starting at 1st character
	{
		std::getline( old_xmp_file, line);
		while( !old_xmp_file.eof() && line.substr( 0, Any::m_endUPToken.length()) != Any::m_endUPToken) 
		{
			up1 += line + "\n";
			std::getline( old_xmp_file, line);
		}
		if ( !old_xmp_file.eof()) // found the end token
		{
		}
		else
		{
			old_xmp_file.close();
			up1 = "// Ending token (" + Any::m_endUPToken + ") for first global user part not found\n";
			up2 = "// Error in first global user part\n";
			return;
		}
	}
	else
	{
		old_xmp_file.close();
		up1 = "// Starting token (" + Any::m_startUPToken + ") for first global user part not found\n";
		up2 = "// Error in first global user part\n";
		return; 
	}

	while( !old_xmp_file.eof() && line.substr(0, Any::m_startUPToken.length()) != Any::m_startUPToken) 
	{
		std::getline( old_xmp_file, line);
	}
	if ( !old_xmp_file.eof()) // found the start token starting at 1st character
	{
		std::getline( old_xmp_file, line);
		while( !old_xmp_file.eof() && line.substr( 0, Any::m_endUPToken.length()) != Any::m_endUPToken) 
		{
			up2 += line + "\n";
			std::getline( old_xmp_file, line);
		}
		if ( !old_xmp_file.eof()) // found the end token
		{
		}
		else
		{
			old_xmp_file.close();
			up2 = "// Ending token (" + Any::m_endUPToken + ") for second global user part not found\n";
			return;
		}
	}
	else
	{
		old_xmp_file.close();
		up2 = "// Starting token (" + Any::m_startUPToken + ") for second global user part not found\n";
		return; 
	}

	old_xmp_file.close();
	return;
}


