#include "stdafx.h"
#include <afxdlgs.h>

#include <exception>

#include <stdio.h>
#include "Component.h"
#include "MetaInterpreter.h"

#include "aspectspectbl.h"
#include "aspectspecdlg.h"
#include "Regexp.h"
#include "TokenEx.h"




#pragma bookmark( "Make constraints from cardinalities" )
#pragma bookmark( "Inheritance checking could be better: implementation->interface loops could be allowed" )
#pragma bookmark( "Empty aspect do not show up during aspect mapping spec...! Need to fix (?)." )
#pragma bookmark( "Default aspects + warnings without havings" )
#pragma bookmark( "Store last metaref -> never assign same metaref to different classes" )
#pragma bookmark( "Use the same assoc class for multiple association (connector) & multiple inheritance !" )



CAspectSpecDlg	*theAspectDlg;
static CParadigmBuilder *theParadigm;
static CMetaRefBroker *theBroker;

#define DeleteMembers(list)							\
	{												\
		POSITION pos = list.GetHeadPosition();		\
		while(pos)									\
			delete list.GetNext(pos);				\
	}


#define MAXINDENT	256
#define Indent()									\
	{												\
		char tmp[MAXINDENT];						\
		memset(tmp,' ',MAXINDENT);					\
		tmp[min(MAXINDENT - 1,depth * 3)] = '\0';	\
		xml << tmp;									\
	}



void SortList<CRoleList>( CRoleList &listToSort )
{
	if( !listToSort.IsEmpty( ) )
	{
		CRoleList *temps = &listToSort;
		// now, sort listToSort alphabetically
		int count = listToSort.GetCount( );
		for( int i=0; i < count; i++ )
		{
			for( int j=0; j < count - i - 1; j++ )
			{
				POSITION pos = listToSort.FindIndex( j );
				POSITION pos2 = listToSort.FindIndex( j+1 );
				if( pos && pos2 )
				{
					CRole *obj = listToSort.GetAt( pos );
					CRole *obj2 = listToSort.GetAt( pos2 );
					if( ( obj && obj2 ) &&
						( strcmp( obj->name, obj2->name ) > 0 ) )
					{
						// obj is after obj2, must switch them
						listToSort.SetAt( pos, obj2 );
						listToSort.SetAt( pos2, obj );
					}
				}
			}
		}
	}

}

void SortList<CPointerItemList>( CPointerItemList &listToSort )
{
	if( !listToSort.IsEmpty( ) )
	{
		// now, sort listToSort alphabetically, through the CPointerItem->object
		int count = listToSort.GetCount( );
		for( int i=0; i < count; i++ )
		{
			for( int j=0; j < count - i - 1; j++ )
			{
				POSITION pos = listToSort.FindIndex( j );
				POSITION pos2 = listToSort.FindIndex( j+1 );
				if( pos && pos2 )
				{
					CPointerItem *item = listToSort.GetAt( pos );
					CPointerItem *item2 = listToSort.GetAt( pos2 );
					CRole *obj = item->object;
					CRole *obj2 = item2->object;
					if( ( obj && obj2 ) &&
						( obj->name.CompareNoCase( obj2->name ) > 0 ) )
					{
						listToSort.SetAt( pos, item2 );
						listToSort.SetAt( pos2, item );
					}
				}
			}
		}
	}

}



static void DumpDispName(ofstream xml, const CString& dispName)
{
	xml << "<dispname>" << dispName << "</dispname>";
}

static void DumpMetaRef(ofstream xml, const CBuilderObject *obj = NULL, const CString &path = CString(""))
{
	metaref_type mr = obj ? theBroker->GetMetaRef(obj, path) : theBroker->GetRootFolderMetaRef();
	xml << "metaref = \"" << mr << "\" ";
}

static bool ParseCardinality(const CString& cardinality, const CString &target, const CString &contextDesc, CString &expr)
{
	expr = "";
	CString card(cardinality);
	card.Remove(_T('\t'));
	card.Remove(_T('\n'));
	card.Remove(_T(' '));

	bool berr = false;
	CStringArray ORs;
	CTokenEx	tok;
	tok.Split(card, ",", ORs);
	for (int i = 0; i < ORs.GetSize(); i++) {
		CStringArray bounds;
		tok.Split(ORs[i], "..", bounds);

		CString orexpr;

		if (bounds.GetSize() == 1) {
			int multip;
			if(sscanf((LPCTSTR)bounds[0],"%d",&multip) != 1) {
				berr = true;
				break;
			}
			else {
				if (multip < 0) {
					AfxMessageBox("Multiplicity is negative: " + contextDesc, MB_ICONEXCLAMATION);
				}
				orexpr.Format("(%s = %d)", target, multip);
			}
		}
		else if (bounds.GetSize() == 2) {
			int multipmin, multipmax;
			CString andexpr1, andexpr2;

			if(sscanf((LPCTSTR)bounds[0],"%d",&multipmin) != 1) {
				berr = true;
				break;
			}
			else {
				if (multipmin < 0) {
					AfxMessageBox("Minimum multiplicity is negative: " + contextDesc, MB_ICONEXCLAMATION);
				}
				if (multipmin > 0) {
					andexpr1.Format("(%s >= %d)", target, multipmin);
				}
			}

			if(sscanf((LPCTSTR)bounds[1],"%d",&multipmax) != 1) {
				if (bounds[1] != "*") {
					berr = true;
					break;
				}
			}
			else {
				andexpr2.Format("(%s <= %d)", target, multipmax);
				if (multipmax < 0) {
					AfxMessageBox("Maximum multiplicity is negative: " + contextDesc, MB_ICONEXCLAMATION);
				}
				if (multipmax < multipmin) {
					AfxMessageBox("Minimum multiplicity > Maximum multiplicity : " + contextDesc, MB_ICONEXCLAMATION);
				}
			}

			if ((!andexpr1.IsEmpty()) && (!andexpr2.IsEmpty())) {
				orexpr.Format("( %s and %s )", andexpr1, andexpr2);
			}
			else if (!andexpr1.IsEmpty()) {
				orexpr = andexpr1;
			}
			else if (!andexpr2.IsEmpty()) {
				orexpr = andexpr2;
			}
		}

		if (!orexpr.IsEmpty()) {
			if (!expr.IsEmpty()) {
				expr += " or ";
			}
			expr += orexpr;
		}
	}

	if (berr) {
		expr = "";
	}

	return berr;
}
///////////////////////////// CMetarefBroker ///////////////////////////////
const metaref_type CMetaRefBroker::INITIAL_METAREF = 1000; // Compatibility with the implied method
const metaref_type CMetaRefBroker::INVALID_METAREF = 0;

CMetaRefBroker::CMetaRefBroker()
{
	firstfree = INITIAL_METAREF + 1; // Allocate root folder metaref
}


void CMetaRefBroker::InitFromObj(const CBuilderObject *obj)
{
	try {
		CComPtr<IMgaRegNode>	regNode;
		CComBSTR path("MetaRef");

		CComPtr<IMgaFCO> fco = obj->GetIObject();
		COMTHROW(fco->get_RegistryNode(path, &regNode));
		InitFromRegistry(obj, regNode);
	}
	catch (...) {
		AfxMessageBox("Error in object (" + obj->GetName() + ") metaref registry.\nNew metarefs will be generated.", MB_OK | MB_ICONEXCLAMATION);
	}
}

void CMetaRefBroker::InitFromRegistry(const CBuilderObject *obj, CComPtr<IMgaRegNode> regNode)
{
	long status;
	COMTHROW(regNode->get_Status(&status));
	if (status == ATTSTATUS_HERE) {
		CBstr bstr;
		CString path;
		CString sval;
		metaref_type val;

		COMTHROW(regNode->get_Path(bstr));
		path = bstr;

		int idx = path.Find(_T("/"));
		if (idx == -1) {
			path.Empty();
		}
		else {
			path.Delete(0, idx+1);
		}


		COMTHROW(regNode->get_Value(bstr));
		sval = bstr;
		val = _ttoi((LPCTSTR)sval);
		if (val) {
			CMetaRefNode node(obj, path, val);
			metaRefDB.AddTail(node);
			firstfree = max(firstfree, val+1);
		}
	}

	CComPtr<IMgaRegNodes>	subNodes;
	COMTHROW(regNode->get_SubNodes(VARIANT_FALSE, &subNodes));

	MGACOLL_ITERATE(IMgaRegNode, subNodes) {
		CComPtr<IMgaRegNode> subNode = MGACOLL_ITER;
		InitFromRegistry(obj, subNode);
	} MGACOLL_ITERATE_END;
}


metaref_type CMetaRefBroker::GetMetaRef(const CBuilderObject *obj, const CString &extraPath)
{
	metaref_type newmr = INVALID_METAREF;


	POSITION pos = metaRefDB.GetHeadPosition();
	while (pos) {
		POSITION apos = pos;
		CMetaRefNode node = metaRefDB.GetNext(pos);
		if ( (node.obj == obj) && (node.path == extraPath) ) {
			newmr = node.metaref;
			metaRefDB.RemoveAt(apos);
		}
	}

	if (newmr == INVALID_METAREF) {
		newmr = firstfree++;

		CString val;
		val.Format("%ld", newmr);
		CString path("MetaRef");
		if (!extraPath.IsEmpty()) {
			path += "/" + extraPath;
		}
		obj->SetRegistryValue(path, val);
		// CMetaRefNode node(obj, extraPath, newmr);
		// metaRefDB.AddTail(node);
	}
	return newmr;
}

void CMetaRefBroker::CheckDulicates(void)
{
	CMap<metaref_type, metaref_type, POSITION, POSITION> checkMap;
	POSITION pos = metaRefDB.GetHeadPosition();
	while (pos) {
		POSITION actpos = pos;
		CMetaRefNode node = metaRefDB.GetAt(actpos);
		metaRefDB.GetNext(pos);
		POSITION otherPos;
		if (checkMap.Lookup(node.metaref, otherPos)) {
			CMetaRefNode otherNode = metaRefDB.GetAt(otherPos);
			bool flag = false;
			if (node.obj->GetObjId() < otherNode.obj->GetObjId()) {
				metaRefDB.RemoveAt(otherPos);
				checkMap[node.metaref] = actpos;
				flag = true;
			}
			else {
				metaRefDB.RemoveAt(actpos);
				flag = false;
			}
			CString fmtstr;
			fmtstr.Format("Duplicate metaref found.\nClass %s, subnode: %s will get a new metaref.", (flag ? otherNode : node).obj->GetName(), node.path.IsEmpty() ? "n/a" : node.path);
			CString dpath("MetaRef");
			CString extraPath = (flag ? otherNode : node).path;
			if (!extraPath.IsEmpty()) {
				dpath += "/" + extraPath;
			}
			(flag ? otherNode : node).obj->SetRegistryValue(dpath, CString(""));
			AfxMessageBox(fmtstr, MB_OK | MB_ICONEXCLAMATION);

		}
		else {
			checkMap[node.metaref] = actpos;
		}
	}

}

///////////////////////////// CComponent ///////////////////////////////
void CComponent::InvokeEx(CBuilder &builder,CBuilderObject *focus, CBuilderObjectList &selected, long param)
{
	try
	{
		theParadigm = new CParadigmBuilder;
		theBroker = new CMetaRefBroker;

		CBuilderFolder *root = builder.GetRootFolder();
		if(!root)
			AfxMessageBox("Cannot find the root of the Paradigm !",MB_OK | MB_ICONSTOP);
		if(root && theParadigm->Build(root, &builder)) {
			CFileDialog dlg(FALSE, "xmp", builder.GetProjectName( ),
				OFN_EXPLORER | OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT,
				"XML Paradigm Files (*.xmp)|*.xmp|");
			if(dlg.DoModal() == IDOK) {
				CWaitCursor wait;
				CString filename = dlg.GetPathName();
				if (theParadigm->DumpXML(filename)) {
					if (AfxMessageBox("Successfully generated " + filename + "\n\nWould you like to register your new paradigm ?", MB_YESNO | MB_ICONINFORMATION) == IDYES) {
						CComPtr<IMgaRegistrar> registrar;
						HRESULT hr = registrar.CoCreateInstance(L"MGA.MgaRegistrar");
						if (SUCCEEDED(hr)) {
							CComBSTR path = CString("XML=") + filename;
							hr = registrar->RegisterParadigmFromData(path, NULL, REGACCESS_USER);
						}
						if (FAILED(hr)) {
							AfxMessageBox("Error occured while registering new paradigm.", MB_ICONSTOP | MB_OK);
						}
					}
				}
				else {
					AfxMessageBox("Cannot open/create file " + filename, MB_OK | MB_ICONSTOP);
				}
				delete theParadigm;	theParadigm = NULL;
				delete theBroker;	theBroker = NULL;
				return;
			}
		}
		AfxMessageBox("No output was produced!", MB_OK | MB_ICONSTOP);
		delete theParadigm;	theParadigm = NULL;
		delete theBroker;	theBroker = NULL;
	}
	catch( ... )
	{
		AfxMessageBox( "An exception occurred in the Metamodeling2000 Interpreter." );
		throw;
	}
}





///////////////////////////// CParadigmBuilder ///////////////////////////////


CParadigmBuilder::~CParadigmBuilder()
{
	DeleteMembers(aspects);
}

void CParadigmBuilder::DiscardFCOClass(CFCOClassBase *classToDrop)
{
	fcoClasses.RemoveAt(fcoClasses.Find(classToDrop));

	CAtomBuilder *cab = dynamic_cast<CAtomBuilder*>(classToDrop);
	CModelBuilder *cmb = dynamic_cast<CModelBuilder*>(classToDrop);
	CReferenceBuilder *crb = dynamic_cast<CReferenceBuilder*>(classToDrop);
	CSetBuilder *csb = dynamic_cast<CSetBuilder*>(classToDrop);
	CConnectionBuilder *ccb = dynamic_cast<CConnectionBuilder*>(classToDrop);
	CFCOBuilder * cfb = dynamic_cast<CFCOBuilder*>(classToDrop);
	if (cab) {
		TRACE("Atom discarded from Paradigm");
		atoms.RemoveAt(atoms.Find(cab));
	}
	if (cmb) {
		TRACE("Model discarded from Paradigm");
		models.RemoveAt(models.Find(cmb));
	}
	if (crb) {
		TRACE("Reference discarded from Paradigm");
		refs.RemoveAt(refs.Find(crb));
	}
	if (csb) {
		TRACE("Set discarded from Paradigm");
		sets.RemoveAt(sets.Find(csb));
	}
	if (ccb) {
		TRACE("Connection discarded from Paradigm");
		conns.RemoveAt(conns.Find(ccb));
	}
	if (cfb) {
		TRACE("FCO discarded from Paradigm");
		fcos.RemoveAt(fcos.Find(cfb));
	}
}

void CParadigmBuilder::DiscardFolder(CFolderBuilder *folderToDrop)
{
	folders.RemoveAt(folders.Find(folderToDrop));
}

void CParadigmBuilder::DiscardAspect(CAspect *aspectToDrop)
{
	aspects.RemoveAt(aspects.Find(aspectToDrop));
	delete aspectToDrop;
}

bool CParadigmBuilder::Build(CBuilderFolder *root, CBuilder *builder)
{
	name = builder->GetProjectName();

	GatherSheets(root);
	BuildLists();
	GatherProxies();
	FindEquivObjects();
	BuildInheritance();
	if(!CheckInheritance())
		return false;

	// TODO: Check Implementation and Interface Inheritance + EQ

	BuildFolders();
	BuildComposition();
	BuildReferences();
	BuildConnections();
	BuildSets();
	BuildAttributes();
	BuildAspects();
	BuildConstraints();
	if (!CheckDuplicateNames())
		return false;
	InitMetaRefs();
	return true;
}

void CParadigmBuilder::GatherSheets(CBuilderFolder *folder)
{
	const CBuilderFolderList *folders = folder->GetSubFolders();
	POSITION pos = folders->GetHeadPosition();
	while(pos)
		GatherSheets(folders->GetNext(pos));
	const CBuilderModelList *roots = folder->GetRootModels();
	pos = roots->GetHeadPosition();
	while(pos) {
		CBuilderModel *sheet = dynamic_cast<CBuilderModel *>(roots->GetNext(pos));
		if(!sheet) {
			AfxMessageBox("Unexpected root model kind found");
			continue;
		}
		sheets.AddTail(sheet);
	}
}

void CParadigmBuilder::BuildLists()
{
	POSITION spos = sheets.GetHeadPosition();
	while (spos) {
		CBuilderModel	* sheet = sheets.GetNext(spos);
		const CBuilderObjectList *kidsList = sheet->GetChildren( );
		CBuilderObjectList kids;
		SortList( kidsList, kids );
		CBuilderObjectList *children = &kids;
		POSITION pos = children->GetHeadPosition();
		while(pos) {
			CBuilderObject *child = children->GetNext(pos);
			CFolderBuilder *folder = dynamic_cast<CFolderBuilder *>(child);
			CFCOClassBase *fcoClass = dynamic_cast<CFCOClassBase *>(child);
			CModelBuilder *model = dynamic_cast<CModelBuilder *>(child);
			CAtomBuilder *atom = dynamic_cast<CAtomBuilder *>(child);
			CReferenceBuilder *ref = dynamic_cast<CReferenceBuilder *>(child);
			CSetBuilder *set = dynamic_cast<CSetBuilder *>(child);
			CConnectionBuilder *conn = dynamic_cast<CConnectionBuilder *>(child);
			CFCOBuilder *fco = dynamic_cast<CFCOBuilder *>(child);
			CAttributeBuilder *attr = dynamic_cast<CAttributeBuilder *>(child);
			CConstraintBuilder *constraint = dynamic_cast<CConstraintBuilder *>(child);
			CConstraintFuncBuilder *constraintfunc = dynamic_cast<CConstraintFuncBuilder *>(child);
			if(fcoClass)
				fcoClasses.AddTail(fcoClass);

			if(folder)
				folders.AddTail(folder);
			else if(model)
				models.AddTail(model);
			else if(atom)
				atoms.AddTail(atom);
			else if(ref)
				refs.AddTail(ref);
			else if(set)
				sets.AddTail(set);
			else if(conn)
				conns.AddTail(conn);
			else if(attr)
				attrs.AddTail(attr);
			else if(constraint)
				allConstraints.AddTail(constraint);
			else if(constraintfunc)
				constraintfuncs.AddTail(constraintfunc);
		}
	}
}

void CParadigmBuilder::GatherProxies()
{
	POSITION pos = folders.GetHeadPosition();
	while(pos)
		folders.GetNext(pos)->GatherProxies();

	pos = fcoClasses.GetHeadPosition();
	while(pos)
		fcoClasses.GetNext(pos)->GatherProxies();

}

void CParadigmBuilder::FindEquivObjects()
{
	POSITION pos = folders.GetHeadPosition();
	while(pos) {
		folders.GetAt(pos)->FindSameFolders();
		folders.GetNext(pos);
	}

	pos = fcoClasses.GetHeadPosition();
	while(pos) {
		fcoClasses.GetAt(pos)->FindEQClasses();
		fcoClasses.GetNext(pos);
	}

}


void CParadigmBuilder::BuildFolders()
{
	POSITION pos = folders.GetHeadPosition();
	while(pos)
		folders.GetNext(pos)->BuildComposition();
}

void CParadigmBuilder::BuildInheritance()
{
	POSITION pos = fcoClasses.GetHeadPosition();
	while(pos)
		fcoClasses.GetNext(pos)->FindSubClasses();

}

bool CParadigmBuilder::CheckInheritance()
{
	POSITION pos = fcoClasses.GetHeadPosition();
	while(pos) {
		CStringList trace;
		if(!fcoClasses.GetNext(pos)->CheckInheritance(trace)) {
			CString traceTxt;
			POSITION pos = trace.GetHeadPosition();
			while(pos)
				traceTxt += trace.GetNext(pos) + " -> ";
			traceTxt += trace.GetHead();
			AfxMessageBox("Loop detected in the inheritance hierarchy: \n" + traceTxt,MB_OK | MB_ICONSTOP);
			return false;
		}
	}
	return true;
}

bool CParadigmBuilder::CheckDuplicateNames()
{
	CString warnTop("Duplicate names at same scope were found.\n");
	CString warnBottom("\nThe paradigm is invalid.");

	// Checking for duplicate attribute names at global scope
	CTypedPtrMap<CMapStringToPtr, CString, CAttributeBuilder *>	attrmap;
	POSITION attrpos = attrs.GetHeadPosition();
	while (attrpos) {
		CAttributeBuilder *ab = attrs.GetNext(attrpos);
		CAttributeBuilder *abLookUp;
		if (ab->IsGlobal()) {
			if (attrmap.Lookup(ab->GetName(), abLookUp)) {
				AfxMessageBox( warnTop + "Attribute: " + ab->GetName() + " Scope: global" + warnBottom, MB_OK | MB_ICONEXCLAMATION);
				return false;
			}
			else {
				attrmap[ab->GetName()] = ab;
			}
		}
	}

	// Checking for duplicate class names at global scope and attribute names at local scope
	CTypedPtrMap<CMapStringToPtr, CString, CFCOClassBase *>	objmap;
	POSITION objpos = fcoClasses.GetHeadPosition();
	while (objpos) {
		CFCOClassBase *obj = fcoClasses.GetNext(objpos);
		CFCOClassBase *objLookUp;
		CString fullName;
		fullName.Format("%s <<%s>>", obj->GetName(), obj->GetRuntimeClass()->m_lpszClassName); // Dirty trick by Peter
		if (objmap.Lookup(fullName, objLookUp)) {
			AfxMessageBox( warnTop + "Class: " + fullName + " Scope: global" + warnBottom, MB_OK | MB_ICONEXCLAMATION);
			return false;
		}
		else {
			objmap[fullName] = obj;
		}

		attrmap.RemoveAll();
		attrpos = obj->GetAttributes().GetHeadPosition();
		while (attrpos) {
			CAttributeBuilder *ab = obj->GetAttributes().GetNext(attrpos);
			CAttributeBuilder *abLookUp;
			if (!ab->IsGlobal()) {
				if (attrmap.Lookup(ab->GetName(), abLookUp)) {
					AfxMessageBox( warnTop + "Attribute: " + ab->GetName() + " Scope: " + obj->GetName() + warnBottom, MB_OK | MB_ICONEXCLAMATION);
					return false;
				}
				else {
					attrmap[ab->GetName()] = ab;
				}
			}
		}
	}

	// Checking for duplicate aspect names at global scope
	CTypedPtrMap<CMapStringToPtr, CString, CAspect *>	aspmap;
	POSITION asppos = aspects.GetHeadPosition();
	while (asppos) {
		CAspect *asp = aspects.GetNext(asppos);
		CAspect *aspLookUp;
		if (aspmap.Lookup(asp->GetName(), aspLookUp)) {
			AfxMessageBox( warnTop + "Aspect: " + asp->GetName() + " Scope: global" + warnBottom, MB_OK | MB_ICONEXCLAMATION);
			return false;
		}
		else {
			aspmap[asp->GetName()] = asp;
		}
	}
	if (aspects.GetCount() < 1) {
		AfxMessageBox("There is no aspect defined in your metamodel.\nThis is an invalid paradigm.", MB_OK | MB_ICONSTOP);
		return false;
	}

	// Checking for duplicate folder names at global scope
	CTypedPtrMap<CMapStringToPtr, CString, CFolderBuilder *>	fldmap;
	POSITION fldpos = folders.GetHeadPosition();
	while (fldpos) {
		CFolderBuilder *fld = folders.GetNext(fldpos);
		CFolderBuilder *fldLookUp;
		if (fldmap.Lookup(fld->GetName(), fldLookUp)) {
			AfxMessageBox( warnTop + "Folder: " + fld->GetName() + " Scope: global" + warnBottom, MB_OK | MB_ICONEXCLAMATION);
			return false;
		}
		else {
			fldmap[fld->GetName()] = fld;
		}
	}

	return true;
}



void CParadigmBuilder::BuildComposition()
{
	POSITION pos = models.GetHeadPosition();
	while(pos)
		models.GetNext(pos)->FindRoles();
}

void CParadigmBuilder::BuildConnections()
{
	POSITION pos = conns.GetHeadPosition();
	while(pos)
		conns.GetNext(pos)->BuildInstances();
}

void CParadigmBuilder::BuildReferences()
{
	POSITION pos = refs.GetHeadPosition();
	while(pos)
		refs.GetNext(pos)->FindRefs();
}

void CParadigmBuilder::BuildSets()
{
	POSITION pos = sets.GetHeadPosition();
	while(pos)
		sets.GetNext(pos)->FindMembers();
}

void CParadigmBuilder::BuildAttributes()
{
	POSITION pos = fcoClasses.GetHeadPosition();
	while(pos)
		fcoClasses.GetNext(pos)->FindAttributes();
}

void CParadigmBuilder::BuildConstraints()
{
	POSITION pos = allConstraints.GetHeadPosition();
	while(pos)
		allConstraints.GetNext(pos)->Attach();
}

void CParadigmBuilder::AddConstraint(CConstraintBuilder *constraint)
{
	constraints.AddTail(constraint);
}

void CParadigmBuilder::BuildAspects()
{
	{
		POSITION spos = sheets.GetHeadPosition();
		while (spos) {
			CBuilderModel *sheet = sheets.GetNext(spos);
			const CBuilderSetList *sets = sheet->GetSets("Aspect");
			if(sets) {
				POSITION pos = sets->GetHeadPosition();
				while(pos) {
					CBuilderSet *set = sets->GetNext(pos);
					CAspect *aspect = new CAspect(set);
					aspects.AddTail(aspect);
				}
			}
		}
	}
	{
		POSITION pos = aspects.GetHeadPosition();
		while(pos) {
			aspects.GetAt(pos)->FindSameAspects();
			aspects.GetNext(pos);
		}
	}
	{
		POSITION pos = models.GetHeadPosition();
		while(pos)
			models.GetNext(pos)->FindAspects();
	}
	{
		CMapWordToPtr aspMap;
		theAspectDlg = new CAspectSpecDlg("Configure Aspect Mapping");
		POSITION pos = aspects.GetHeadPosition();
		while (pos) {
			CAspect *asp = aspects.GetNext(pos);
			POSITION ppos = (asp->allParts).GetHeadPosition();
			while (ppos) {
				CPart *par = (asp->allParts).GetNext(ppos);

				CString regPath, regVal;

				if ((par->role->part->GetKindName() == "Model") || (par->role->part->GetKindName() == "Reference")) {
					regPath.Format("KindAspects/%s:%s/%s", par->role->parent->GetName(), par->role->name, par->aspect->GetName());
					(par->role->part)->GetRegistryValue(regPath, regVal);
					if (!regVal.IsEmpty()) {
						POSITION kapos = aspects.GetHeadPosition();
						while (kapos) {
							CAspect *kaspect = aspects.GetNext(kapos);
							if (kaspect->GetName() == regVal)
								par->kindaspect = kaspect;
						}
						regVal.Empty();
					}
				}
				else {
					par->kindaspect = NULL;
				}


				regPath.Format("PrimaryAspects/%s:%s/%s", par->role->parent->GetName(), par->role->name, par->aspect->GetName());
				(par->role->part)->GetRegistryValue(regPath, regVal);
				if (!regVal.IsEmpty())
					if (regVal == "no")
						par->isPrimary = false;
				aspMap[ (theAspectDlg->AddEntry(asp->GetName(),par->role->parent->GetName() + ":" + par->role->name, (par->kindaspect ? (par->kindaspect)->GetName() : CString("N/A")), par->isPrimary)) ] = par;
			}
		}

		if (theAspectDlg->DoModal() == IDOK) {
			POSITION mpos = aspMap.GetStartPosition();
			while (mpos) {
				unsigned short rowID;
				CString regPath, regVal;
				CPart *par;
				CString dummy;
				CString kindaspName;
				aspMap.GetNextAssoc(mpos, rowID, (void*&)par);
				theAspectDlg->GetEntry(rowID, dummy, kindaspName, par->isPrimary);
				if ( (par->role->part->GetKindName() == "Model") || (par->role->part->GetKindName() == "Reference") ) {
					POSITION apos = aspects.GetHeadPosition() ;
					while (apos) {
						CAspect *kindasp = aspects.GetNext(apos);
						if (kindasp->GetName() == kindaspName)
							par->kindaspect = kindasp;
					}
					regPath.Format("KindAspects/%s:%s/%s", par->role->parent->GetName(), par->role->name, par->aspect->GetName());
					regVal = par->kindaspect->GetName();
					(par->role->part)->SetRegistryValue(regPath, regVal);
				}
				else {
					par->kindaspect = NULL;
				}
				regPath.Format("PrimaryAspects/%s:%s/%s", par->role->parent->GetName(), par->role->name, par->aspect->GetName());
				regVal = par->isPrimary ? "yes" : "no";
				(par->role->part)->SetRegistryValue(regPath, regVal);

			}
		}

		delete theAspectDlg;
	}
}

void CParadigmBuilder::InitMetaRefs()
{
	POSITION pos = folders.GetHeadPosition();
	while (pos) {
		theBroker->InitFromObj(folders.GetNext(pos));
	}

	pos = aspects.GetHeadPosition();
	while (pos) {
		theBroker->InitFromObj(aspects.GetNext(pos)->GetBuilderObject());
	}

	pos = attrs.GetHeadPosition();
	while (pos) {
		theBroker->InitFromObj(attrs.GetNext(pos));
	}

	pos = fcoClasses.GetHeadPosition();
	while (pos) {
		theBroker->InitFromObj(fcoClasses.GetNext(pos));
	}
	theBroker->CheckDulicates();

}

CAspect *CParadigmBuilder::FindAspect(CBuilderObject *obj)
{
	POSITION pos = aspects.GetHeadPosition();
	while(pos) {
		CAspect *asp = aspects.GetNext(pos);
		POSITION ppos = asp->parts.GetHeadPosition();
		while (ppos) {
			CBuilderObject *aobj = asp->parts.GetNext(ppos);
			if (aobj == obj)
				return asp;
		}
	}
	return 0;
}

void CParadigmBuilder::SetFlags(bool val)
{
	CFCOClassBase::SetFlags(fcoClasses,val);
}

void CParadigmBuilder::DumpFolders(ofstream xml)
{
	bool eFlag = true;

	xml << "   <folder name = \"RootFolder\" ";
	DumpMetaRef(xml);
	POSITION pos = folders.GetHeadPosition();
	CString subfolderNames;
	while(pos) {
		CFolderBuilder *subf = folders.GetNext(pos);
		if(subf->IsRoot()) {
			if (!subf->IsEmpty()) {
				eFlag = false;
			}
			subfolderNames += subf->GetName();
			if(pos)
				subfolderNames += " ";
		}
	}

	if(!subfolderNames.IsEmpty())
		xml << "subfolders = \"" << subfolderNames << "\" ";

	pos = fcoClasses.GetHeadPosition();
	CString fcoNames;
	while(pos) {
		CFCOClassBase *fco = fcoClasses.GetNext(pos);
		if(fco->IsRoot() && !fco->IsAbstract()) {
			eFlag = false;
			fcoNames += fco->GetName();
			if(pos)
				fcoNames += " ";
		}
	}
	if(!fcoNames.IsEmpty()) {
		xml << "rootobjects = \"" <<	fcoNames << "\" ";
	}

	xml << ">" << endl;

	if (eFlag) {
		AfxMessageBox("Warning: The RootFolder and all other folders are empty !", MB_ICONEXCLAMATION);
	}

	pos = constraints.GetHeadPosition();
	while(pos)
		constraints.GetNext(pos)->DumpXML(xml,2);

	pos = constraintfuncs.GetHeadPosition();
	while(pos)
		constraintfuncs.GetNext(pos)->DumpXML(xml,2);


	pos = attrs.GetHeadPosition();
	while(pos) {
		CAttributeBuilder *attr = attrs.GetNext(pos);
		if(attr->IsGlobal())
			attr->DumpXML(xml,2, "");
	}

	pos = atoms.GetHeadPosition();
	while(pos) {
		CAtomBuilder *atom = atoms.GetNext(pos);
		if(!atom->IsAbstract())
			atom->DumpXML(xml,2);
	}

	pos = refs.GetHeadPosition();
	while(pos) {
		CReferenceBuilder *ref = refs.GetNext(pos);
		if(!ref->IsAbstract())
			ref->DumpXML(xml,2);
	}

	pos = sets.GetHeadPosition();
	while(pos) {
		CSetBuilder *set = sets.GetNext(pos);
		if(!set->IsAbstract())
			set->DumpXML(xml,2);
	}

	pos = conns.GetHeadPosition();
	while(pos) {
		CConnectionBuilder *conn = conns.GetNext(pos);
		if(!conn->IsAbstract())
			conn->DumpXML(xml,2);
	}

	pos = models.GetHeadPosition();
	while(pos) {
		CModelBuilder *model = models.GetNext(pos);
		if(!model->IsAbstract())
			model->DumpXML(xml,2);
	}

	pos = folders.GetHeadPosition();
	while(pos)
		folders.GetNext(pos)->DumpXML(xml,2);

	xml << "   </folder>" << endl;
}

bool CParadigmBuilder::DumpXML(CString filename)
{
	ofstream xml((LPCTSTR)filename);
	if (!xml.is_open())
		return false;
	xml << "<?xml version=\"1.0\"?>" <<	endl << "<!DOCTYPE paradigm SYSTEM \"edf.dtd\">" << endl << endl;
	xml << "<paradigm name =\"" << name << "\">" << endl << endl;
	xml << "   <comment> Generated by the Meta2000 interpreter </comment>" << endl << endl;

	DumpFolders(xml);

	xml << "</paradigm>" << endl;
	xml.close();
	return true;
}

///////////////////////////// CUMLClassBase ///////////////////////////////


///////////////////////// CConstraintFuncBuilder ///////////////////////////
IMPLEMENT_CUSTOMATOM(CConstraintFuncBuilder, CBuilderAtom, "ConstraintFunc");

void CConstraintFuncBuilder::Initialize()
{
	CBuilderAtom::Initialize();

	CString strWS = "[\t ]*";
	CString strID = "[_a-zA-Z][_a-zA-Z0-9]*";
	CString strT = strID + "(::" + strID + ")*";
	CString strP = strID + strWS + ":" + strWS + strT;

	GetAttribute( "CFuncParamList", paramlist );
	Regexp rePL( "^(" + strWS + strP + ")?" + "(" + strWS + "," + strWS + strP + ")*$", TRUE );
	if ( ! rePL.CompiledOK() )
		AfxMessageBox( "Internal Interpreter Error: [REGEXP] " + rePL.GetErrorString() );
	else
		if ( ! rePL.Match( (LPCSTR) paramlist ) )
			AfxMessageBox( "Invalid format for parameterlist in Constraint Function: " + name + "\r\nCorrect format: <paramname> : <paramtype> [, <paramname> : <paramtype>] ..." );

	GetAttribute( "CFuncContext", contexttype );
	if ( contexttype.IsEmpty() ) {
		AfxMessageBox( "No context found for Constraint Function: " + name + "\r\nIt will be Project implicitly." );
		contexttype = "gme::Project";
	}

	GetAttribute( "CFuncReturnType", returntype );
	if ( returntype.IsEmpty() ) {
		AfxMessageBox( "No return type found for Constraint Function: " + name + "\r\nIt will be bool implicitly." );
		returntype = "ocl::Boolean";
	}

	GetAttribute( "CFuncDefinition", definition );
	if ( definition.IsEmpty() )
		AfxMessageBox( "No definition found for Constraint Function: " + name );

	GetAttribute( "CFuncStereotype", stereotype );
	if ( stereotype == "attribute" && ! paramlist.IsEmpty() ) {
		AfxMessageBox( "Attribute cannot have parameters. Parameterlist is ignored for Constraint Function: " + name );
		paramlist.Empty();
	}
}

void CConstraintFuncBuilder::DumpXML(ofstream xml,int depth)
{
	Indent();
	xml << "<constraint type=\"function\" name=\"" << name << "\">" << endl;
	depth++;
	Indent();
	xml << "<![CDATA[" << stereotype << ";" << contexttype << ";" << paramlist << ";" << returntype << ";" << endl << definition << "]]>" << endl;
	depth--;
	Indent();
	xml << "</constraint>" << endl;
}




///////////////////////////// CConstraintBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CConstraintBuilder, CBuilderAtom, "Constraint");

void CConstraintBuilder::Initialize()
{
	CBuilderAtom::Initialize();
	GetAttribute("ConstraintDescription",desc);
	if(desc.IsEmpty())
		AfxMessageBox("No description found for constraint " + name);
	GetAttribute("ConstraintEqn",expr);
	if(expr.IsEmpty())
		AfxMessageBox("No equation found for constraint " + name);
	GetAttribute("Depth",eventDepth);
	CString num;
	GetAttribute("ConstraintPriority",num);
	if(sscanf((LPCTSTR)num,"%d",&priority) != 1)
		priority = 2;
	priority = max(0,min(5,priority));

	eventMask = 0;
	GetEventAttribute("CloseEvent",OBJEVENT_CLOSEMODEL);
	GetEventAttribute("NewChildEvent",OBJEVENT_NEWCHILD);
	GetEventAttribute("LostChildEvent",OBJEVENT_LOSTCHILD);
	GetEventAttribute("CreateEvent",OBJEVENT_CREATED);
	GetEventAttribute("DeleteEvent",OBJEVENT_DESTROYED);
	GetEventAttribute("ConnectEvent",OBJEVENT_CONNECTED);
	GetEventAttribute("DisconnectEvent",OBJEVENT_DISCONNECTED);
	GetEventAttribute("ReferenceEvent",OBJEVENT_REFERENCED);
	GetEventAttribute("UnReferenceEvent",OBJEVENT_REFRELEASED);
	GetEventAttribute("AddSetEvent",OBJEVENT_SETINCLUDED);
	GetEventAttribute("RemoveSetEvent",OBJEVENT_SETEXCLUDED);
	GetEventAttribute("ChangeRelationEvent",OBJEVENT_RELATION);
	GetEventAttribute("ChangeAttributeEvent",OBJEVENT_ATTR);
	GetEventAttribute("ChangePropertyEvent",OBJEVENT_PROPERTIES);
	GetEventAttribute("DeriveEvent",OBJEVENT_SUBT_INST);
	GetEventAttribute("MoveEvent",OBJEVENT_PARENT);
}

void CConstraintBuilder::GetEventAttribute(CString eventName,unsigned int eventFlag)
{
	bool flag;
	GetAttribute(eventName,flag);
	if(flag)
		eventMask |= eventFlag;
}

void CConstraintBuilder::Attach()
{
	CBuilderObjectList objs;
	GetOutConnectedObjects("HasConstraint",objs);
	SortList( objs );
	POSITION pos = objs.GetHeadPosition();
	if (pos) {
		while(pos) {
			CBuilderObject *obj = objs.GetNext(pos);
			CFCOClassBase *fcoClass;
			CObjectClassBase *objectProxy = dynamic_cast<CObjectClassBase *>(obj);
			if (objectProxy && objectProxy->real)
				fcoClass = objectProxy->real;
			else
				fcoClass = dynamic_cast<CFCOClassBase *>(obj);
			if(fcoClass == 0)
				AfxMessageBox("Unexpected object " + obj->GetName() + " contains constraint " + name,MB_OK | MB_ICONSTOP);
			else
				fcoClass->AddConstraint(this);
		}
	}
	else {
		theParadigm->AddConstraint(this);
	}
}

void CConstraintBuilder::DumpXML(ofstream xml,int depth)
{
	Indent();
	char mask[64];
	sprintf(mask,"%x",eventMask);
	xml << "<constraint name=\"" << name << "\" eventmask = \"0x" << mask << "\" depth = \"" << eventDepth;
	xml << "\" priority = \"" << priority << "\">" << endl;
	depth++;
	Indent();
	xml << "<![CDATA[" << expr << "]]>" << endl;
	Indent();
	::DumpDispName(xml,desc);
	xml << endl;
	depth--;
	Indent();
	xml << "</constraint>" << endl;
}


///////////////////////////// CFolderBuilder ///////////////////////////////
void CFolderBase::Initialize()
{
		builder = dynamic_cast<CBuilderObject*>(this);
		real = NULL;
}


IMPLEMENT_CUSTOMATOMREF(CFolderProxy, CBuilderAtomReference, "FolderProxy");

void CFolderProxy::Initialize()
{
	CBuilderAtomReference::Initialize();
	CFolderBase::Initialize();
}


IMPLEMENT_CUSTOMATOM(CFolderBuilder, CBuilderAtom, "Folder");

void CFolderBuilder::Initialize()
{
	CBuilderAtom::Initialize();
	CFolderBase::Initialize();

	GetAttribute("InRootFolder",root);

	GetAttribute("DisplayedName",dispName);
	if(dispName.IsEmpty())
		dispName = name;
}

void CFolderBuilder::GatherProxies()
{
	parts.AddTail(this);
	CBuilderObjectList refs;
	GetReferencedBy(refs);
	POSITION pos = refs.GetHeadPosition();
	while (pos) {
		CFolderProxy *ref = dynamic_cast<CFolderProxy *>(refs.GetNext(pos));
		ASSERT(ref);
		ref->real = this;
		parts.AddTail(ref);
	}
}

void CFolderBuilder::BuildComposition()
{
	CBuilderObjectList children;
	POSITION pos = parts.GetHeadPosition();
	while (pos) {
		CBuilderObjectList partchildren;
		parts.GetNext(pos)->builder->GetInConnectedObjects("FolderContainment",partchildren);
		children.AddTail(&partchildren);
	}
	SortList( children );
	if(children.IsEmpty())
		AfxMessageBox("Folder " + name + " cannot contain any folders or objects");
	else {
		POSITION pos = children.GetHeadPosition();
		while(pos) {
			CBuilderObject *child = children.GetNext(pos);
			CFolderBuilder *folder = dynamic_cast<CFolderBuilder *>(child);
			CFCOClassBase *fco = dynamic_cast<CFCOClassBase *>(child);
			CObjectClassBase *fcoproxy = dynamic_cast<CObjectClassBase *>(child);
			CFolderBase *folderproxy = dynamic_cast<CFolderBase *>(child);

			if (folderproxy && folderproxy->real)
					folder = folderproxy->real;
			if (fcoproxy && fcoproxy->real)
					fco = dynamic_cast<CFCOClassBase*>(fcoproxy->real);


			if(folder) {
				subfolders.AddTail(folder);
			}
			else if(fco) {
				CFCOClassBaseList subs;
				fco->FindAllSubClasses(subs,true,false,interf);
				POSITION mpos = subs.GetHeadPosition();
				while(mpos) {
					// CModelBuilder *sub = dynamic_cast<CModelBuilder *>(subs.GetNext(mpos));
					// if(sub)
					//	models.AddTail(sub);
					fcos.AddTail(subs.GetNext(mpos));
				}
			}
			else
				AfxMessageBox("Unexpected object " + child->GetName() + " in folder " + name + " composition");
		}
	}
}

////////////////////////////////////////////////////
void CFolderBuilder::FindSameFolders()
{
	POSITION ppos = parts.GetHeadPosition();
	while (ppos) {
		bool leftIsProxy = false;
		CBuilderObject *part = parts.GetAt(ppos)->builder;
		if (part->IsKindOf(RUNTIME_CLASS(CFolderProxy)))
			leftIsProxy = true;
		CBuilderObjectList diamonds;
		part->GetInConnectedObjects("SameFolderLeft", diamonds);
		SortList ( diamonds );
		FindSameFolders(diamonds, leftIsProxy);

		diamonds.RemoveAll();
		part->GetOutConnectedObjects("SameFolderLeft", diamonds);
		SortList ( diamonds );
		FindSameFolders(diamonds, leftIsProxy);
		parts.GetNext(ppos);
	}
}

void CFolderBuilder::FindSameFolders(CBuilderObjectList &diamonds, bool leftIsProxy)
{
	POSITION pos = diamonds.GetHeadPosition();
	while(pos) {
		CBuilderObject *diamond = diamonds.GetNext(pos);
		CBuilderObjectList sames;
		diamond->GetInConnectedObjects("SameFolderRight",sames);
		SortList( sames );
		JoinSameFolders(sames, leftIsProxy, diamond->GetName());
		sames.RemoveAll();
		diamond->GetOutConnectedObjects("SameFolderRight",sames);
		SortList( sames );
		JoinSameFolders(sames, leftIsProxy, diamond->GetName());
	}
}

void CFolderBuilder::JoinSameFolders(CBuilderObjectList &samefldrs, bool leftIsProxy, CString commonName)
{
	POSITION spos = samefldrs.GetHeadPosition();
	while (spos) {
		bool rightIsProxy = false;
		CBuilderObject *same = samefldrs.GetNext(spos);
		CFolderBuilder *samefolder = dynamic_cast<CFolderBuilder *>(same);
		CFolderBase *sameproxy = dynamic_cast<CFolderBase *>(same);
		if (sameproxy && sameproxy->real) {
			samefolder = sameproxy->real;
			if (same->IsKindOf(RUNTIME_CLASS(CFolderProxy)))
				rightIsProxy = true;
		}
		if (!this->IsKindOf(samefolder->GetRuntimeClass())) {
			AfxMessageBox("Equivalence between classes with different types ! Class: " + this->GetName() + " cannot be equal to class: " +samefolder->GetName());
		}
		else if (samefolder != this) {
			if ( (rightIsProxy && leftIsProxy) || ((!rightIsProxy) && (!leftIsProxy)) ) {
				if (commonName == "SameFolder")
					AfxMessageBox("Equivalence between two folders. Both are proxy or real.\nIn this case the name of the equivalence operator will be given to the common folder.\nGive it a bit more meaningful name than \"SameFolder\"", MB_OK | MB_ICONEXCLAMATION);
				this->name = commonName;
			}
			else if (leftIsProxy) {
				this->name = samefolder->GetName();
			}

			POSITION ppos = samefolder->parts.GetHeadPosition();
			while (ppos) {
				CFolderBase *ppart = samefolder->parts.GetNext(ppos);
				if (ppart) {
					ppart->real = this;
					parts.AddTail(ppart);
				}
			}
			theParadigm->DiscardFolder(samefolder);
		}
	}
}

bool CFolderBuilder::IsEmpty()
{
	if (fcos.GetCount() > 0) {
		return false;
	}
	POSITION pos = subfolders.GetHeadPosition();
	while (pos) {
		if (!(subfolders.GetNext(pos)->IsEmpty())) {
			return false;
		}
	}
	return true;
}
////////////////////////////////////////////////////


void CFolderBuilder::DumpXML(ofstream xml,int depth)
{
	Indent();
	xml << "<folder name = \"" << name << "\" ";

	DumpMetaRef(xml, this);
	POSITION pos = subfolders.GetHeadPosition();
	CString subfolderNames;
	while(pos) {
		CFolderBuilder *subf = subfolders.GetNext(pos);
		subfolderNames += subf->GetName();
		if(pos)
			subfolderNames += " ";
	}

	if(!subfolderNames.IsEmpty())
		xml << "subfolders = \"" << subfolderNames << "\" ";

	pos = fcos.GetHeadPosition();
	CString fcoNames;
	while(pos) {
		CFCOClassBase *fco = fcos.GetNext(pos);
		if(!fco->IsAbstract()) {
			fcoNames += fco->GetName();
			if(pos)
				fcoNames += " ";
		}
	}
	if(!fcoNames.IsEmpty())
		xml << " rootobjects = \"" <<	fcoNames << "\"";

	xml << ">";
	xml << endl;

	// DumpCardinalities(xml, depth); Needs some refactoring


	depth++;
	Indent();
	::DumpDispName(xml,dispName);
	xml << endl;
	depth--;

	Indent();
	xml << "</folder>" << endl;
}

///////////////////////////// CObjectClassBase ///////////////////////////////
void CObjectClassBase::Initialize()
{
	builder = dynamic_cast<CBuilderObject*>(this);
	real = NULL;
	flag = false;
	if(!builder->GetAttribute("IsAbstract",abstract))
		AfxMessageBox("Cannot get IsAbstract attribute of " + builder->GetName());
}


///////////////////////////// CObjectProxyBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOMREF(CObjectProxyBuilder, CBuilderAtomReference, "AtomProxy,ConnectionProxy,ModelProxy,ReferenceProxy,SetProxy,FCOProxy");

void CObjectProxyBuilder::Initialize()
{
	CBuilderAtomReference::Initialize();
	CObjectClassBase::Initialize();
}

///////////////////////////// CFCOClassBase ///////////////////////////////
IMPLEMENT_CUSTOMATOMBASE(CFCOClassBase, CBuilderAtom);



void CFCOClassBase::Initialize()
{
	CBuilderAtom::Initialize();
	CObjectClassBase::Initialize();

	GetAttribute("NamePosition",namePos);
	namePos = min(8,max(0,namePos));
	GetAttribute("InRootFolder",root);

	GetAttribute("DisplayedName",dispName);
	if(dispName.IsEmpty())
		dispName = name;
}


void CFCOClassBase::GatherProxies()
{
	parts.AddTail(this);
	CBuilderObjectList refs;
	GetReferencedBy(refs);
	POSITION pos = refs.GetHeadPosition();
	while (pos) {
		CObjectProxyBuilder *ref = dynamic_cast<CObjectProxyBuilder *>(refs.GetNext(pos));
		ASSERT(ref);
		ref->real = this;
		parts.AddTail(ref);
		if (ref->IsAbstract())
			abstract = true;
	}
}


void CFCOClassBase::AddRole(CRole *role)
{
	role->part = this;
	roles.AddTail(role);
}

void CFCOClassBase::RegisterSubClasses(CBuilderObjectList &subs, inheritance_type itype)
{
	POSITION pos = subs.GetHeadPosition();
	while(pos) {
		CBuilderObject *obj = subs.GetNext(pos);
		CFCOClassBase *subFcoClass = dynamic_cast<CFCOClassBase *>(obj);
		CObjectClassBase *subProxy = dynamic_cast<CObjectClassBase *>(obj);
		if(subProxy && subProxy->real) {
			subFcoClass = subProxy->real;
		}
		if(subFcoClass == 0)
			AfxMessageBox("Unexpected subclass " + obj->GetName() + " of " + name,MB_OK | MB_ICONSTOP);
		else {
			switch (itype) {
			case full:
				if (subClasses.Find(subFcoClass) == NULL) {
					subFcoClass->baseClasses.AddTail(this);
					subClasses.AddTail(subFcoClass);
				}
				break;
			case interf:
				if (if_subClasses.Find(subFcoClass) == NULL) {
					subFcoClass->if_baseClasses.AddTail(this);
					if_subClasses.AddTail(subFcoClass);
				}
				break;
			case implement:
				if (imp_subClasses.Find(subFcoClass) == NULL) {
					subFcoClass->imp_baseClasses.AddTail(this);
					imp_subClasses.AddTail(subFcoClass);
				}
				break;
			}
		}
	}
}

void CFCOClassBase::FindAllSubClasses(CFCOClassBaseList &subs,bool includeSelf,bool includeAbstract, inheritance_type itype)
{
	try {
		if(includeSelf && (includeAbstract || !abstract))
			subs.AddTail(this);

		CFCOClassBaseList tmpsubClasses;

		switch (itype)
		{
		case interf:
			tmpsubClasses.AddTail(&if_subClasses);
			tmpsubClasses.AddTail(&subClasses);
			break;
		case implement:
			tmpsubClasses.AddTail(&imp_subClasses);
			tmpsubClasses.AddTail(&subClasses);
			break;
		case full:
			tmpsubClasses.AddTail(&subClasses);
			break;
		}

		POSITION pos = tmpsubClasses.GetHeadPosition();
		while(pos) {
			tmpsubClasses.GetNext(pos)->FindAllSubClasses(subs,true,includeAbstract, itype);
		}
	}
	catch(...) {
		AfxMessageBox("Unhandled exception in CFCOClassBase::FindAllSubclasses()",MB_OK | MB_ICONSTOP);
	}
	SortList( subs );
}

bool CFCOClassBase::CheckInheritance(CStringList &trace)
{
	if(flag) {
		flag = false;
		return false;
	}
	flag = true;


	trace.AddTail(name + " [Full Inheritance]");
	POSITION pos = subClasses.GetHeadPosition();
	while(pos) {
		if(!subClasses.GetNext(pos)->CheckInheritance(trace)) {
			flag = false;
			return false;
		}
	}

	trace.RemoveTail();
	trace.AddTail(name + " [Interface Inheritance]");
	pos = if_subClasses.GetHeadPosition();
	while(pos) {
		if(!if_subClasses.GetNext(pos)->CheckInheritance(trace)) {
			flag = false;
			return false;
		}
	}

	trace.RemoveTail();
	trace.AddTail(name + " [Implementation Inheritance]");
	pos = imp_subClasses.GetHeadPosition();
	while(pos) {
		if(!imp_subClasses.GetNext(pos)->CheckInheritance(trace)) {
			flag = false;
			return false;
		}
	}

	trace.RemoveTail();
	flag = false;
	return true;
}

void CFCOClassBase::FindSubClasses(CBuilderObjectList &triangles, inheritance_type itype)
{
	CString typestr;
	switch (itype) {
	case full:
		typestr = "DerivedInheritance";
		break;
	case interf:
		typestr = "DerivedIntInheritance";
		break;
	case implement:
		typestr = "DerivedImpInheritance";
		break;
	}
	POSITION pos = triangles.GetHeadPosition();
	while(pos) {
		CBuilderObject *triangle = triangles.GetNext(pos);
		CBuilderObjectList subs;
		triangle->GetInConnectedObjects(typestr,subs);
		SortList( subs );
		RegisterSubClasses(subs, itype);
		subs.RemoveAll();
		triangle->GetOutConnectedObjects(typestr,subs);
		SortList( subs );
		RegisterSubClasses(subs, itype);
	}
}

void CFCOClassBase::FindSubClasses()
{
	POSITION ppos = parts.GetHeadPosition();
	while(ppos) {
		CBuilderObject *part = parts.GetNext(ppos)->builder;
		CBuilderObjectList triangles;
		part->GetInConnectedObjects("BaseInheritance",triangles);
		SortList( triangles );
		FindSubClasses(triangles, full);
		triangles.RemoveAll();
		part->GetOutConnectedObjects("BaseInheritance",triangles);
		SortList( triangles );
		FindSubClasses(triangles, full);


		triangles.RemoveAll();
		part->GetInConnectedObjects("BaseIntInheritance",triangles);
		SortList( triangles );
		FindSubClasses(triangles, interf);
		triangles.RemoveAll();
		part->GetOutConnectedObjects("BaseIntInheritance",triangles);
		SortList( triangles );
		FindSubClasses(triangles, interf);

		triangles.RemoveAll();
		part->GetInConnectedObjects("BaseImpInheritance",triangles);
		SortList( triangles );
		FindSubClasses(triangles, implement);
		triangles.RemoveAll();
		part->GetOutConnectedObjects("BaseImpInheritance",triangles);
		SortList( triangles );
		FindSubClasses(triangles, implement);

	}
}

void CFCOClassBase::FindEQClasses()
{
	POSITION ppos = parts.GetHeadPosition();
	while (ppos) {
		bool leftIsProxy = false;
		CBuilderObject *part = parts.GetAt(ppos)->builder;
		if(part->IsKindOf(RUNTIME_CLASS(CObjectProxyBuilder)))
			leftIsProxy = true;
		CBuilderObjectList diamonds;
		part->GetInConnectedObjects("EquivLeft", diamonds);
		SortList ( diamonds );
		FindEQClasses(diamonds, leftIsProxy);

		diamonds.RemoveAll();
		part->GetOutConnectedObjects("EquivLeft", diamonds);
		SortList ( diamonds );
		FindEQClasses(diamonds, leftIsProxy);
		parts.GetNext(ppos);
	}
}

void CFCOClassBase::FindEQClasses(CBuilderObjectList &diamonds, bool leftIsProxy)
{
	POSITION pos = diamonds.GetHeadPosition();
	while(pos) {
		CBuilderObject *diamond = diamonds.GetNext(pos);
		CBuilderObjectList eqs;
		diamond->GetInConnectedObjects("EquivRight",eqs);
		SortList( eqs );
		JoinEQClasses(eqs, leftIsProxy, diamond->GetName());
		eqs.RemoveAll();
		diamond->GetOutConnectedObjects("EquivRight",eqs);
		SortList( eqs );
		JoinEQClasses(eqs, leftIsProxy, diamond->GetName());
	}
}

void CFCOClassBase::JoinEQClasses(CBuilderObjectList &eqs, bool leftIsProxy, CString commonName)
{
	POSITION epos = eqs.GetHeadPosition();
	while (epos) {
		bool rightIsProxy = false;
		CBuilderObject *eq = eqs.GetNext(epos);
		CFCOClassBase *eqfco = dynamic_cast<CFCOClassBase *>(eq);
		CObjectClassBase *eqproxy = dynamic_cast<CObjectClassBase *>(eq);
		if (eqproxy && eqproxy->real) {
			eqfco = eqproxy->real;
			if (eq->IsKindOf(RUNTIME_CLASS(CObjectProxyBuilder)))
				rightIsProxy = true;
		}
		if (!this->IsKindOf(eqfco->GetRuntimeClass())) {
			AfxMessageBox("Equivalence between classes with different types ! Class: " + this->GetName() + " cannot be equal to class: " +eqfco->GetName());
		}
		else if (eqfco != this) {
			if (((!eqfco->IsAbstract()) && (!this->IsAbstract())) || (eqfco->IsAbstract() && this->IsAbstract())) {
				if ( (rightIsProxy && leftIsProxy) || ((!rightIsProxy) && (!leftIsProxy)) ) {
					if (commonName == "Equivalence" )
						AfxMessageBox("Equivalence between two classes. The interpreter cannot select, which is the stronger one.\nIn this case the name of the equivalence operator will be given to the common class.\nGive it a bit more meaningful name than \"Equivalence\"", MB_OK | MB_ICONEXCLAMATION);
					this->name = commonName;
				}
				else if (leftIsProxy) {
					this->name = eqfco->GetName();
				}
			}
			else if (this->IsAbstract()) {
				this->name = eqfco->GetName();
			}

			this->abstract = (eqfco->IsAbstract() && this->IsAbstract());

			POSITION ppos = eqfco->parts.GetHeadPosition();
			while (ppos) {
				CObjectClassBase *ppart = eqfco->parts.GetNext(ppos);
				if (ppart) {
					ppart->real = this;
					parts.AddTail(ppart);
				}
			}
			theParadigm->DiscardFCOClass(eqfco);
		}

	}

}

void CFCOClassBase::FindAttributes(CAttributeBuilderList &attrList)
{
	{
		POSITION pos = baseClasses.GetHeadPosition();
		while(pos)
			baseClasses.GetNext(pos)->FindAttributes(attrList);

		pos = imp_baseClasses.GetHeadPosition();
		while(pos)
			imp_baseClasses.GetNext(pos)->FindAttributes(attrList);
	}

	CBuilderObjectList objs;
	POSITION ppos = parts.GetHeadPosition();
	while (ppos) {
		CObjectClassBase *part = parts.GetNext(ppos);
		part->builder->GetInConnectedObjects("HasAttribute",objs);
		SortList( objs );
		POSITION pos = objs.GetHeadPosition();
		while(pos) {
			CBuilderObject *obj = objs.GetNext(pos);
			CAttributeBuilder *attr = dynamic_cast<CAttributeBuilder *>(obj);
			if(attr == 0)
				AfxMessageBox("Unexpected attribute " + obj->GetName() + " in " + name);
			else {
				if(!attrList.Find(attr))
					attrList.AddTail(attr);
			}
		}
	}
}

void CFCOClassBase::FindAttributes()
{
	FindAttributes(attrs);
	OrderAttributes();
}

CAttributeBuilder *CFCOClassBase::FindAttribute(CString nm)
{
	POSITION pos = attrs.GetHeadPosition();
	while(pos) {
		CAttributeBuilder *attr = attrs.GetNext(pos);
		if(attr->GetName() == nm)
			return attr;
	}
	return 0;
}


//	This method finds the "last" attribute to be sorted.
//	it looks through all of the attributes and finds
//	the one that is highest on the screen.  in the
//	event it is tied for being highest, then it
//	uses the one with the rightmost coordinate
CAttributeBuilder *CFCOClassBase::FindLastAttribute()
{
	CAttributeBuilder *found = attrs.GetHead();
	POSITION pos = attrs.GetHeadPosition();
	while(pos) {
		CAttributeBuilder *attr = attrs.GetNext(pos);
		if(attr->Y() >= found->Y())
		{
			if( attr->Y( ) == found->Y( ) )
			{
				if( attr->X( ) > found->X( ) )
				{
					found = attr;
				}
			}
			else
			{
				found = attr;
			}
		}
	}
	return found;
}

void CFCOClassBase::OrderAttributes()
{
	if(attrs.IsEmpty())
		return;

	CAttributeBuilderList ordered;
	int n = attrs.GetCount();
	for(int i = 0; i < n; i++) {
		CAttributeBuilder *found = FindLastAttribute();
		ordered.AddTail(found);
		POSITION pos = attrs.Find(found);
		ASSERT(pos);
		attrs.RemoveAt(pos);
	}

	POSITION pos = ordered.GetHeadPosition();
	while(pos)
		attrs.AddHead(ordered.GetNext(pos));
}

void CFCOClassBase::AddConstraint(CConstraintBuilder *constraint)
{
	if(!IsAbstract())
		constraints.AddTail(constraint);
	POSITION pos = subClasses.GetHeadPosition();
	while(pos)
		subClasses.GetNext(pos)->AddConstraint(constraint);
}

void CFCOClassBase::SetFlags(CFCOClassBaseList &list,bool val)
{
	POSITION pos = list.GetHeadPosition();
	while(pos)
		list.GetNext(pos)->flag = val;
}

void CFCOClassBase::DumpHeader(ofstream xml,int depth,char *kind)
{
	Indent();
	xml << "<" << kind << " name = \"" << name << "\" ";
	DumpMetaRef(xml, this);
	DumpAttributeNames(xml);
	xml << ">" << endl;
}

void CFCOClassBase::DumpAttributeNames(ofstream xml,bool checkViewable)
{
	if(!attrs.IsEmpty()) {
		CString names;
		POSITION pos = attrs.GetHeadPosition();
		while(pos) {
			CAttributeBuilder *attr = attrs.GetNext(pos);
			if(!checkViewable || attr->IsViewable()) {
				names += attr->GetName();
				if(pos)
					names += " ";
			}
		}
		if(!names.IsEmpty())
			xml << " attributes = \"" << names << "\"";
	}
}

void CFCOClassBase::DumpAttributes(ofstream xml,int depth)
{
	POSITION pos = attrs.GetHeadPosition();
	while(pos) {
		CAttributeBuilder *attr = attrs.GetNext(pos);
		if(!attr->IsGlobal())
			attr->DumpXML(xml,depth, GetName());
	}
}

void CFCOClassBase::DumpGeneralPreferences(ofstream xml,int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpGeneralPreferences(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpGeneralPreferences(xml,depth);

	CString prefs;
	GetAttribute("GeneralPreferences",prefs);
	if(prefs.IsEmpty())
		return;

	bool berr = false;
	CStringArray prefArr;
	CTokenEx	 tok;
	tok.Split(prefs, _T("\n"), prefArr);
	for (int i = 0; i < prefArr.GetSize(); i++) {
		CStringArray prefPair;
		tok.Split(prefArr[i], _T("="), prefPair);
		// Compatibility with the older format
		if (prefPair.GetSize() != 2) {
			prefPair.RemoveAll();
			tok.Split(prefArr[i], _T(","), prefPair);
			if (prefPair.GetSize() == 2) {
				AfxMessageBox("Warning: Deprecated general preferences format for " + name + "\nValid format:\n\t<prefname1> = <value1>\n\t<prefname2> = <value2>\n\t...",MB_OK | MB_ICONEXCLAMATION);
			}
		}
		if (prefPair.GetSize() == 2) {
			prefPair[0].TrimLeft();
			prefPair[0].TrimRight();
			prefPair[1].TrimLeft();
			prefPair[1].TrimRight();
			Indent();
			xml << 	 "<regnode name = \"" << prefPair[0] << "\" value =\"" << prefPair[1] << "\"></regnode>" << endl;
		}
		else {
			berr = true;
			break;
		}
	}
	if(berr) {
		AfxMessageBox("Invalid general preferences specification for " + name + "\nValid format:\n\t<prefname1> = <value1>\n\t<prefname2> = <value2>\n\t...",MB_OK | MB_ICONSTOP);
	}

	/* Old format ! Ugly and now deprecated
	char string[4096];
	strcpy(string,(LPCSTR)prefs);
	char seps[16];
	sprintf(seps,",%c\n",13);
	char *token;

	CString nm;
	CString val;
	token = strtok(string,seps);
	bool first = true;
	while(token) {
		while(isspace(*token) && *token)
			token++;
		if(*token) {
			(first ? nm : val).Format("%s",token);
			if(!first) {
				Indent();
				xml << 	 "<regnode name = \"" << nm << "\" value =\"" << val << "\"></regnode>" << endl;
			}
			first = !first;
		}
		token = strtok(0,seps);
	}
	if(!first)
		AfxMessageBox("Incomplete general preferences specification for " + name,MB_OK | MB_ICONSTOP);
	*/
}

void CFCOClassBase::DumpDispName(ofstream xml,int depth)
{
	CString dispName;
	GetAttribute("DisplayedName",dispName);
	if(dispName.IsEmpty())
		dispName = name;
	Indent();
	::DumpDispName(xml,dispName);
	xml << endl;
}

void CFCOClassBase::DumpNamePosition(ofstream xml,int depth)
{
	Indent();
	xml << "<regnode name = \"namePosition\" value =\"" << namePos << "\"></regnode>" << endl;
}

void CFCOClassBase::DumpConstraints(ofstream xml,int depth)
{
	POSITION pos = constraints.GetHeadPosition();
	while(pos)
		constraints.GetNext(pos)->DumpXML(xml,depth);
}

void CFCOClassBase::DumpConnCardinalities(ofstream xml,int depth)
{
	POSITION pos = theParadigm->GetConnections().GetHeadPosition();
	while (pos) {

		CConnectionBuilder *conn = theParadigm->GetConnections().GetNext(pos);

		if ( ! conn->dstRole.IsEmpty() && ! conn->srcRole.IsEmpty() ) {

			if ( conn->dst == this && ! conn->dstRole.IsEmpty() ) {

				// Build Name, EventMask, Description

				CString strCName;
				strCName.Format( "Valid%sDstCardinality%d", conn->GetName(), theParadigm->GetNextConstraintID() );

				int iEventMask = 0;
				char chMask[64];
				sprintf( chMask, "%x", iEventMask );

				CString strDesc;
				strDesc.Format( "Multiplicity of objects, which are associated to %s as \"%s\" over %s, has to match %s.", GetName(), conn->dstRole, conn->GetName(), conn->dstCard );

				// Parse Cardinality, build Expression

				CString strExprBegin;
				strExprBegin.Format( "let dstCount = self.attachingConnections( \"%s\", %s ) -> size in\n", conn->dstRole, conn->GetName( ) );

				CString strExprEnd;
				CString strCardContext;
				strCardContext.Format("[connection] Connection: %s, Conn.Role: %s", conn->GetName(), conn->dstRole);
				if ( ParseCardinality( conn->dstCard, "dstCount", strCardContext, strExprEnd ) )
					AfxMessageBox("Ignoring invalid cardinality string in connection: " + conn->GetName() + "\nString: " + conn->dstCard, MB_ICONSTOP);

				// Dump constraint if cardinality is appropriate

				if ( ! strExprEnd.IsEmpty() ) {
					Indent();
					xml << "<constraint name=\"" << strCName << "\" eventmask = \"0x" << chMask << "\" depth = \"" << 0 << "\" priority = \"" << 2 << "\">" << endl;
					depth++;
					Indent();
					xml << "<![CDATA[" << strExprBegin << strExprEnd << "]]>" << endl;
					Indent();
					::DumpDispName(xml,strDesc);
					xml << endl;
					depth--;
					Indent();
					xml << "</constraint>" << endl;
				}
			}

			if ( conn->src == this && ! conn->srcRole.IsEmpty() ) {

				// Build Name, EventMask, Description

				CString strCName;
				strCName.Format( "Valid%sSrcCardinality%d", conn->GetName(), theParadigm->GetNextConstraintID() );

				int iEventMask = 0;
				char chMask[64];
				sprintf( chMask, "%x", iEventMask );

				CString strDesc;
				strDesc.Format( "Multiplicity of objects, which are associated to %s as \"%s\" over %s, has to match %s.", GetName(), conn->srcRole, conn->GetName(), conn->srcCard );

				// Parse Cardinality, build Expression

				CString strExprBegin;
				strExprBegin.Format( "let srcCount = self.attachingConnections( \"%s\", %s ) -> size in\n", conn->srcRole, conn->GetName( ) );

				CString strExprEnd;
				CString strCardContext;
				strCardContext.Format("[connection] Connection: %s, Conn.Role: %s", conn->GetName(), conn->srcRole);
				if ( ParseCardinality( conn->srcCard, "srcCount", strCardContext, strExprEnd ) )
					AfxMessageBox("Ignoring invalid cardinality string in connection: " + conn->GetName() + "\nString: " + conn->srcCard, MB_ICONSTOP);

				// Dump constraint if cardinality is appropriate

				if ( ! strExprEnd.IsEmpty() ) {
					Indent();
					xml << "<constraint name=\"" << strCName << "\" eventmask = \"0x" << chMask << "\" depth = \"" << 0 << "\" priority = \"" << 2 << "\">" << endl;
					depth++;
					Indent();
					xml << "<![CDATA[" << strExprBegin << strExprEnd << "]]>" << endl;
					Indent();
					::DumpDispName(xml,strDesc);
					xml << endl;
					depth--;
					Indent();
					xml << "</constraint>" << endl;
				}
			}
		}	// If Connection roles are not empty
	}	// Iteration over Connections

}


void CFCOClassBase::DumpFooter(ofstream xml,int depth,char *kind)
{
	Indent();
	xml << "</" << kind << ">" << endl;
}

void CFCOClassBase::DumpXML(ofstream xml,int depth)
{
	VERIFY(false);
}

void CFCOClassBase::DumpIcon(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpIcon(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpIcon(xml,depth);

	CString icon;
	GetAttribute("Icon",icon);
	if(icon && !icon.IsEmpty()) {
		Indent();
		xml << 	 "<regnode name = \"icon\" value =\"" << icon << "\"></regnode>" << endl;
	}
}

void CFCOClassBase::DumpPortIcon(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpPortIcon(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpPortIcon(xml,depth);

	CString icon;
	GetAttribute("PortIcon",icon);
	if(icon && !icon.IsEmpty()) {
		Indent();
		xml << 	 "<regnode name = \"porticon\" value =\"" << icon << "\"></regnode>" << endl;
	}

}

void CFCOClassBase::DumpDecorator(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpDecorator(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpDecorator(xml,depth);

	CString decor;
	GetAttribute("Decorator",decor);
	if(decor && !decor.IsEmpty()) {
		Indent();
		xml << 	 "<regnode name = \"decorator\" value =\"" << decor << "\"></regnode>" << endl;
	}

}

void CFCOClassBase::DumpHotspotEnabled(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpHotspotEnabled(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpHotspotEnabled(xml,depth);

	bool isHotspotEnabled;
	GetAttribute("IsHotspotEnabled",isHotspotEnabled);
	if(!isHotspotEnabled) {
		Indent();
		xml << 	 "<regnode name = \"isHotspotEnabled\" value =\"false\"></regnode>" << endl;
	}

}

void CFCOClassBase::DumpTypeDisplayed(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpTypeDisplayed(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpTypeDisplayed(xml,depth);

	bool isTypeDisplayed;
	GetAttribute("IsTypeShown", isTypeDisplayed );
	if( isTypeDisplayed ) {
		Indent();
		xml << 	 "<regnode name = \"isTypeShown\" value =\"true\"></regnode>" << endl;
	}

}

void CFCOClassBase::DumpSubtypeIcon(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpSubtypeIcon(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpSubtypeIcon(xml,depth);

	CString strIcon;
	GetAttribute("SubTypeIcon", strIcon );
	strIcon.TrimLeft();
	strIcon.TrimRight();
	if ( ! strIcon.IsEmpty() ) {
		Indent();
		xml << 	 "<regnode name = \"subTypeIcon\" value =\"" << strIcon << "\"></regnode>" << endl;
	}

}

void CFCOClassBase::DumpInstanceIcon(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpInstanceIcon(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpInstanceIcon(xml,depth);

	CString strIcon;
	GetAttribute("InstanceIcon", strIcon );
	strIcon.TrimLeft();
	strIcon.TrimRight();
	if ( ! strIcon.IsEmpty() ) {
		Indent();
		xml << 	 "<regnode name = \"subTypeIcon\" value =\"" << strIcon << "\"></regnode>" << endl;
	}

}

void CFCOClassBase::DumpNameWrapNum(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpNameWrapNum(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpNameWrapNum(xml,depth);

	CString strNum;
	GetAttribute("NameWrapNum", strNum );
	strNum.TrimLeft();
	strNum.TrimRight();
	if ( ! strNum.IsEmpty() && strNum != "0" ) {
		Indent();
		xml << 	 "<regnode name = \"nameWrap\" value =\"" << strNum << "\"></regnode>" << endl;
	}

}

void CFCOClassBase::DumpNameEnabled(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos)
		baseClasses.GetNext(pos)->DumpNameEnabled(xml,depth);

	pos = imp_baseClasses.GetHeadPosition();
	while(pos)
		imp_baseClasses.GetNext(pos)->DumpNameEnabled(xml,depth);

	bool bIsEnabled;
	GetAttribute("IsNameEnabled", bIsEnabled );
	if ( ! bIsEnabled ) {
		Indent();
		xml << 	 "<regnode name = \"isNameEnabled\" value =\"false\"></regnode>" << endl;
	}

}

///////////////////////////// CFCOBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CFCOBuilder, CFCOClassBase, "FCO");

void CFCOBuilder::Initialize()
{
	CFCOClassBase::Initialize();
	if (!IsAbstract()) {
		abstract = true;
		AfxMessageBox("Warning: Non-abstract FCO (" + name + ") has been found.\nMetaInterpreter will handle it as abstract.");
	}
}

///////////////////////////// CModelBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CModelBuilder, CFCOClassBase, "Model");

void CModelBuilder::Initialize()
{
	CFCOClassBase::Initialize();
}

CModelBuilder::~CModelBuilder()
{
	DeleteMembers(roles);
}

//void CModelBuilder::RegisterRole(CFCOClassBase *parent,
void CModelBuilder::RegisterRole(CFCOClassBase *part,
								 CBuilderConnection *conn,
								 bool single)
{
	CRole *role = new CRole;
	if(!conn->GetAttribute("Rolename",role->baseName))
		AfxMessageBox("Cannot find Rolename attribute of " + name);
	role->baseName.TrimLeft( );
	role->baseName.TrimRight( );
//#pragma bookmark( "Changed policy on name mangling for roles to be what it used to be..." )
	//#pragma bookmark( "Changed policy on name mangling for roles..." )
	//old/new
	role->name = role->baseName.IsEmpty() ? part->GetName() : (single ? CString() : part->GetName()) + role->baseName;
	//new!/old
//	role->name = role->baseName.IsEmpty() ? part->GetName() : part->GetName() + role->baseName;
	role->parent = this;
	part->AddRole(role);
	role->conn = conn;
	conn->GetAttribute("IsPort",role->linked);
	conn->GetAttribute("Cardinality", role->cardinality);
	roles.AddTail(role);
}

void CModelBuilder::FindRoles(CFCOClassBaseList &containedparts,
							  CBuilderConnection *conn)
//							  CFCOClassBase *parent)
{
	CFCOClassBaseList containers;
	FindAllSubClasses(containers,true,false, implement);
	POSITION pos = containers.GetHeadPosition();
	while(pos) {
		CFCOClassBase *fcoClass = containers.GetNext(pos);
		CModelBuilder *model = dynamic_cast<CModelBuilder *>(fcoClass);
		if(model == 0)
			AfxMessageBox("Unexpected inheritance: " + name + " - " + fcoClass->GetName());
		else {
			POSITION pos = containedparts.GetHeadPosition();
			while(pos)
//				model->RegisterRole(parent,parts.GetNext(pos),conn,parts.GetCount() == 1);
				model->RegisterRole(containedparts.GetNext(pos),conn,containedparts.GetCount() == 1);
		}
	}
}

void CModelBuilder::FindRoles()
{
	POSITION ppos = parts.GetHeadPosition();
	while (ppos) {
		CObjectClassBase *part = parts.GetNext(ppos);
		const CBuilderConnectionList *other = part->builder->GetInConnections("Containment");
		CBuilderConnectionList others;
		SortList( other, others );
		CBuilderConnectionList *conns = &others;
		if(conns) {
			POSITION pos = conns->GetHeadPosition();
			while(pos) {
				CBuilderConnection *conn = conns->GetNext(pos);
				CBuilderObject *obj = conn->GetSource();
				CFCOClassBase *fcoClass = dynamic_cast<CFCOClassBase *>(obj);
				CObjectClassBase *objectProxy = dynamic_cast<CObjectClassBase *>(obj);
				if (objectProxy && objectProxy->real)
					fcoClass = objectProxy->real;
				if(fcoClass == 0)
					AfxMessageBox("Unexpected composition: " + name + " - " + obj->GetName());
				else {
					CFCOClassBaseList containedparts;
					fcoClass->FindAllSubClasses(containedparts,true,false, interf);
					if(!containedparts.IsEmpty())
	//					FindRoles(parts,conn,fcoClass);
						FindRoles(containedparts,conn);
				}
			}
		}
	}
}

void CModelBuilder::FindAspects()
{
	CBuilderConnectionList other;
	POSITION ppos = parts.GetHeadPosition();
	while (ppos) {
		CObjectClassBase *part = parts.GetNext(ppos);
		const CBuilderConnectionList *partother = part->builder->GetInConnections("HasAspect");
		if(!partother)
			continue;
		POSITION tpos = partother->GetHeadPosition();
		while (tpos) {
			other.AddTail(partother->GetNext(tpos));
		}
	}
	CBuilderConnectionList others;
	SortList( &other, others );
	CBuilderConnectionList *conns = &others;
	if(conns) {
		POSITION pos = conns->GetHeadPosition();
		while(pos) {
			CBuilderConnection *conn = conns->GetNext(pos);
			CBuilderObject *obj = conn->GetSource();
			CAspect *aspect = theParadigm->FindAspect(obj);
			if(aspect == 0)
				AfxMessageBox("Unexpected aspect composition: " + name + " - " + obj->GetName());
			else {
				CFCOClassBaseList subs;
				FindAllSubClasses(subs,true,false, implement);
				POSITION spos = subs.GetHeadPosition();
				while(spos) {
					CFCOClassBase *fcoClass = subs.GetNext(spos);
					CModelBuilder *model = dynamic_cast<CModelBuilder *>(fcoClass);
					if(model == 0)
						AfxMessageBox("Model expected for " + name);
					else if(model->aspects.Find(aspect) == 0)
						model->aspects.AddTail(aspect);
				}
			}
		}
	}
}

void CModelBuilder::FindMatchingPortRoles(CFCOClassBaseList &fcoClasses,CPointerItemList &ptrItems)
{
	POSITION pos = roles.GetHeadPosition();
	while(pos) {
		CRole *role = roles.GetNext(pos);
		CModelBuilder *model = dynamic_cast<CModelBuilder *>(role->part);
		CReferenceBuilder *ref = dynamic_cast<CReferenceBuilder *>(role->part);
		if(model) {
			model->FindMatchingRoles(fcoClasses,ptrItems,role);
		}
		if (ref) {
			CModelBuilderList rms;
			ref->GetReferredModels(rms);
			POSITION rmpos = rms.GetHeadPosition();
			while (rmpos) {
				rms.GetNext(rmpos)->FindMatchingRoles(fcoClasses,ptrItems,role);
			}
		}
	}
}

void CModelBuilder::FindMatchingRoles(CFCOClassBaseList &fcoClasses,CPointerItemList &ptrItems,CRole *mainRole)
{
	POSITION pos = roles.GetHeadPosition();
	while(pos) {
		CRole *role = roles.GetNext(pos);
		if(fcoClasses.Find(role->part)) {
			if(!CPointerItem::FindEqualPointerItem(ptrItems,mainRole,role)) {
				CPointerItem *ptrItem = new CPointerItem;
				ptrItems.AddTail(ptrItem);
				if(mainRole) {
					ptrItem->object = mainRole;
					ptrItem->port = role;
				}
				else
					ptrItem->object = role;
			}
		}
	}
}

CAspect *CModelBuilder::FindLastAspect()
{
	CAspect *found = aspects.GetHead();
	POSITION pos = aspects.GetHeadPosition();
	while(pos) {
		CAspect *asp = aspects.GetNext(pos);
		if(asp->Y() > found->Y())
			found = asp;
	}
	return found;
}

void CModelBuilder::OrderAspects()
{
	if(aspects.IsEmpty()) {
		AfxMessageBox("Model " + name + " has no aspects defined");
		return;
	}

	CAspectList ordered;
	int n = aspects.GetCount();
	for(int i = 0; i < n; i++) {
		CAspect *found = FindLastAspect();
		ordered.AddTail(found);
		POSITION pos = aspects.Find(found);
		ASSERT(pos);
		aspects.RemoveAt(pos);
	}

	POSITION pos = ordered.GetHeadPosition();
	while(pos)
		aspects.AddHead(ordered.GetNext(pos));
}

void CModelBuilder::DumpXML(ofstream xml,int depth)
{
	DumpHeader(xml,depth,"model");
	DumpDispName(xml, depth+1);
	DumpNamePosition(xml,depth+1);
	DumpGeneralPreferences(xml,depth+1);
	DumpConstraints(xml,depth+1);
	DumpConnCardinalities(xml, depth+1);
	DumpContainmentCardinalities(xml, depth+1);
	DumpAttributes(xml,depth+1);
	DumpIcon(xml,depth+1);
	DumpPortIcon(xml,depth+1);
	DumpDecorator(xml,depth+1);
	DumpHotspotEnabled(xml, depth+1);
	DumpTypeDisplayed( xml, depth+1 );
	DumpTypeInfoDisplayed( xml, depth+1 );
	DumpSubtypeIcon( xml, depth + 1 );
	DumpInstanceIcon( xml, depth + 1 );
	DumpNameWrapNum( xml, depth + 1 );
	DumpNameEnabled( xml, depth + 1 );
	SortList( roles );
	POSITION pos = roles.GetHeadPosition();
	while(pos) {
		CRole *role = roles.GetNext(pos);
		CRole *dup = CRole::FindRoleByName(roles,role);
		if(dup)
		{
			POSITION t = roles.Find( dup );
			roles.RemoveAt( t );
			AfxMessageBox("Duplicate role name \"" + role->name + "\" found in model " + name,MB_OK | MB_ICONSTOP);
			pos = roles.GetHeadPosition( );
		}
	}

	pos = roles.GetHeadPosition();
	while(pos)
		roles.GetNext(pos)->DumpXML(xml,depth+1);

	OrderAspects();
	pos = aspects.GetHeadPosition();
	while(pos)
		aspects.GetNext(pos)->DumpXML(xml,depth+1,this);

	DumpFooter(xml,depth,"model");
}

void CModelBuilder::DumpContainmentCardinalities(ofstream xml,int depth)
{
	CRoleList lstAlreadyDone;
	POSITION pos = roles.GetHeadPosition();

	while( pos ) {
		POSITION pos2 = pos;
		CRole* pRole = roles.GetNext( pos );
		if ( lstAlreadyDone.Find( pRole ) )
			continue;

		// Build the End of the Expression

		CString strExprEnd;
		CString strCardContext;
		strCardContext.Format( "[containment] In model: %s, Role: %s", pRole->parent->GetName(), pRole->name );
		bool bDoConstraint = ! ParseCardinality( pRole->cardinality, "partCount", strCardContext, strExprEnd );
		if ( ! bDoConstraint )
			AfxMessageBox("Ignoring invalid cardinality string in role: " + pRole->name + "\nString: " + pRole->cardinality, MB_ICONSTOP);

		CBuilderConnection* pContainment = pRole->conn;

		// Build the Beginning of the Expression

		CString strExprBegin = "let partCount = ";
		int iCount = 0;
		while( pos2 ) {
			pRole = roles.GetNext( pos2 );
			if ( pRole->conn == pContainment ) {
				if ( iCount++ != 0 )
					strExprBegin += "+ ";
				strExprBegin += "self.parts( \"" + pRole->name + "\" ) -> size ";
				lstAlreadyDone.AddTail( pRole );
			}
		}
		strExprBegin += "in\n";

		// If Cardinality was appropriate

		if ( bDoConstraint && ! strExprEnd.IsEmpty() ) {

			// Build Name, EventMask, Description

			CString strCName;
			strCName.Format( "Valid%sPartCardinality%d", GetName(), theParadigm->GetNextConstraintID() );

			int iEventMask = 0;
			char chMask[64];
			sprintf( chMask, "%x", iEventMask );

			CString strDesc;
			strDesc.Format( "Multiplicity of parts, which are contained by %s, has to match %s.", GetName(), pRole->cardinality );

			// Dump constraint

			Indent();
			xml << "<constraint name=\"" << strCName << "\" eventmask = \"0x" << chMask << "\" depth = \"" << 0 << "\" priority = \"" << 2 << "\">" << endl;
			depth++;
			Indent();
			xml << "<![CDATA[" << strExprBegin << strExprEnd << "]]>" << endl;
			Indent();
			::DumpDispName(xml,strDesc);
			xml << endl;
			depth--;
			Indent();
			xml << "</constraint>" << endl;
		}
	}
}

void CModelBuilder::DumpTypeInfoDisplayed(ofstream xml, int depth)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos) {
		CModelBuilder* pModel = dynamic_cast<CModelBuilder*>( baseClasses.GetNext(pos) );
		if ( pModel )
			pModel->DumpTypeInfoDisplayed(xml,depth);
	}

	pos = imp_baseClasses.GetHeadPosition();
	while(pos) {
		CModelBuilder* pModel = dynamic_cast<CModelBuilder*>( imp_baseClasses.GetNext(pos) );
		if ( pModel )
			pModel->DumpTypeInfoDisplayed(xml,depth);
	}

	bool isTypeDisplayed = true;
	GetAttribute("IsTypeInfoShown", isTypeDisplayed );
	if( ! isTypeDisplayed ) {
		Indent();
		xml << 	 "<regnode name = \"isTypeInfoShown\" value =\"false\"></regnode>" << endl;
	}

}

///////////////////////////// CAtomBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CAtomBuilder, CFCOClassBase, "Atom");

void CAtomBuilder::DumpXML(ofstream xml,int depth)
{
	DumpHeader(xml,depth,"atom");
	depth++;
	DumpDispName(xml, depth);
	DumpNamePosition(xml,depth+1);
	DumpGeneralPreferences(xml,depth);
	DumpConstraints(xml,depth);
	DumpConnCardinalities(xml, depth+1);
	DumpAttributes(xml,depth+1);
	DumpIcon(xml,depth+1);
	DumpPortIcon(xml,depth+1);
	DumpDecorator(xml,depth+1);
	DumpHotspotEnabled(xml, depth+1);
	DumpTypeDisplayed( xml, depth+1 );
	DumpSubtypeIcon( xml, depth + 1 );
	DumpInstanceIcon( xml, depth + 1 );
	DumpNameWrapNum( xml, depth + 1 );
	DumpNameEnabled( xml, depth + 1 );

	depth--;
	DumpFooter(xml,depth,"atom");
}

///////////////////////////// CReferenceBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CReferenceBuilder, CFCOClassBase, "Reference");

void CReferenceBuilder::FindRefs()
{
	POSITION ppos = parts.GetHeadPosition();
	while (ppos) {
		CObjectClassBase	*part = parts.GetNext(ppos);
		CBuilderObjectList refObjs;
		part->builder->GetOutConnectedObjects("ReferTo",refObjs);
		if(refObjs.IsEmpty()) {
			if (dynamic_cast<CReferenceBuilder*>(part))
				AfxMessageBox("Unspecified reference " + name);
		}
		else {
			POSITION pos = refObjs.GetHeadPosition();
			while(pos) {
				CBuilderObject *obj = refObjs.GetNext(pos);
				CObjectClassBase *objectProxy = dynamic_cast<CObjectClassBase *>(obj);
				CFCOClassBase *ref;
				if (objectProxy && objectProxy->real)
					ref = objectProxy->real;
				else
					ref = dynamic_cast<CFCOClassBase *>(obj);
				if(ref == 0)
					AfxMessageBox("Unexpected reference specification " + obj->GetName() + " for " + name);
				else {
					CFCOClassBaseList subs;
					ref->FindAllSubClasses(subs,true,false, interf);
					if(subs.IsEmpty())
						AfxMessageBox("Only abstract objects found for reference " + name + " (" + ref->GetName() + ")");
					else {
						POSITION rpos = subs.GetHeadPosition();
						while(rpos)	{
							CFCOClassBase *sub = subs.GetNext(rpos);
							if(!refs.Find(sub))
								refs.AddTail(sub);
						}
					}
				}
			}
		}
	}
}

void CReferenceBuilder::GetReferredModels(CModelBuilderList &rmlist)
{
	POSITION pos = refs.GetHeadPosition();
	while (pos) {
		CModelBuilder *mb = dynamic_cast<CModelBuilder*>(refs.GetNext(pos));
		if (mb)
			rmlist.AddTail(mb);
	}
}

void CReferenceBuilder::DumpXML(ofstream xml,int depth)
{
	DumpHeader(xml,depth,"reference");
	depth++;
	DumpDispName(xml, depth);
	DumpNamePosition(xml,depth+1);
	DumpGeneralPreferences(xml,depth);
	DumpConstraints(xml,depth);
	DumpConnCardinalities(xml, depth+1);
	DumpAttributes(xml,depth+1);
	DumpIcon(xml, depth+1);
	DumpPortIcon(xml,depth+1);
	DumpDecorator(xml,depth+1);
	DumpHotspotEnabled(xml, depth+1);
	DumpNameWrapNum( xml, depth + 1 );
	DumpNameEnabled( xml, depth + 1 );
	Indent();
	xml <<	"<pointerspec name =\"ref\">" << endl;
	depth++;

	POSITION pos = refs.GetHeadPosition();
	while(pos) {
		CFCOClassBase *ref = refs.GetNext(pos);
		if(!ref->IsAbstract()) {
			Indent();
			xml << "<pointeritem desc = \"";
			xml << ref->GetName();
			xml << "\"></pointeritem>" << endl;
		}
	}

	depth--;
	Indent();
	xml << "</pointerspec>" << endl;
	depth--;
	DumpFooter(xml,depth,"reference");
}

///////////////////////////// CSetBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CSetBuilder, CFCOClassBase, "Set");

CSetBuilder::~CSetBuilder()
{
	DeleteMembers(members);
}

void CSetBuilder::FindMembers()
{
	CFCOClassBaseList potentialMembers;
	FindPotentialMembers(potentialMembers);
	MatchMembers(potentialMembers,members);
}

void CSetBuilder::FindPotentialMembers(CFCOClassBaseList &potentialMembers)
{
	POSITION pos = baseClasses.GetHeadPosition();
	while(pos) {
		baseClasses.GetNext(pos)->FindPotentialMembers(potentialMembers);
	}

	pos = imp_baseClasses.GetHeadPosition();
	while(pos) {
		imp_baseClasses.GetNext(pos)->FindPotentialMembers(potentialMembers);
	}

	POSITION ppos = parts.GetHeadPosition();
	while (ppos)
	{
		CObjectClassBase *part = parts.GetNext(ppos);
		CBuilderObjectList partobjs;
		part->builder->GetInConnectedObjects("SetMembership",partobjs);
		POSITION cpos = partobjs.GetHeadPosition();
		while (cpos) {
			CBuilderObject *obj = partobjs.GetNext(cpos);
			CFCOClassBase *fcoClass;
			CObjectClassBase *objectProxy = dynamic_cast<CObjectClassBase *>(obj);
			if (objectProxy && objectProxy->real)
				fcoClass = objectProxy->real;
			else
				fcoClass = dynamic_cast<CFCOClassBase *>(obj);
			if(fcoClass == 0)
				AfxMessageBox("Unexpected object " + obj->GetName() + " for set " + name + " member specification");
			else {
				CFCOClassBaseList subs;
				fcoClass->FindAllSubClasses(subs,true,false, interf);
				potentialMembers.AddTail(&subs);
			}
		}
	}
}

void CSetBuilder::MatchMembers(CFCOClassBaseList &potentialMembers, CPointerItemList &list)
{
	SortList( potentialMembers );
	POSITION rpos = roles.GetHeadPosition();
	while(rpos) {
		CModelBuilder *owner = roles.GetNext(rpos)->parent;
		if (!potentialMembers.IsEmpty()) {
			owner->FindMatchingRoles(potentialMembers,list);
		}
	}
}


void CSetBuilder::DumpXML(ofstream xml,int depth)
{
	DumpHeader(xml,depth,"set");
	depth++;
	DumpDispName(xml, depth);
	DumpNamePosition(xml,depth+1);
	DumpGeneralPreferences(xml,depth);
	DumpConstraints(xml,depth);
	DumpConnCardinalities(xml, depth+1);
	DumpAttributes(xml,depth+1);
	DumpIcon(xml,depth+1);
	DumpPortIcon(xml,depth+1);
	DumpDecorator(xml,depth+1);
	DumpHotspotEnabled(xml, depth+1);
	DumpTypeDisplayed( xml, depth+1 );
	DumpSubtypeIcon( xml, depth + 1 );
	DumpInstanceIcon( xml, depth + 1 );
	DumpNameWrapNum( xml, depth + 1 );
	DumpNameEnabled( xml, depth + 1 );

	Indent();
	xml << "<pointerspec name = \"set\">" << endl;
	depth++;

	if(members.IsEmpty()) {
		AfxMessageBox("Missing member specification for set " + name, MB_ICONEXCLAMATION);
	}
	SortList( members );

	POSITION pos = members.GetHeadPosition();
	while(pos)
		members.GetNext(pos)->DumpXML(xml,depth);

	depth--;
	Indent();
	xml << "</pointerspec>" << endl;
	depth--;
	DumpFooter(xml,depth,"set");
}

///////////////////////////// CConnectionBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CConnectionBuilder, CFCOClassBase, "Connection");

CConnectionBuilder::~CConnectionBuilder()
{
	DeleteMembers(srcPtrItems);
	DeleteMembers(dstPtrItems);
}

bool CConnectionBuilder::FindSrcAndDst(bool hasBaseClasses)
{
// "Inserted logic here to allow for role names OTHER than src/dst, and no role names."
	/*	The basic idea of this was to associate the rolename of the SourceToConnector/ConnectorToSource
		with the actual rolename from the paradigm.  Normally, the rolenames will be "src" and "dst".
		However, it is possible that other rolenames will be used in the future, as well as the present
		notion of connections being able to be both source and destination.

		In that special case, this method will make the 'srcs' and 'dsts' be exactly the same.


	*/

	/*
		Connector Proxies are not allowed to connect FCO-s !
	*/
	src = dst = 0;
	bIsBidirectional = false;
	CBuilderObject *ass;
	CBuilderObjectList ass1;
	GetInConnectedObjects("AssociationClass",ass1);
	SortList( ass1 );
	CBuilderObjectList ass2;
	GetOutConnectedObjects("AssociationClass",ass2);
	SortList( ass2 );
	if(ass1.GetCount() + ass2.GetCount() != 1) {
		if (!hasBaseClasses && !IsAbstract()) {
			AfxMessageBox("Connection class " + name + " must be assigned to exactly one association or inherited from another connections !");
		}
	}
	else {
		ass = (ass1.IsEmpty() ? ass2 : ass1).GetHead();
		ASSERT(ass);
		// ass is the AssocConnectorObject...next, let's find the src/dst role objects
		CBuilderObjectList srcObjs;
		const CBuilderConnectionList *srcConns = ass->GetInConnections( "SourceToConnector" );
		const CBuilderConnectionList *dstConns = ass->GetOutConnections( "ConnectorToDestination" );
		// the sum of these two connector lists should be '2'
		int nConnectionCount = 0;
		if( srcConns || dstConns )
		{
			if( srcConns )
			{
				nConnectionCount += srcConns->GetCount( );
			}
			if( dstConns )
			{
				nConnectionCount += dstConns->GetCount( );
			}
		}

		if( nConnectionCount != 2 )
		{
			AfxMessageBox("Incomplete association specification for connection '" + name +
				"'.  There should be one source and one destination." );
		}
		else
		{
			// everything should be ok...must determine what configuration we have...
			//  1 - one srcConn and one dstConn
			//	2 - no  srcConn and two dstConn
			//	3 - two srcConn and no  dstConn
			// must assign roles and cardinalities to the associations.
			CBuilderConnection *srcConn;
			CBuilderConnection *dstConn;
			CObjectClassBase *objectProxy;
			if( srcConns && srcConns->GetCount( ) == 2 )
			{
				srcConn = srcConns->GetHead( );
				srcConn->GetAttribute( "srcRolename", srcRole );
				srcConn->GetAttribute( "Cardinality", srcCard );

				objectProxy = dynamic_cast<CObjectClassBase *>( srcConn->GetSource( ) );
				if (objectProxy && objectProxy->real)
					src = objectProxy->real;
				else
					src = dynamic_cast<CFCOClassBase *>( srcConn->GetSource( ) );

				dstConn = srcConns->GetTail( );
				dstConn->GetAttribute( "srcRolename", dstRole );
				dstConn->GetAttribute( "Cardinality", dstCard );

				objectProxy = dynamic_cast<CObjectClassBase *>( dstConn->GetSource( ) );
				if (objectProxy && objectProxy->real)
					dst = objectProxy->real;
				else
					dst = dynamic_cast<CFCOClassBase *>( dstConn->GetSource( ) );


			}
			else if( dstConns && dstConns->GetCount( ) == 2 )
			{
				srcConn = dstConns->GetHead( );
				srcConn->GetAttribute( "dstRolename", srcRole );
				srcConn->GetAttribute( "Cardinality", srcCard );

				objectProxy = dynamic_cast<CObjectClassBase *>( srcConn->GetDestination( ) );
				if (objectProxy && objectProxy->real)
					src = objectProxy->real;
				else
					src = dynamic_cast<CFCOClassBase *>( srcConn->GetDestination( ) );

				dstConn = dstConns->GetTail( );
				dstConn->GetAttribute( "dstRolename", dstRole );
				dstConn->GetAttribute( "Cardinality", dstCard );

				objectProxy = dynamic_cast<CObjectClassBase *>( dstConn->GetDestination( ) );
				if (objectProxy && objectProxy->real)
					dst = objectProxy->real;
				else
					dst = dynamic_cast<CFCOClassBase *>( dstConn->GetDestination( ) );

			}
			else // one of each...
			{
				srcConn = srcConns->GetHead( );
				srcConn->GetAttribute( "srcRolename", srcRole );
				srcConn->GetAttribute( "Cardinality", srcCard );

				objectProxy = dynamic_cast<CObjectClassBase *>( srcConn->GetSource( ) );
				if (objectProxy && objectProxy->real)
					src = objectProxy->real;
				else
					src = dynamic_cast<CFCOClassBase *>( srcConn->GetSource( ) );

				dstConn = dstConns->GetHead( );
				dstConn->GetAttribute( "dstRolename", dstRole );
				dstConn->GetAttribute( "Cardinality", dstCard );

				objectProxy = dynamic_cast<CObjectClassBase *>( dstConn->GetDestination( ) );
				if (objectProxy && objectProxy->real)
					dst = objectProxy->real;
				else
					dst = dynamic_cast<CFCOClassBase *>( dstConn->GetDestination( ) );

			}

			// now we have the following:
			// src - represents one role of the connection (probably src role)
			// dst - represents the other role of the conn (probably dst role)
			// [src/dst]Role - the rolename of the [src/dst] (possibly: srcRole="dst", etc.)
			// [src/dst]Card - ditto for connections (probably "0..*")
			//

			src->FindAllSubClasses(srcs,true,false, interf);
			dst->FindAllSubClasses(dsts,true,false, interf);

			// if srcRole=dstRole="", then we need to duplicate the roles across the board...
			srcRole.TrimLeft( );
			srcRole.TrimRight( );
			dstRole.TrimLeft( );
			dstRole.TrimRight( );
			if( srcRole.IsEmpty( ) && dstRole.IsEmpty( ) )
			{
				bIsBidirectional = true;
				srcRole = "src";
				dstRole = "dst";
//				CFCOClassBaseList both;
//				POSITION pos = srcs.GetHeadPosition( );
//				while( pos )
//				{
//					both.AddTail( srcs.GetNext( pos ) );
//				}
//				pos = dsts.GetHeadPosition( );
//				while( pos )
//				{
//					both.AddTail( dsts.GetNext( pos ) );
//				}
//				srcs.RemoveAll( );
//				dsts.RemoveAll( );
//				SortList( both );
//				pos = both.GetHeadPosition( );
//				while( pos )
//				{
//					CFCOClassBase *t = both.GetNext( pos );
//					srcs.AddTail( t );
//					dsts.AddTail( t );
//				}
			}
			else if( srcRole.IsEmpty( ) || dstRole.IsEmpty( ) )
			{
				AfxMessageBox("You must specify roles for BOTH the source and destination, or \
NEITHER the source and destination.  Your output will not be valid.  See connection " + name);
				// remove this connection from the role of its owner
				return false;
			}
			else if( ( srcRole.CompareNoCase( "src" ) != 0 ||
					   dstRole.CompareNoCase( "dst" ) != 0 ) &&
					 ( srcRole.CompareNoCase( "dst" ) != 0 ||
					   dstRole.CompareNoCase( "src" ) != 0 ) )
			{
				// they are not labeled "src" and "dst"
				AfxMessageBox( "Warning: connection roles do not equal 'src' and 'dst' for connection " +
								name );
			}
			return( src != NULL && dst != NULL );
		}
	}
//		ass->GetInConnectedObjects("SourceToConnector",srcObjs);
//		SortList( srcObjs );
//		if(srcObjs.GetCount() != 1)
//			AfxMessageBox("Incomplete association specification for connection " + name);
//		else {
//			CBuilderObjectList dstObjs;
//			ass->GetOutConnectedObjects("ConnectorToDestination",dstObjs);
//			SortList( dstObjs );
//			if(dstObjs.GetCount() != 1)
//				AfxMessageBox("Incomplete association specification for connection " + name);
//			else {
//				src = dynamic_cast<CFCOClassBase *>(srcObjs.GetHead());
//				dst = dynamic_cast<CFCOClassBase *>(dstObjs.GetHead());
//				if(src == 0 || dst == 0)
//					AfxMessageBox("Unexpected object in connection specification for " + name);
//				else {
//					src->FindAllSubClasses(srcs,true,false);
//					dst->FindAllSubClasses(dsts,true,false);
//					// now, we have all of the subclasses, but it time to find all
//					// models of which these src/dst are ports
//				}
//				return (src != 0 && dst != 0);
//			}
//		}
//	}
	// <old version........>
//	src = dst = 0;
//	CBuilderObject *ass;
//	CBuilderObjectList ass1;
//	GetInConnectedObjects("AssociationClass",ass1);
//	SortList( ass1 );
//	CBuilderObjectList ass2;
//	GetOutConnectedObjects("AssociationClass",ass2);
//	SortList( ass2 );
//	if(ass1.GetCount() + ass2.GetCount() != 1)
//		AfxMessageBox("Connection class " + name + " must be assigned to exactly one association!");
//	else {
//		ass = (ass1.IsEmpty() ? ass2 : ass1).GetHead();
//		ASSERT(ass);
//		CBuilderObjectList srcObjs;
//		ass->GetInConnectedObjects("SourceToConnector",srcObjs);
//		SortList( srcObjs );
//		if(srcObjs.GetCount() != 1)
//			AfxMessageBox("Incomplete association specification for connection " + name);
//		else {
//			CBuilderObjectList dstObjs;
//			ass->GetOutConnectedObjects("ConnectorToDestination",dstObjs);
//			SortList( dstObjs );
//			if(dstObjs.GetCount() != 1)
//				AfxMessageBox("Incomplete association specification for connection " + name);
//			else {
//				src = dynamic_cast<CFCOClassBase *>(srcObjs.GetHead());
//				dst = dynamic_cast<CFCOClassBase *>(dstObjs.GetHead());
//				if(src == 0 || dst == 0)
//					AfxMessageBox("Unexpected object in connection specification for " + name);
//				else {
//					src->FindAllSubClasses(srcs,true,false);
//					dst->FindAllSubClasses(dsts,true,false);
//					// now, we have all of the subclasses, but it time to find all
//					// models of which these src/dst are ports
//				}
//				return (src != 0 && dst != 0);
//			}
//		}
//	}
	// </old version........>
	return false;
}

void CConnectionBuilder::BuildInstances()
{
	bool inherited = false;
	if (baseClasses.GetCount()) {
		POSITION pos = baseClasses.GetHeadPosition();
		while (pos) {
			CConnectionBuilder *conn = dynamic_cast<CConnectionBuilder*>(baseClasses.GetNext(pos));
			if (conn) {
				inherited = true;
			}
		}
	}
	if (imp_baseClasses.GetCount()) {
		POSITION pos = imp_baseClasses.GetHeadPosition();
		while (pos) {
			CConnectionBuilder *conn = dynamic_cast<CConnectionBuilder*>(imp_baseClasses.GetNext(pos));
			if (conn) {
				inherited = true;
			}
		}
	}

	// Need some redesign
	CRoleList connroles;
	connroles.AddTail(&roles);
	POSITION spos = subClasses.GetHeadPosition();
	while (spos) {
		CConnectionBuilder *conn = dynamic_cast<CConnectionBuilder*>(subClasses.GetNext(spos));
		if (conn) {
			connroles.AddTail(&(conn->roles));
		}
	}
	spos = imp_subClasses.GetHeadPosition();
	while (spos) {
		CConnectionBuilder *conn = dynamic_cast<CConnectionBuilder*>(imp_subClasses.GetNext(spos));
		if (conn) {
			connroles.AddTail(&(conn->roles));
		}
	}

	POSITION rpos = connroles.GetHeadPosition();
	while (rpos) {
		CRole *r = connroles.GetNext(rpos);
		POSITION fpos;
		if ( rpos && ((fpos = connroles.Find(r, rpos)) != NULL) ) {
			connroles.RemoveAt(fpos);
			rpos = connroles.GetHeadPosition();
		}
	}

	if(FindSrcAndDst(inherited)) {
		POSITION pos = connroles.GetHeadPosition();
		while(pos) {
			CModelBuilder *owner = connroles.GetNext(pos)->parent;
			owner->FindMatchingPortRoles(srcs,srcPtrItems);
			owner->FindMatchingRoles(srcs,srcPtrItems);
			owner->FindMatchingPortRoles(dsts,dstPtrItems);
			owner->FindMatchingRoles(dsts,dstPtrItems);
		}
	}
}

void CConnectionBuilder::DumpLabelSpec(ofstream xml,int depth,CString which,CString regLabel)
{
	CString label;
	GetAttribute(which,label);
	if(!label.IsEmpty()) {
		CAttributeBuilder *attr = FindAttribute(label);
		if(attr) {
			Indent();
			xml << "<regnode name= \"" << regLabel + "\" value=\"%" + label + "%\"/>" << endl;
		}
		else
//			AfxMessageBox("Attribute " + attr->GetName() + " not found for connection " + name + " label specification",MB_OK | MB_ICONSTOP);
			AfxMessageBox("Attribute '" + label + "' not found for connection " + name + " label specification",MB_OK | MB_ICONSTOP);
	}
}

void CConnectionBuilder::DumpXML(ofstream xml,int depth)
{
	DumpHeader(xml,depth,"connection");
	depth++;
	DumpDispName(xml, depth);
	DumpGeneralPreferences(xml,depth);
	DumpConstraints(xml,depth);
	DumpAttributes(xml,depth+1);

	Indent();
	CString color;
	GetAttribute("Color",color);
	if(!color.IsEmpty())
		xml << 	 "<regnode name = \"color\" value =\"" << color << "\"></regnode>" << endl;
	else
		xml << 	 "<regnode name = \"color\" value =\"0x000000\"></regnode>" << endl;


	CString end;
	GetAttribute("ConnLineEnd",end);
	if(!end.IsEmpty()) {
		Indent();
		xml << 	 "<regnode name = \"dstStyle\" value =\"" << end << "\"></regnode>" << endl;
	}

	CString start;
	GetAttribute("ConnLineStart",start);
	if(!end.IsEmpty()) {
		Indent();
		xml << 	 "<regnode name = \"srcStyle\" value =\"" << start << "\"></regnode>" << endl;
	}

	CString line;
	GetAttribute("ConnLineType",line);
	if(!line.IsEmpty()) {
		Indent();
		xml << 	 "<regnode name = \"lineType\" value =\"" << line << "\"></regnode>" << endl;
	}

	DumpLabelSpec(xml,depth,"SrcAttrLabel1","srcLabel1");
	DumpLabelSpec(xml,depth,"SrcAttrLabel2","srcLabel2");
	DumpLabelSpec(xml,depth,"DstAttrLabel1","dstLabel1");
	DumpLabelSpec(xml,depth,"DstAttrLabel2","dstLabel2");

	CConnectionBuilderList conns;
	conns.AddTail(this);
	POSITION ipos = baseClasses.GetHeadPosition();
	while (ipos) {
		CConnectionBuilder *conn = dynamic_cast<CConnectionBuilder*>(baseClasses.GetNext(ipos));
		if (conn) {
			conns.AddTail(conn);
		}
	}
	ipos = imp_baseClasses.GetHeadPosition();
	while (ipos) {
		CConnectionBuilder *conn = dynamic_cast<CConnectionBuilder*>(imp_baseClasses.GetNext(ipos));
		if (conn) {
			conns.AddTail(conn);
		}
	}

	POSITION cpos = conns.GetHeadPosition();
	while (cpos) {
		CConnectionBuilder	*conn = conns.GetNext(cpos);
		SortList( conn->srcPtrItems );
		SortList( conn->dstPtrItems );
		Indent();
		xml << "<connjoint>" << endl;
		if( !conn->srcRole.IsEmpty( ) && !conn->dstRole.IsEmpty( ) )
		{
			depth++;
			Indent();
			xml << "<pointerspec name = \"" + conn->srcRole + "\">" << endl;
			{
				POSITION pos = conn->srcPtrItems.GetHeadPosition();
				while(pos)
					conn->srcPtrItems.GetNext(pos)->DumpXML(xml,depth+1);
			}
			Indent();
			xml << "</pointerspec>" << endl;

			Indent();
			xml << "<pointerspec name = \"" + conn->dstRole + "\">" << endl;
			{
				POSITION pos = conn->dstPtrItems.GetHeadPosition();
				while(pos)
					conn->dstPtrItems.GetNext(pos)->DumpXML(xml,depth+1);
			}
			Indent();
			xml << "</pointerspec>" << endl;

			depth--;
		}
		Indent();
		xml << "</connjoint>" << endl;


		if( conn->bIsBidirectional )
		{
			Indent();
			xml << "<connjoint>" << endl;
			depth++;
			Indent();
			xml << "<pointerspec name = \"" + conn->dstRole + "\">" << endl;
			{
				POSITION pos = srcPtrItems.GetHeadPosition();
				while(pos)
					conn->srcPtrItems.GetNext(pos)->DumpXML(xml,depth+1);
			}
			Indent();
			xml << "</pointerspec>" << endl;

			Indent();
			xml << "<pointerspec name = \"" + conn->srcRole + "\">" << endl;
			{
				POSITION pos = conn->dstPtrItems.GetHeadPosition();
				while(pos)
					conn->dstPtrItems.GetNext(pos)->DumpXML(xml,depth+1);
			}
			Indent();
			xml << "</pointerspec>" << endl;

			depth--;
			Indent();
			xml << "</connjoint>" << endl;
		}
	}

	depth--;
	DumpFooter(xml,depth,"connection");
}

///////////////////////////// CAttributeBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOMBASE(CAttributeBuilder, CBuilderAtom);

void CAttributeBuilder::Initialize()
{
	CBuilderAtom::Initialize();
	GetAttribute("Prompt",dispName);
	GetAttribute("IsViewable",viewable);
	GetAttribute("GlobalScope",global);
	GetAttribute("Help", help);
	GetLocation(CString("Attributes"),loc);
}

///////////////////////////// CBooleanAttributeBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CBooleanAttributeBuilder, CAttributeBuilder, "BooleanAttribute");

void CBooleanAttributeBuilder::DumpXML(ofstream xml,int depth, const CString &path)
{
	Indent();
	xml << "<attrdef name=\"" << name << "\" ";
	DumpMetaRef(xml, this, path);
	xml << "valuetype = \"boolean\" defvalue = ";
	bool on;
	GetAttribute("BooleanDefault",on);
	xml << (on ? "\"true\"" : "\"false\"");
	xml << ">" << endl;
	depth++;

	Indent();
	::DumpDispName(xml,dispName);
	xml << endl;

	if(!help.IsEmpty()) {
		Indent();
		xml << "<regnode name=\"help\" value = \"" << help << "\"></regnode>" << endl;
	}


	depth--;
	Indent();
	xml << "</attrdef>" << endl;
}

///////////////////////////// CFieldAttributeBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CFieldAttributeBuilder, CAttributeBuilder, "FieldAttribute");

void CFieldAttributeBuilder::DumpXML(ofstream xml,int depth, const CString &path)
{
	Indent();
	xml << "<attrdef name=\"" << name << "\" ";
	DumpMetaRef(xml, this, path);
	xml << "valuetype = \"";
	CString valType;
	GetAttribute("DataType",valType);
	CString def;
	GetAttribute("FieldDefault",def);
	xml << valType << "\" defvalue = \"" << def << "\">" << endl;

	depth++;
	Indent();
	::DumpDispName(xml,dispName);
	xml << endl;

	int mul;
	GetAttribute("Multiline",mul);
	if(mul > 1) {
		Indent();
		xml << "<regnode name=\"multiLine\" value = \"" << mul << "\"></regnode>" << endl;
	}

	depth--;
	Indent();
	xml << "</attrdef>" << endl;
}

///////////////////////////// CEnumeratedAttributeBuilder ///////////////////////////////
IMPLEMENT_CUSTOMATOM(CEnumeratedAttributeBuilder, CAttributeBuilder, "EnumAttribute");


/*	Dump XML of an EnumeratedAttribute
	1. make a hashed listing of displayed names and values
	2. print out that hashed table of names and values (in that order)
*/
void CEnumeratedAttributeBuilder::DumpXML(ofstream xml,int depth, const CString &path)
{
	CMapStringToString hashtable( 20 );
	CStringList attrList;
	CString defaultFirstValue;

	// ...must...make...hash listing...
	CString items;
	GetAttribute("MenuItems",items);
	char string[4096];
	strcpy(string,(LPCSTR)items);
	char seps[16];
	sprintf(seps,"%c\n",13);
	char *token;

	token = strtok(string,seps);
	while(token) {
		while(isspace(*token) && *token)
			token++;
		if(*token) {
			// must not use strtok again, b/c it uses static data...grrrrr
			char *first, *second;
			first = token;
			second = strchr( first, ',' );
			CString sName;
			CString sValue;
			if( second == NULL )
			{
				sName = first;
				second = first;
			}
			else
			{
				char name[STR_MAX];
				memset( name, NULL, sizeof(name) );
				strncpy( name, first, second-first );
				second++;
				while( isspace(*second) && *second )
				{
					second++;
				}
				sName = name;
//				sValue = second;
			}
			sValue = second;
			hashtable.SetAt( sName, sValue );
			if (defaultFirstValue.IsEmpty()) {
				defaultFirstValue = sValue;
			}
			attrList.AddTail( sName );
		}
		token = strtok(0,seps);
	}


	Indent();
	xml << "<attrdef name=\"" << name << "\" ";
	DumpMetaRef(xml, this, path);
	xml << "valuetype = \"enum\" defvalue = \"";
	CString def;
	GetAttribute("DefaultItem",def);
	CString sDefaultValue;
	if (!hashtable.Lookup( def, sDefaultValue )) {
		sDefaultValue = defaultFirstValue;
	}
	xml << sDefaultValue << "\">" << endl;

	depth++;
	Indent();
	::DumpDispName(xml,dispName);
	xml << endl;


	// now, get all values from the hashtable and use them
	POSITION pos = attrList.GetHeadPosition( );
	while( pos )
	{
		CString sName  = attrList.GetNext( pos );
		CString sValue;
		hashtable.Lookup( sName, sValue );
		Indent();
		xml << "<enumitem dispname = \"" << sName << "\" value = \"" << sValue << "\"></enumitem>" << endl;
	}

	depth--;
	Indent();
	xml << "</attrdef>" << endl;
}

///////////////////////////// CRole ///////////////////////////////

CRole::CRole( )
{
	baseName = "";
	name = "";
	parent = NULL;
	conn = NULL;
	part = NULL;
	linked = flag = false;
	nMyOrder = nOrderNumber++;
	cardinality = "";
}

void CRole::SetFlags(CRoleList &roles,bool val)
{
	POSITION pos = roles.GetHeadPosition();
	while(pos)
		roles.GetNext(pos)->flag = val;
}
CRole *CRole::FindRoleByName(CRoleList &roles,CRole *start)
{
	if( start )
	{
		POSITION pos = roles.Find(start);
		VERIFY(pos);
		roles.GetNext(pos);
		while(pos) {
			CRole *found = roles.GetNext(pos);
			if(start->name == found->name)
				return found;
		}
	}
	return 0;
}

CRole *CRole::FindRoleByName(CRoleList &roles, CString sName )
{
	POSITION pos = roles.GetHeadPosition( );
//	VERIFY(pos);
	while(pos) {
		CRole *found = roles.GetNext(pos);
		if(sName == found->name)
			return found;
	}
	return 0;
}


void CRole::DumpXML(ofstream xml,int depth)
{
	Indent();
	xml << "<role name = \"" << name << "\" ";
	DumpMetaRef(xml, part, parent->GetName() + "/" + name);
	xml << "kind = \"" << part->GetName() << "\">";

	::DumpDispName(xml, name);

	xml << "</role>" << endl;
}

///////////////////////////// CPart ///////////////////////////////
void CPart::DumpXML(ofstream xml,int depth)
{
	Indent();
	xml << "<part ";
	CString path;
	path.Format("%s/%s/%s", role->parent->GetName(),role->name, aspect->GetName());
	DumpMetaRef(xml, role->part, path);
	xml << "role = \"" << role->name << "\" primary = ";
	xml << (isPrimary ? "\"yes\"" : "\"no\"");

	if (kindaspect && (aspect != kindaspect))
		xml << " kindaspect = \"" << kindaspect->GetName() <<"\"";

	xml << " linked = ";
	xml << (role->linked ? "\"yes\"" : "\"no\"") << "></part>" << endl;
}

///////////////////////////// CAspect ///////////////////////////////


/* Things changed... now we put roles into aspects !
CAspect::CAspect(CBuilderSet *s) : set(s)
{
	name = set->GetName();
	set->GetLocation(CString("Visualization"),loc);

	const CBuilderObjectList *those = set->GetMembers();
	CBuilderObjectList mmbers;
	SortList( those, mmbers );
	CBuilderObjectList *fcos = &mmbers;

	if(fcos) {
		POSITION pos = fcos->GetHeadPosition();
		while(pos) {
			CBuilderObject *obj = fcos->GetNext(pos);
			CFCOClassBase *fcoClass;
			CObjectProxyBuilder	*objectProxy = dynamic_cast<CObjectProxyBuilder *>(obj);
			if (objectProxy)
				fcoClass = objectProxy->real;
			else
				fcoClass = dynamic_cast<CFCOClassBase *>(obj);
			if(fcoClass == 0)
				AfxMessageBox("Unexpected member " + obj->GetName() + " of set " + name);
			else {
				CFCOClassBaseList subs;
				fcoClass->FindAllSubClasses(subs,true,false, interf);
				SortList( subs );
				POSITION spos = subs.GetHeadPosition();
				while(spos) {
					CFCOClassBase *sub = subs.GetNext(spos);
					POSITION rpos = sub->GetRoles().GetHeadPosition();
					while(rpos) {
						CRole *role = sub->GetRoles().GetNext(rpos);
						if(!FindPrimaryPart(role)) {
							CPart *part = new CPart(role,this);
							primaryParts.AddTail(part);
						}
					}
				}
			}
		}
	}
} */


CAspect::CAspect(CBuilderSet *s) : set(s)
{
	name = set->GetName();
	set->GetLocation(CString("Visualization"),loc);

	const CBuilderObjectList *those = set->GetMembers();
	CBuilderObjectList mmbers;
	SortList( those, mmbers );
	CBuilderObjectList *fcos = &mmbers;

	set->GetReferencedBy(parts);
	parts.AddHead(set);

	if(fcos) {
		POSITION pos = fcos->GetHeadPosition();
		while(pos) {
			CBuilderObject *obj = fcos->GetNext(pos);
			CBuilderConnection *roleConn = dynamic_cast<CBuilderConnection*>(obj);
			CObjectClassBase *objectProxy = dynamic_cast<CObjectClassBase*>(obj);
			CFCOClassBase *fcoClass = dynamic_cast<CFCOClassBase *>(obj);

			if(objectProxy && objectProxy->real)
				fcoClass = objectProxy->real;

			if (roleConn && (roleConn->GetName() == "Containment")) {
				obj = roleConn->GetSource();
				objectProxy = dynamic_cast<CObjectClassBase *>(obj);
				if (objectProxy && objectProxy->real)
					fcoClass = objectProxy->real;
				else
					fcoClass = dynamic_cast<CFCOClassBase *>(obj);
			}

			if (!fcoClass) {
				AfxMessageBox("Unexpected object " + obj->GetName() + " in aspect " + name);
			}
			else {
				CFCOClassBaseList subs;
				fcoClass->FindAllSubClasses(subs,true,false, interf);
				SortList( subs );
				POSITION spos = subs.GetHeadPosition();
				while(spos) {
					CFCOClassBase *sub = subs.GetNext(spos);
					POSITION rpos = sub->GetRoles().GetHeadPosition();
					while(rpos) {
						CRole *role = sub->GetRoles().GetNext(rpos);
						if((!FindPrimaryPart(role)) && ((!roleConn) || (role->conn == roleConn))) {
							CPart *part = new CPart(role,this);
							allParts.AddTail(part);
						}
					}
				}
			}
		}
	}
}

CAspect::~CAspect()
{
	DeleteMembers(allParts);
}

CPart *CAspect::FindPrimaryPart(CRole *role)
{
	POSITION pos = allParts.GetHeadPosition();
	while(pos) {
		CPart *part = allParts.GetNext(pos);
		if(part->role == role)
			return part;
	}
	return 0;
}

////////////////////////////////////////////////////
void CAspect::FindSameAspects()
{
	POSITION ppos = parts.GetHeadPosition();
	while (ppos) {
		bool leftIsProxy = false;
		CBuilderObject *part = parts.GetAt(ppos);
		if (!part->IsKindOf(RUNTIME_CLASS(CBuilderSet)))
			leftIsProxy = true;
		CBuilderObjectList diamonds;
		part->GetInConnectedObjects("SameAspectLeft", diamonds);
		SortList ( diamonds );
		FindSameAspects(diamonds, leftIsProxy);

		diamonds.RemoveAll();
		part->GetOutConnectedObjects("SameAspectLeft", diamonds);
		SortList ( diamonds );
		FindSameAspects(diamonds, leftIsProxy);
		parts.GetNext(ppos);
	}
}

void CAspect::FindSameAspects(CBuilderObjectList &diamonds, bool leftIsProxy)
{
	POSITION pos = diamonds.GetHeadPosition();
	while(pos) {
		CBuilderObject *diamond = diamonds.GetNext(pos);
		CBuilderObjectList sames;
		diamond->GetInConnectedObjects("SameAspectRight",sames);
		SortList( sames );
		JoinSameAspects(sames, leftIsProxy, diamond->GetName());
		sames.RemoveAll();
		diamond->GetOutConnectedObjects("SameAspectRight",sames);
		SortList( sames );
		JoinSameAspects(sames, leftIsProxy, diamond->GetName());
	}
}

void CAspect::JoinSameAspects(CBuilderObjectList &sameaspects, bool leftIsProxy, CString commonName)
{
	POSITION spos = sameaspects.GetHeadPosition();
	while (spos) {
		bool rightIsProxy = false;
		CBuilderObject *same = sameaspects.GetNext(spos);
		CBuilderSet *sameaspect = dynamic_cast<CBuilderSet *>(same);
		CBuilderReference *sameproxy = dynamic_cast<CBuilderReference *>(same);
		CAspect* otheraspect;
		if (sameproxy || sameaspect) {
			otheraspect = theParadigm->FindAspect(same);
			if (sameproxy)
				rightIsProxy = true;
		}
		else  {
			AfxMessageBox("Equivalence between classes with different types ! Class: " + this->GetName() + " cannot be equal to class: " +same->GetName());
			return;
		}
		if (otheraspect != this) {
			if ( (rightIsProxy && leftIsProxy) || ((!rightIsProxy) && (!leftIsProxy)) ) {
				if (commonName == "SameAspect")
					AfxMessageBox("Equivalence between two aspects. Both are proxy or real.\nIn this case the name of the equivalence operator will be given to the common aspect.\nGive it a bit more meaningful name than \"SameAspect\"", MB_OK | MB_ICONEXCLAMATION);
				this->name = commonName;
			}
			else if (leftIsProxy) {
				this->name = otheraspect->GetName();
			}

			parts.AddTail(&(otheraspect->parts));
			POSITION appos = otheraspect->allParts.GetHeadPosition();
			while (appos) {
				CPart *tpart = otheraspect->allParts.GetNext(appos);
				tpart->aspect = this;
				tpart->kindaspect = this;
				if (this->FindPrimaryPart(tpart->role)) {
					delete tpart;
				}
				else {
					allParts.AddTail(tpart);
				}
			}
			otheraspect->allParts.RemoveAll();
			theParadigm->DiscardAspect(otheraspect);
		}
	}
}
////////////////////////////////////////////////////


void CAspect::DumpXML(ofstream xml,int depth,CModelBuilder *owner)
{
	Indent();
	xml << "<aspect name = \"" << name << "\" ";
	DumpMetaRef(xml, GetBuilderObject(), owner->GetName());
	owner->DumpAttributeNames(xml,true);
	xml << ">" << endl;

	depth++;
	CString dispName;
	set->GetAttribute("DisplayedName",dispName);
	if(!dispName.IsEmpty()) {
		Indent();
		::DumpDispName(xml,dispName);
		xml << endl;
	}

	// must eliminate duplicate roles in aspects
/*
	POSITION tempPos = primaryParts.GetHeadPosition();
	while(tempPos) {
		CPart *part = primaryParts.GetNext( tempPos );
		POSITION newPos = tempPos;
		bool bFoundDup = false;
		while( newPos && !bFoundDup )
		{
			POSITION oldPos = newPos;
			CPart *dup = primaryParts.GetNext( newPos );
			if( part->role->name.CompareNoCase( dup->role->name ) == 0 )
			{
				primaryParts.RemoveAt( oldPos );
				bFoundDup = true;
			}
		}
		if( bFoundDup )
		{
			tempPos = primaryParts.GetHeadPosition( );
		}
	}
*/
	POSITION pos = allParts.GetHeadPosition();
	while(pos) {
		CPart *part = allParts.GetNext(pos);

		// if(part->role->parent == owner)
		if(owner->HasRole(part->role))
			part->DumpXML(xml,depth);
	}

	depth--;
	Indent();
	xml << "</aspect>" << endl;
}

///////////////////////////// CPointerItem ///////////////////////////////

CPointerItem *CPointerItem::FindEqualPointerItem(CPointerItemList &list,CRole *mainRole,CRole *role)
{
	POSITION pos = list.GetHeadPosition();
	while(pos) {
		CPointerItem *item = list.GetNext(pos);
		if((mainRole && item->object->name == mainRole->name && item->port && item->port->name == role->name) ||
			(mainRole == 0 && item->object->name == role->name && item->port == 0))
			return item;
	}
	return 0;
}

void CPointerItem::DumpXML(ofstream xml,int depth)
{
	Indent();
	xml << "<pointeritem desc = \"";
	xml << object->name;
	if(port)
		xml << " " << port->name;
	xml << "\"></pointeritem>" << endl;
}


